/* 	File: <confg2.h>					*/
/*								*/
/*	SAIL RUNTIME SECONDARY Configuration package for 	*/
/*	<sairun.c> the runtime package for the PSAIL compiler.	*/
/*								*/
/*								*/
/*	P. Lemkin						*/
/*	Image Processing Section				*/
/*	National Cancer Institute, DCBD, LTB			*/
/*	FCRF, Build 469 Room 150B				*/
/*	Frederick, MD 21701					*/
/*								*/
/*	Version: 57.10						*/
/*                                                              */
/*	Revised: October 9, 1987		        	*/
/*	Revised: September 7, 1987		        	*/
/*	Revised: June 30, 1987 			        	*/
/*	Revised: June 26, 1987 			        	*/
/*	Revised: March 26, 1987 				*/
/*	Revised: March 15, 1987 				*/
/*	Revised: February 28, 1987 				*/
/*	Revised: February 27, 1987 				*/
/*	Revised: February 18, 1987 				*/
/*	Revised: February 14, 1987 				*/
/*	Revised: September 20, 1986 				*/
/*	Revised: September 14, 1986 				*/
/*	Revised: August 11, 1986 				*/
/*	Revised: April 22, 1986 				*/
/*	Revised: March 17, 1986 				*/
/*	Revised: March 16, 1986 				*/
/*	Revised: March 8, 1986 					*/
/*	Revised: March 5, 1986 					*/
/*	Revised: February 28, 1986 				*/
/*	Revised: January 18, 1986				*/
/*	May 6, 1984						*/
/*								*/
/*
*    COPYRIGHT NOTICE - copyright (c) 1983-1986 by Peter F. Lemkin.
*                     All rights reserved.
*       Permission is granted to any individual or institution to
*       copy, use or modify this program, except for explicitly
*       commercial purposes, provided this copyright notice is
*       included in its entirety in all derived works.  No warranty
*       of the software or of the accuracy of any documentation
*       surrounding it is expressed or implied, and the author does
*       not acknowledge any liability resulting from program or
*       documentation errors.
*/
/*								*/
/* ***********************************************************
	In order to implement the <sairun.c> compatibility package, a
minimum subset of C/UNIX system calls are required. These are:

                int open(STRING fileName, int rwMode)
    or          FILE fopen(STRING fileName, STRING fMode)

(opt.)          int creat(STRING fileName, int rwMode)

                close(int fd)
    or          fclose(FILE fd)

(opt.)          lseek(int fd, INTEGER byteCnt, int mode)
    or          seek(int fd, int byteCnt, int mode)
    or          fseek(FILE fd, int byteCnt, int ptrName)

                read(int fd, char *bufAdr, int byteCnt)
    or          read(int fd, int *bufAdr, int wordCnt)
    or          fread(void *bufAdr, size_t size, int nelem, FILE *fd)

                write(int fd, char *bufAdr, int byteCnt)
    or          write(int fd, int *bufAdr, int wordCnt)
    or          fwrite(void *bufAdr, size_t size, int nelem, FILE *fd)

(opt.)          int eof(int fd)
    or          int feof(FILE fd)

                _exit()
    or          exit(int status)

(opt.)          char *sbrk(int nBytes)
    or          char *malloc(int nBytes)
    or          char *calloc(int nBytes)
    or          void *malloc(size_t size)

                char getchar(void)
    or          char getc(void)
    or          char getc(int fd)
    or          int getc(int fd)
    or          char fgetc(FILE fd)

                putchar(int c)
    or          putc(int c)
    or          putc(int c, int fd)
    or          fputc(int c, FILE fd)

(opt.)		memcpy(void *src, void *dest, size_t nbytes)

where opt. indicates that it would be nice but not necessary.

The intent is to permit one to configure the <sairun.c> package using
whatever exists on the host system (at <sairun.c> compile time).
Then emulate the SAIL I/O functions with these.

N.B. You MUST have the C compiler conditional operators

	#ifdef, #ifndef, and #endif 

available to easily use <config.h> with <sairun.c> and the other
runtime packages. Otherwise you will have a MAJOR editing job ahead
of you. Note that the PSAIL output also contains some conditional
code.
NOTE: the entries in <confg2.h> must match those in <config.h>.
NOTE: all #define and #ifdef control lines MUST start in COLUMN 1
since some 'wimpy' C compilers can not handle the free form!
*************************************************************
*/


/*	Configuring <sairun.c> for your C/UNIX environment
	--------------------------------------------------
*/

#define HAS2CONFIG 1	/* stops multiple confg2.h includes 	*/

/* 	Each of <config.h> SYSTEM environments in turn defines, in 
* <confg2.h>, a set of additional compile time switches which permits
* the use of particular runtime C/UNIX system calls in <sairun.c>. 
* For example, some versions of read()/write() are word, not byte,
* oriented since the underlying hardware may be word oriented (eg. 
* DEC10 in the case of SARGASSO and MIDIC). The seek()/lseek() may be 
* block oriented rather than byte oriented (again in the case of the 
* DEC10). The <sairun.c> then has to sort this mess out using these 
* compile time switches.
*
* In addition, a 32-bit (or longer) integer data type must exist for
* many <sairun.c> functions. This will be either a type int or long on
* many machines. To avoid messy code, a new data type INTEGER is
* typedef'ed in <config.h> and used wherever a 32-bit integer is
* needed.
*
* Some functions, such as catlist() and PSprintf() have a variable # of
* arguments. Some machines have a increasing arg stack while others
* have a decreasing stack. So the ...VARG macros are also defined here.
*
* The remainder of the <config.h> file looks like this:
*	The possible switches are given here alphabetically:
* NOTE: If HASANSIC is set, then most of the other switches are NOT set!.
*
.LABEL [<CONFG2-H-SECONDARY-SWITCHES>]
      HASACORE - implements acore() for SARGASSO
      HASANSIC - implement ANSI X3J11 standard C
      HASSBRK - implements sbrk() core allocation
      HASCALLOC - implements calloc()/cfree() core allocation
      HASCFREE - implements cfree()
      HASCIO - use CIO not STDIO for SARGASSO
      HASCLOSE - implements close()
      HASCREAT - implements creat()
      HASEOF - implements eof()
      HASEXIT - implements exit()
      HASFCLOSE - implements fclose()
      HASFERROR - implements ferror() 
      HASFEOF - implements feof()
      HASFILES - implements FILE typedef for file i/o
      HASFOPEN - implements fopen()
      HASFREE - implements free()
      HASFREAD - implements fread()
      HASFSEEKBYTE - implements fseek() for a BYTE count
      HASFWRITE - implements fwrite()
      HASIOBYTE - implements read()/write() for a BYTE count
      HASIOWORD - implements read()/write() for a WORD count
      HASLOGICALSHIFT - Processor has 0-shifted LOGICALSHIFT 
      HASLSEEKBLOCK - implements lseek() for a BLOCK count
      HASLSEEKBYTE - implements lseek() for a BYTE count
      HASMALLOC - implements malloc()/free() core allocation
      HASOPEN - implements open()
      HASREAD - implements read()
      HASRENAME- implements rename()
      HASSEEKBYTE - implements seek() for a BYTE count
      HASSEEKBLOCK - implements seek() for a BLOCK count
      HASWRITE - implements write()
      HAS_EXIT - implements _exit()
      -----------------------------------------------------
      HASCLIPPER - Processor is CLIPPER 32-bits 
      HASCONNECTIONMACHINE - Processor is CONNECTION MACHINE 
      HASCONVEX - Processor is CONVEX 64-bits 
      HASCRAY - Processor is CRAY-XMP or CRAY-2 64-bits 
      HASDEC10 - Processor is DEC10 36-bits 
      HASIBM370 - Processor uses IBM 32-bit floating point 
      HASMIPS - Processor is MIPS 32-bits 
      HASVAX - Processor is VAX 32-bits 
      HAS32016 - Processor is 32016 32-bits 
      HAS32032 - Processor is 32032 32-bits 
      HAS68000 - Processor is 68000 32-bits 
      HAS68010 - Processor is 68010 32-bits 
      HAS68020 - Processor is 68020 32-bits 
      HAS80286 - Processor is 80286 16/32-bits 
      HAS80386 - Processor is 80386 16/32-bits 
      -----------------------------------------------------
      HASIEEEFLOAT - Processor uses IEEE 32-bit float-point 
      HASMOTOROLAFP - Processor uses MOTOROLA 32-bit F.P. 
      HASSKYBOARDFP - Processor uses SKYBOARD 32-bit F.P. 
      -----------------------------------------------------
      HASPMATH - use the target C system's <math.h> package
      			rather than <psrun9.c>.
      HASASM - can use in-line code
      HASMEMCPY - has memcpy() library function
      HASMVARGS - has variable args in decreasing memory
      HASPVARGS - has variable args in increasing memory
.ENDLABEL
*/

		/* The following lets us test <confg2.h> by itself. */
#ifndef HASCONFIG
#include "config.h"
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASCLIPPER
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASREAD 1
#define HASWRITE 1
#define HASEOF 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASPVARGS 1
typedef unsigned int UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASCONNECTIONMACHINE
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASREAD 1
#define HASWRITE 1
#define HASEOF 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASPVARGS 1
typedef unsigned int UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASCONVEX
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASREAD 1
#define HASWRITE 1
#define HASEOF 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASPVARGS 1
/*	NOTE: char = 8-bits, int = 32 bits, short = 32 bits,      */
/*	      long = 64-bits, f.p.= S,15-bits -16384,48-bit mantissa */
typedef unsigned int UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASCRAY
#define COS 1	/* Or UNICOS - not both */
/*#define UNICOS 1 */	/* Or COS - not both */
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASREAD 1
#define HASWRITE 1
#define HASEOF 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASPVARGS 1
/*	NOTE: char = 8-bits, int = 32 bits, short = 32 bits,      */
/*	      long = 64-bits, f.p.= S,15-bits -16384,48-bit mantissa */
typedef unsigned int UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASEUNICE
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASREAD 1
#define HASWRITE 1
#define HASEOF 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASPVARGS 1
#define HASVAX 1
typedef unsigned int UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASLATTICEC
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASREAD 1
#define HASWRITE 1
#define HASEOF 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASPVARGS 1
typedef unsigned long UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASMIPS
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASREAD 1
#define HASWRITE 1
#define HASEOF 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASPVARGS 1
typedef unsigned int UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASMIDIC
#define HASTOPS10 1		/*This or TOPS20 */
/* #define HASTOPS20 1 */	/* This or TOPS10 */
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOWORD 1
#define HASSEEKBLOCK 1
#define HASREAD 1
#define HASWRITE 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASMVARGS 1
#define HASASM 1
#define HASLOGICALSHIFT 1
#define HASDEC10 1
typedef unsigned int UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASPORTABLEC
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASREAD 1
#define HASWRITE 1
#define HASEOF 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASPVARGS 1
typedef unsigned long UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASSARGASSOC
#define HASTOPS10 1		/*This or TOPS20 */
/* #define HASTOPS20 1 */	/* This or TOPS10 */
#define HASIOWORD 1
#define HASSEEKBLOCK 1
#define HASEOF 1
#define HAS_EXIT 1
#define HASACORE 1  	    /* like sbrk but words/SARGASSO */
			    /* use TOPS10 OPEN(), CLOSE, */
			    /* READ(), WRITE(), RENAME, SEEK(), */
			    /* GETC(), PUTC(). If <stdio> has all */
			    /* functions including fread()/fwrite()*/
			    /* then can use <stdio.h>. */
#define HASMVARGS 1
#define HASLOGICALSHIFT 1
/* NOTE: you MUST use CIO not STDIO since STDIO does not have seek() */
#define HASCIO 1    /* specific switch/SARGASSO */
	extern INTEGER *alcore(); /* because of CLUDGE in SARGASSO */
#define HASDEC10 1
typedef unsigned int UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASSUN3
#define HASFOPEN 1
#define HASFILES 1
#define HASFCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASFREAD 1
#define HASFWRITE 1
#define HASFEOF 1
#define HASFERROR 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASCALLOC 1
#define HASCFREE 1
#define HASPVARGS 1
typedef unsigned int UINTEGER;		/* unsigned 32-bit integer */
#endif
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASSYSTEMIII
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASREAD 1
#define HASWRITE 1
#define HASEOF 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASPVARGS 1
typedef unsigned int UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASSYSTEMV
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASREAD 1
#define HASWRITE 1
#define HASEOF 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASPVARGS 1
typedef unsigned int UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASULTRIX32
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASREAD 1
#define HASWRITE 1
#define HASEOF 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASPVARGS 1
#define HASVAX 1
typedef unsigned int UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASUNITY
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASREAD 1
#define HASWRITE 1
#define HASEOF 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASPVARGS 1
#define HASVAX 1
typedef unsigned int UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASUNIX7
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASREAD 1
#define HASWRITE 1
#define HASEOF 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASPVARGS 1
typedef unsigned long UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASUTS
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASREAD 1
#define HASWRITE 1
#define HASEOF 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASPVARGS 1
#define HASIBM370
typedef unsigned int UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASVMSC
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASREAD 1
#define HASWRITE 1
#define HASEOF 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASPVARGS 1
#define HASVAX 1
typedef unsigned int UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASWHITESMITH
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASREAD 1
#define HASWRITE 1
#define HASEOF 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASPVARGS 1
typedef unsigned long UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASXENIX
#define HASOPEN 1
#define HASCREAT 1
#define HASCLOSE 1
#define HASIOBYTE 1
#define HASLSEEKBYTE 1
#define HASREAD 1
#define HASWRITE 1
#define HASEOF 1
#define HASRENAME 1
#define HAS_EXIT 1
#define HASSBRK 1
#define HASPVARGS 1
#define HAS80286 1
typedef unsigned long UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASX3J11
#define HASANSIC 1
#define HASFILES 1
typedef unsigned long UINTEGER;		/* unsigned 32-bit integer */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* ---------------------------------------------------------------- */
/*     D E F I N E       N U M B E R    S I Z E S                   */
/* ---------------------------------------------------------------- */
/* If your machine was not defined using the PRIMARY macros, then you
* must (1) define a F.P. processor type (eg HASIEEEFLOAT etc) or (2)
* add new processor #ifdef definition and then do (1).
* @V.56.35  - check all these numbers!!!!
*/

#ifdef HASDEC10
#define posINF 1.7014117E+38	/* largest PDP10 single prec F.P.# */
#define negINF -1.7014117E+38	/* largest PDP10 single prec F.P.# */
#define posINT	0377777777777   /* Largest + PDP10 INTEGER */
#define negINT	0400000000000   /* Largest - PDP10 INTEGER */
#define maxBITS 36		/* # bits/INTEGER */
#endif

#ifdef HASIEEEFLOAT  /* IEEE P754 std used by 80287 & 68881 & 32081 */
		     /* floating point chips */
#define posINF 1.7000000E+38	/* largest 32-bit single prec F.P.# */
#define negINF -1.700000E+38	/* largest 32-bit single prec F.P.# */
#define posINT 	017777777777    /* Largest + 32-bit integer */
				/* i.e. 2,147,483,848		*/
#define negINT 	020000000000    /* Largest - 32-bit integer */
#define maxBITS 32		 /* # bits/INTEGER */
#endif

#ifdef HASVAX
#define posINF 1.7000000E+38	/* largest 32-bit single prec F.P.# */
#define negINF 0.2900000E+38	/* largest 32-bit single prec F.P.# */
#define posINT 	017777777777    /* Largest + 32-bit integer */
#define negINT 	020000000000    /* Largest - 32-bit integer */
#define maxBITS 32		 /* # bits/INTEGER */
#endif

#ifdef HASIBM370
#define posINF 1.7000000E+38	/* largest 32-bit single prec F.P.# */
#define negINF 0.2900000E+38	/* largest 32-bit single prec F.P.# */
#define posINT 	017777777777    /* Largest + 32-bit integer */
#define negINT 	020000000000    /* Largest - 32-bit integer */
#define maxBITS 32		 /* # bits/INTEGER */
#endif

#ifdef HASMOTOROLAFP		/* old standard */
#define posINF 1.7000000E+38	/* largest 32-bit single prec F.P.# */
#define negINF 0.2900000E+38	/* largest 32-bit single prec F.P.# */
#define posINT 	017777777777    /* Largest + 32-bit integer */
#define negINT 	020000000000    /* Largest - 32-bit integer */
#define maxBITS 32		 /* # bits/INTEGER */
#endif

#ifdef HASCRAY
#define posINF 1.00000E+2466	/* largest 64-bit single prec F.P.# */
#define negINF -1.00000E+2466	/* largest 64-bit single prec F.P.# */
#define posINT 	0377777777777777777777  /* Largest + 64-bit integer */
#define negINT 	0400000000000000000000  /* Largest - 64-bit integer */
#define maxBITS 64		 /* # bits/INTEGER */
#endif

#ifndef posINF
	/* Then default to HASIEEEFLOAT */
	/* IEEE P754 std used by 80287 & 68881 & 32081 FP chips*/
#define posINF 1.7000000E+38	/* largest 32-bit single prec F.P.# */
#define negINF -1.700000E+38	/* largest 32-bit single prec F.P.# */
#define posINT 	017777777777    /* Largest + 32-bit integer */
				/* i.e. 2,147,483,848		*/
#define negINT 	020000000000    /* Largest - 32-bit integer */
#define maxBITS 32		 /* # bits/INTEGER */
#endif
/* -------------------------------------------------------------- */
/*	C O N S I S T E N C Y   C H E C K			  */
/* -------------------------------------------------------------- */
/*	NOTE: these macros are used to define a special macro     */
/*		PSCFG_CONSISTENCY				  */
/*	which  should evaluate to 0. If not, the Octal   	  */
/*	Bit indicates which set of facilities is inconsistent -   */
/*	either none is defined or more than one. 		  */
/*		 						  */
/*	[1] Define bits according to what was defined		  */
/*
*	--> OPERATING SYSTEM NAMES <---
*/
#ifndef HASBSD
#define H_BSD 0
#else
#define H_BSD 1
#endif

#ifndef HASCOS
#define H_COS 1
#else
#define H_COS 0
#endif

#ifndef HASEUNICE
#define H_EUNICE 0
#else
#define H_EUNICE 1
#endif

#ifndef HASLATTICEC
#define H_LATTICEC 0
#else
#define H_LATTICEC 1
#endif

#ifndef HASPORTABLEC
#define H_PORTABLEC 0
#else
#define H_PORTABLEC 1
#endif

#ifndef HASSYSTEMIII
#define H_SYSTEMIII 0
#else
#define H_SYSTEMIII 1
#endif

#ifndef HASSYSTEMV
#define H_SYSTEMV 0
#else
#define H_SYSTEMV 1
#endif

#ifndef HASTOPS10
#define H_TOPS10 0
#else
#define H_TOPS10 1
#endif

#ifndef HASTOPS20
#define H_TOPS20 0
#else
#define H_TOPS20 1
#endif

#ifndef HASUNIX7
#define H_UNIX7 0
#else
#define H_UNIX7 1
#endif

#ifndef HASULTRIX32
#define H_ULTRIX32 0
#else
#define H_ULTRIX32 1
#endif

#ifndef HASUNICOS
#define H_UNICOS 0
#else
#define H_UNICOS 1
#endif

#ifndef HASUTS
#define H_UTS 0
#else
#define H_UTS 1
#endif

#ifndef HASVMSC
#define H_VMSC 0
#else
#define H_VMSC 1
#endif

#ifndef HASWHITESMITH
#define H_WHITESMITH 0
#else
#define H_WHITESMITH 1
#endif

#ifndef HASXENIX
#define H_XENIX 0
#else
#define H_XENIX 1
#endif

#ifndef HASX3J11
#define H_X3J11 0
#else
#define H_X3J11 1
#endif

/*	  - - end of OS definition - - */
/*
*	--> C LIBRARY SYSTEM CALLS <---
*/
#ifndef HASACORE
#define H_ACORE 0
#else
#define H_ACORE 1
#endif

#ifndef HASANSIC
#define H_ANSIC 0
#else
#define H_ANSIC 1
#endif

#ifndef HASCALLOC
#define H_CALLOC 0
#else
#define H_CALLOC 1
#endif

#ifndef HASCFREE
#define H_CFREE 0
#else
#define H_CFREE 1
#endif

#ifndef HASCIO
#define H_CIO 0
#else
#define H_CIO 1
#endif

#ifndef HASCLOSE
#define H_CLOSE 0
#else
#define H_CLOSE 1
#endif

#ifndef HASCREAT
#define H_CREAT 0
#else
#define H_CREAT 1
#endif

#ifndef HASEOF
#define H_EOF 0
#else
#define H_EOF 1
#endif

#ifndef HASERROR
#define H_ERROR 0
#else
#define H_ERROR 1
#endif

#ifndef HASEXIT
#define H_EXIT 0
#else
#define H_EXIT 1
#endif

#ifndef HASFCLOSE
#define H_FCLOSE 0
#else
#define H_FCLOSE 1
#endif

#ifndef HASFEOF
#define H_FEOF 0
#else
#define H_FEOF 1
#endif

#ifndef HASFERROR
#define H_FERROR 0
#else
#define H_FERROR 1
#endif

#ifndef HASFILES
#define H_FILES 0
#else
#define H_FILES 1
#endif

#ifndef HASFOPEN
#define H_FOPEN 0
#else
#define H_FOPEN 1
#endif

#ifndef HASFREAD
#define H_FREAD 0
#else
#define H_FREAD 1
#endif

#ifndef HASFREE
#define H_FREE 0
#else
#define H_FREE 1
#endif

#ifndef HASFSEEKBYTE
#define H_FSEEKBYTE 0
#else
#define H_FSEEKBYTE 1
#endif

#ifndef HASFWRITE
#define H_FWRITE 0
#else
#define H_FWRITE 1
#endif

#ifndef HASIOBYTE
#define H_IOBYTE 0
#else
#define H_IOBYTE 1
#endif

#ifndef HASIOWORD
#define H_IOWORD 0
#else
#define H_IOWORD 1
#endif

#ifndef HASLSEEKBLOCK
#define H_LSEEKBLOCK 0
#else
#define H_LSEEKBLOCK 1
#endif

#ifndef HASLSEEKBYTE
#define H_LSEEKBYTE 0
#else
#define H_LSEEKBYTE 1
#endif

#ifndef HASSEEKBLOCK
#define H_SEEKBLOCK 0
#else
#define H_SEEKBLOCK 1
#endif

#ifndef HASMALLOC
#define H_MALLOC 0
#else
#define H_MALLOC 1
#endif

#ifndef HASOPEN
#define H_OPEN 0
#else
#define H_OPEN 1
#endif

#ifndef HASRENAME
#define H_RENAME 0
#else
#define H_RENAME 1
#endif

#ifndef HASSBRK
#define H_SBRK 0
#else
#define H_SBRK 1
#endif

#ifndef HASREAD
#define H_READ 0
#else
#define H_READ 1
#endif

#ifndef HASSEEKBYTE
#define H_SEEKBYTE 0
#else
#define H_SEEKBYTE 1
#endif

#ifndef HASWRITE
#define H_WRITE 0
#else
#define H_WRITE 1
#endif

#ifndef HAS_EXIT
#define H__EXIT 0
#else
#define H__EXIT 1
#endif

/*
*	--> UNDERLYING HARDWARD CAPABILITIES <---
*/
#ifndef HASLOGICALSHIFT
#define H_LOGICALSHIFT 0
#else
#define H_LOGICALSHIFT 1
#endif

#ifndef HASDEC10
#define H_DEC10 0
#else
#define H_DEC10 1
#endif

#ifndef HASVAX
#define H_VAX 0
#else
#define H_VAX 1
#endif

#ifndef HASCRAY
#define H_CRAY 0
#else
#define H_CRAY 1
#endif

#ifndef HASCONVEX
#define H_CONVEX 0
#else
#define H_CONVEX 1
#endif

#ifndef HASCONNECTIONMACHINE
#define H_CONNECTIONMACHINE 0
#else
#define H_CONNECTIONMACHINE 1
#endif

#ifndef HASMIPS
#define H_MIPS 0
#else
#define H_MIPS 1
#endif

#ifndef HASCLIPPER
#define H_CLIPPER 0
#else
#define H_CLIPPER 1
#endif

#ifndef HAS68000
#define H_68000 0
#else
#define H_68000 1
#endif

#ifndef HAS68010
#define H_68010 0
#else
#define H_68010 1
#endif

#ifndef HAS68020
#define H_68020 0
#else
#define H_68020 1
#endif

#ifndef HAS32016
#define H_32016 0
#else
#define H_32016 1
#endif

#ifndef HAS32032
#define H_32032 0
#else
#define H_32032 1
#endif

#ifndef HAS80286
#define H_80286 0
#else
#define H_80286 1
#endif

#ifndef HAS80386
#define H_80386 0
#else
#define H_80386 1
#endif

/*
*	--> UNDERLYING HARDWARE FLOATING POINT <---
*/
#ifndef HASIBM370
#define H_IBM370 0
#else
#define H_IBM370 1
#endif

#ifndef HASIEEEFLOAT
#define H_IEEEFLOAT 0
#else
#define H_IEEEFLOAT 1
#endif

#ifndef HASMOTOROLAFP
#define H_MOTOROLAFP 0
#else
#define H_MOTOROLAFP 1
#endif

#ifndef HASSKYBOARDFP
#define H_SKYBOARDFP 0
#else
#define H_SKYBOARDFP 1
#endif
/*
*	--> UNDERLYING HARDWARE/SYSTEM MISC. STUFF <---
*/
#ifndef HASASM
#define H_ASM 0
#else
#define H_ASM 1
#endif

#ifndef HASMEMCPY
#define H_MEMCPY 0
#else
#define H_MEMCPY 1
#endif

#ifndef HASMVARGS
#define H_MVARGS 0
#else
#define H_MVARGS 1
#endif

#ifndef HASPVARGS
#define H_PVARGS 0
#else
#define H_PVARGS 1
#endif

/*								*/
/*
*	[2] Define consistency check bits
*/
/*								*/

/* NOTE: if you add a bit here then also add it to the
* PSCFG_CONSISTENCY test macro, the h_chk_congf2_consistency()
* function and to the #undef list at the end.
*/
#define H_BAD_OS	01		/* BAD OPERATING SYSTEM */
#define H_BAD_OPEN	02		/* BAD OPEN CALL */
#define H_BAD_CLOSE	04		/* BAD CLOSE CALL */
#define H_BAD_EOF	010		/* BAD EOF CALL */
#define H_BAD_READ	020		/* BAD READ CALL */
#define H_BAD_WRITE	040		/* BAD WRITE CALL */
#define H_BAD_IOSIZE	0100		/* BAD IO SIZE */
#define H_BAD_SEEK	0200		/* BAD SEEK CALL */
#define H_BAD_EXIT	0400		/* BAD EXIT CALL */
#define H_BAD_ALLOC	01000		/* BAD ALLOC CALL */
#define H_BAD_CPU	02000		/* BAD CPU SPEC */
#define H_BAD_FP	04000		/* BAD FP SPEC */
#define H_BAD_VARARGS	010000		/* BAD VARARGS SPEC */

#define PSCFG_CONSISTENCY (\
  (((H_TOPS10 + H_TOPS20 + H_UNIX7 + H_BSD + H_EUNICE + \
	H_ULTRIX32 + H_SYSTEMIII + H_SYSTEMV + H_UNICOS + H_COS + \
	H_VMSC + H_XENIX + H_PORTABLEC + H_WHITESMITH + \
	H_LATTICEC + H_UTS + H_X3J11)!=1) ? 0 : H_BAD_OS) \
  (((H_ANSIC + H_OPEN + H_FOPEN + H_CIO)!=1) ? 0 : H_BAD_OPEN) \
  (((H_ANSIC + H_CLOSE + H_FCLOSE + H_CIO)!=1) ? 0 : H_BAD_CLOSE) \
  (((H_ANSIC + H_EOF + H_FEOF + H_CIO)!=1) ? 0 : H_BAD_EOF) \
  (((H_ANSIC + H_READ + H_FREAD + H_CIO)!=1) ? 0 : H_BAD_READ) \
  (((H_ANSIC + H_WRITE + H_FWRITE + H_CIO)!=1) ? 0 : H_BAD_WRITE) \
  (((H_ANSIC + H_IOWORD + H_IOBYTE + H_CIO)!=1) ? 0 : H_BAD_IOSIZE) \
  (((H_ANSIC + H_SEEKWORD + H_SEEKBYTE + H_LSEEKWORD + H_LSEEKBYTE)!=1)\
					? 0 : H_BAD_SEEK) \
  (((H_ANSIC + H__EXIT + H_EXIT)!=1) ? 0 : H_BAD_EXIT) \
  (((H_ANSIC + H_SBRK + H_MALLOC + H_CALLOC + H_ACORE + H_CIO)!=1)\
				? 0 : H_BAD_ALLOC) \
  (((H_DEC10 + H_VAX + H_IBM370 + \
		H_CRAY + H_CONVEX + H_CONNECTIONMACHINE + H_MIPS + \
		H_68000 + H_68010 + H_68020 + \
		H_32016 + H_32032 + \
		H_80286 + H_80386)!=1) ? 0 : H_BAD_CPU) \
  (((H_DEC10 + H_VAX + H_IBM370 + \
		H_CRAY + H_CONVEX + \
		H_IEEEFLOAT + H_MOTOROLAFP + H_SKYBOARDFP)!=1)\
				? 0 : H_BAD_FP) \
  (((H_MVARGS + H_PVARGS)!=1) ? 0 : H_BAD_VARARGS) \
	)


#ifdef HAS_CHK_CONFIG2_CONSISTENCY

/*	Build procedure to check the <confg2.h> consistency and print
* out a message of where it is screwed up if at all.
*/
void PROCEDURE h_chk_config2()
{
INTEGER bits, i;
STRING s;

bits = PSCFG_CONSISTENCY;	/* expand macro */
if (bits==0)
	printf(
	   "TERRIFIC! Your <confg2.h> file is correctly configured.\n")
else 	{
	printf(
	  "TROUBLE! Your <confg2.h> file is incorrectly configured:\n")
	for(i=1; i!=0; i<<1)
	    { /* TEST BIT i */
	    s = "\0";		/* set to no error */
	    switch (bits & i)
		{
		case H_BAD_OS:		s = "BAD OPERATING SYSTEM";
		case H_BAD_OPEN:	s = "BAD OPEN CALL";
		case H_BAD_CLOSE:	s = "BAD CLOSE CALL";
		case H_BAD_EOF:		s = "BAD EOF CALL";
		case H_BAD_READ:	s = "BAD READ CALL";
		case H_BAD_WRITE:	s = "BAD WRITE CALL";
		case H_BAD_IOSIZE:	s = "BAD IO SIZE";
		case H_BAD_SEEK:	s = "BAD SEEK CALL";
		case H_BAD_EXIT:	s = "BAD EXIT CALL";
		case H_BAD_ALLOC:	s = "BAD ALLOC CALL";
		case H_BAD_CPU:		s = "BAD CPU SPEC";
		case H_BAD_FP:		s = "BAD FP SPEC";
		case H_BAD_VARARGS:	s = "BAD VARARGS SPEC";
		}
	    if (s != "\0") 
			printf("%s .\n",s);
	    } /* TEST BIT i */
}
#endif /* HAS_CHK_CONFIG_CONSISTENCY */

#undef H_BAD_OS
#undef H_BAD_OPEN
#undef H_BAD_CLOSE
#undef H_BAD_EOF
#undef H_BAD_READ
#undef H_BAD_WRITE
#undef H_BAD_IOSIZE
#undef H_BAD_SEEK
#undef H_BAD_EXIT
#undef H_BAD_ALLOC
#undef H_BAD_CPU
#undef H_BAD_FP
#undef H_BAD_VARARGS
/*		 						*/
/*	[3] Remove the temporary macros.			*/
#undef H_ACORE 
#undef H_ANSIC
#undef H_CALLOC 
#undef H_CFREE
#undef H_CIO 
#undef H_CLOSE 
#undef H_CREAT 
#undef H_EOF
#undef H_ERROR
#undef H_EXIT 
#undef H_FCLOSE 
#undef H_FERROR
#undef H_FEOF 
#undef H_FILES 
#undef H_FOPEN
#undef H_FREAD 
#undef H_FSEEKBYTE 
#undef H_FWRITE 
#undef H_LOGICALSHIFT 
#undef H_LSEEKBLOCK 
#undef H_LSEEKBYTE 
#undef H_MALLOC 
#undef H_IOBYTE 
#undef H_IOWORD 
#undef H_OPEN
#undef H_READ 
#undef H_RENAME
#undef H_SBRK 
#undef H_SEEKBLOCK 
#undef H_SEEKBYTE 
#undef H_WRITE 
#undef H__EXIT 

#undef H_CLIPPER
#undef H_CONVEX
#undef H_CONNECTIONMACHINE
#undef H_CRAY 
#undef H_IBM370 
#undef H_DEC10 
#undef H_MIPS 
#undef H_VAX 
#undef H_32016 
#undef H_32032 
#undef H_68000 
#undef H_68010 
#undef H_68020 
#undef H_80286 
#undef H_80386 

#undef H_IEEEFLOATFP
#undef H_MOTOROLAFP

#undef H_ASM 
#undef H_MEMCPY 
#undef H_MVARGS 
#undef H_PVARGS 

/*    ---> END OF <confg2.h> <--- * */
