/* 	File: <config.h>					*/
/*								*/
/*	SAIL RUNTIME Configuration package for <sairun.c>	*/
/*	the runtime package for the PSAIL compiler.		*/
/*								*/
/*								*/
/*        P. Lemkin                                             */
/*        Image Processing Section                              */
/*        National Cancer Institute, DCBD, LTB                  */
/*        FCRF, Build 469 Room 150B			   	*/
/*        Frederick, MD 21701				   	*/
/*								*/
/*	Version: 57.10						*/
/*                                                              */
/*      Revised: October 9, 1987                                */
/*      Revised: September 7, 1987                              */
/*      Revised: July 23, 1987                                  */
/*	Revised: March 26, 1987 				*/
/*	Revised: March 2, 1987 					*/
/*	Revised: September 20, 1986 				*/
/*	Revised: September 19, 1986 				*/
/*	Revised: August 14, 1986 				*/
/*	Revised: August 11, 1986 				*/
/*	Revised: August 7, 1986	 				*/
/*	Revised: March 5, 1986	 				*/
/*	Revised: February 28, 1986 				*/
/*	Revised: January 18, 1986 - split into confg2.h		*/
/*	Revised: January 9, 1986				*/
/*	Revised: December 18, 1985				*/
/*	Revised: December 13, 1985				*/
/*	Revised: December 12, 1985				*/
/*	Revised: December 6, 1985				*/
/*	Revised: November 22, 1985				*/
/*	Revised: November 21, 1985-B.S.				*/
/*	Revised: November 21, 1985				*/
/*	Revised: November 5, 1985				*/
/*	Revised: October 31, 1985				*/
/*	Revised: September 7, 1985				*/
/*	Revised: August 6, 1985					*/
/*	Revised: June 29, 1985					*/
/*	Revised: June 7, 1985					*/
/*	Revised: February 17, 1985				*/
/*	Revised: Nov 19, 1984					*/
/*	May 6, 1984						*/
/*								*/
/*
*    COPYRIGHT NOTICE - Copyright (c) 1983-1987 by Peter F. Lemkin.
*                     All rights reserved.
*       Permission is granted to any individual or institution to
*       copy, use or modify this program, except for explicitly
*       commercial purposes, provided this copyright notice is
*       included in its entirety in all derived works.  No warranty
*       of the software or of the accuracy of any documentation
*       surrounding it is expressed or implied, and the author does
*       not acknowledge any liability resulting from program or
*       documentation errors.
*/
/*								*/
/* ***********************************************************
	In order to implement the <sairun.c> compatibility package, a
minimum subset of C/UNIX system calls are required. These are
selected using Secondary macros in the <confg2.h> package which is
included ONLY where required (i.e. in <sairun.h>).

The <confg2.h> is ONLY #included when compiling the PSAIL runtimes -
never include them when compiling PSAIL generated (i.e. user) code.

N.B. You MUST have the C compiler conditional operators

	#ifdef, #ifndef, and #endif 

available to easily use <config.h> and <confg2.h> with <sairun.c> and
the other runtime packages. Otherwise you will have a MAJOR editing
job ahead of you. Note that the PSAIL output also contains some
conditional code.
NOTE: the entries in <confg2.h> must match those in <config.h>.
NOTE: all #define and #ifdef control lines MUST start in COLUMN 1
since some 'wimpy' C compilers can not handle the free form!
*************************************************************
*/


/*	Configuring <sairun.c> for your C/UNIX environment
	--------------------------------------------------
*/
/* The following macro ermits "file" instead of <file> access. Take this
 * OUT if you want <sys/file> access.
*/
#ifndef HASLOCALFILE
/* #define HASLOCALFILE 1 */
#endif


/* The following flag may be used while debugging salloc() &
 * Pmalloc(). Take this OUT if you want GC.
*/
#ifndef HASNOGC
#define HASNOGC 1
#endif


/* To define the trig package to be used. Take this OUT if you want to
* use YOUR <math.h> package. I.e. use <psrun9.c> if you leave it in.
*/
#ifndef HASPMATH
/* #define HASPMATH 1 */
#endif


#define HASCONFIG 1	/* stops multiple config.h includes 	*/

/*	File <config.h> declares an extensible set of declarations
for specific systems, some of which are listed here. The user should
edit <config.h> to allow ONLY ONE of the following to be defined.
*/

/*
.LABEL [<CONFIG-H-ENVIRONMENTS>]
*/
/* #define HASCLIPPER 1               */
/* #define HASCONNECTIONMACHINE 1     */
/* #define HASCONVEX 1                */
/* #define HASCRAY 1                  */
/* #define HASEUNICE 1                */
/* #define HASLATTICEC 1              */
/* #define HASMIDIC 1                 */ /* DEC10 compiler TOPS10/20 */
/* #define HASMIPS 1                  */
/* #define HASPORTABLEC 1             */
/* #define HASSARGASSOC 1             */ /* DEC10 compiler TOPS10/20 */
#define HASSUN3 1
/* #define HASSUN4 1                  */
/* #define HASSYSTEMIII 1             */
/* #define HASSYSTEMV 1               */
/* #define HASULTRIX32 1              */
/* #define HASUNITY 1                 */
/* #define HASUNIX7 1                 */
/* #define HASUTS 1                   */
/* #define HASVMSC 1                  */
/* #define HASWHITESMITH 1            */
/* #define HASXENIX 1                 */
/*
.ENDLABEL
*/
/* #define HASX3J11 1		*/

/* 	Each of these in turn defines, in <config.h>, a set of
* additional compile time switches which permit the use of particular
* runtime C/UNIX system calls in <sairun.c>. These are further
* selected using the secondary macros defined in this file and in
* <confg2.h>.
*
* In addition, a 32-bit (or longer) integer data type must exist for
* many <sairun.c> functions. This will be either a type int or long on
* many machines. To avoid messy code, a new data type INTEGER is
* typedef'ed in <config.h> and used whenever a 32-bit integer is
* needed.
*
*/

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASCLIPPER
	typedef int INTEGER;	/* size of CV... fcts */
#define HASBSD 1
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASCONNECTIONMACHINE
	typedef int INTEGER;	/* size of CV... fcts */
#define HASBSD 1
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASCONVEX
	typedef int INTEGER;	/* [CHECK??] size of CV... fcts */
/*	[CHK??] NOTE: char=8-bits, int=32 bits, short=32 bits,      */
/*	      long = 64-bits, f.p.= S,15-bits -16384,48-bit mantissa */
#define HASBSD 1
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASCRAY
	typedef int INTEGER;	/* size of CV... fcts */
/*	NOTE: char = 8-bits, int = 32 bits, short = 32 bits,      */
/*	      long = 64-bits, f.p.= S,15-bits -16384,48-bit mantissa */
#define HASBSD 1
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASEUNICE
	typedef int INTEGER;	/* size of CV... fcts */
#define HASBSD 1
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASLATTICEC
typedef long INTEGER;	/* size of CV... fcts */
typedef int void;  /* void isn't keyword in C compiler */
#define HASMSDOS 1
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASMIPS
	typedef int INTEGER;	/* [CHECK??] size of CV... fcts */
#define HASBSD 1
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASPORTABLEC
	typedef long INTEGER;	/* size of CV... fcts */
	typedef int void;  /* void isn't keyword in C compiler */
#define HASSYSTEMV 1
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASSARGASSOC
	typedef int INTEGER;	/* size of CV... fcts */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASSYSTEMIII
	typedef int INTEGER;	/* size of CV... fcts */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASSUN3
	typedef int INTEGER;	/* size of CV... fcts */
#define HASBSD 1
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASSUN4
	typedef int INTEGER;	/* size of CV... fcts */
#define HASBSD 1
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASSYSTEMV
	typedef int INTEGER;	/* size of CV... fcts */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASULTRIX32
	typedef int INTEGER;	/* size of CV... fcts */
#define HASBSD 1
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASUNITY
	typedef int INTEGER;	/* size of CV... fcts */
#define HASBSD 1
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASUNIX7
	typedef long INTEGER;	/* size of CV... fcts */
	typedef int void;  /* void isn't keyword in C compiler */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASVMSC
	typedef int INTEGER;	/* size of CV... fcts */
#define HASBSD 1
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASWHITESMITH
	typedef long INTEGER;	/* size of CV... fcts */
	typedef int void;  /* void isn't keyword in C compiler */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASXENIX
	typedef long INTEGER;	/* size of CV... fcts */
	typedef int void;  /* void isn't keyword in C compiler */
#define HASSYSTEMV
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASUTS
	typedef int INTEGER;	/* size of CV... fcts */
	typedef int void;  /* void isn't keyword in C compiler */
#define HASSYSTEMV 1
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
#ifdef HASX3J11
	typedef long INTEGER;	/* size of CV... fcts */
#endif

/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* --------------------------------------------------------------- */
/*	T Y P E D E F ' S					   */
/* --------------------------------------------------------------- */

/*	The following typedefs are used for present and future	*/
/*	PSAIL data structures.					*/
/*
.LABEL [<CONFIG-H-TYPEDEFS>]
*/
typedef INTEGER  EVENT;         /* for Processes */
typedef INTEGER  FAKEFLOAT;     /* used with with math routines*/
typedef INTEGER ITEM;           /* for LEAP */
typedef INTEGER ITEMVAR;        /* for LEAP */
typedef INTEGER *LISP_SEXPR;    /* <future possible use */
typedef INTEGER *LISP_ATOM;     /* future possible use */
typedef INTEGER LIST;           /* for LEAP */
typedef INTEGER OBJECT;         /* <future possible use> */
typedef INTEGER SET;            /* for LEAP */
typedef char    *STRING;        /* so compatible with C strings! */
/*
.ENDLABEL
*/
/* ---> END OF CONFIG.H <--- */
