/* 	File: PSRUN2.C 						*/
/*								*/
/*	SAIL RUNTIME  PROGRAM CONTROL PROCEDURES		*/
/*								*/
/*								*/
/*	P. Lemkin						*/
/*	Image Processing Section				*/
/*	National Cancer Institute, DCBD, LTB			*/
/*	FCRF, Build 469 Room 150B				*/
/*	Frederick, MD 21701					*/
/*
*    COPYRIGHT NOTICE - Copyright (c) 1983-1987 by Peter F. Lemkin.
*                     All rights reserved.
*       Permission is granted to any individual or institution to
*       copy, use or modify this program, except for explicitly
*       commercial purposes, provided this copyright notice is
*       included in its entirety in all derived works.  No warranty
*       of the software or of the accuracy of any documentation
*       surrounding it is expressed or implied, and the author does
*       not acknowledge any liability resulting from program or
*       documentation errors.
*/
/*								*/
/*	Version: 57.10						*/
/*                                                              */
/*      Revised October 10, 1987                                */
/*      Revised October 9, 1987                                 */
/*      Revised October 4, 1987                                 */
/*      Revised September 7, 1987                               */
/*      Revised July 23, 1987                                   */
/*	Revised June 30, 1987				        */
/*	Revised February 28, 1987				*/
/*	Revised September 26, 1986				*/
/*	Revised August 12, 1986				        */
/*	Revised August 11, 1986				        */
/*	Revised August 7, 1986				        */
/*	Revised August 6, 1986				        */
/*	Revised June 26, 1986				        */
/*	Revised December 13, 1985				*/
/*	Revised December 2, 1985				*/
/*	Revised November 21, 1985				*/
/*	Revised November 5, 1985				*/
/*	Revised October 11, 1985				*/
/*	Revised August 6, 1985					*/
/*	Revised June 7, 1985					*/
/*	Revised March 8, 1985					*/
/*	June 29, 1984						*/
/*								*/
/*	This procedure package includes the following standard 	*/
/*	runtime functions necessary for easy use of the PSAIL	*/
/*	compiler in a UNIX environment.				*/
/*								*/
/*  2. Program control procedures: sub-module <psrun2.c>	*/
/*	PSlongjmp(newsp) - goto address by INTEGER newsp stack ptr.*/
/*	PSsetjmp() - return this stack ptr for later PSlongjump.	*/
/*	callsys(acc,*uuo) - emulate SAIL CALL to DEC10 UUOs	*/
/*								*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    C O M P I L E   T I M E   M A C R O S              */
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/

#ifndef HASCONFIG
#include "config.h"	/* declare HAS... macros */
#endif

#ifndef PNULL
#include "psrdef.h"	/* need <sairun.c> & <psrung.h>  compile-time macros */
#include "psrglb.h"	/* SAIRUN global variables */


/*	The following are found in PSRUN4.C			*/
INTEGER equ();          /* test if two strings are equal */

/*	The following are found in PSRUN7.C			*/
void PSprintf();	/* in PSRUN7.C */

#endif			/* end of code brought in if compiling alone */

/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    G L O B A L    V A R I A B L E S / M A C R O S	*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/


/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    P R O G R A M   C O N T R O L   P R O C.	+	*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	PSlongjmp(newsp) - goto address by INTEGER newsp stack ptr. */
/*								*/
void PROCEDURE PSlongjmp(newsp)
	INTEGER *newsp;	/* new stack pointer to return jump to */
{
#ifdef HASDEC10
/*	MOVE	.SP, .PRI  ;Trash the current stack, restore old PDL */
/*	POPJ	.SP,	;Force 0 arg function return */
#endif

#ifndef HASDEC10
	longjmp(newsp);		 /* use whatever there is */
#endif
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	PSsetjmp() - return this stack ptr for later PSlongjump.*/
/*								*/
INTEGER PROCEDURE PSsetjmp()
{
#ifdef HASDEC10
/*	MOVE	.PRI, .SP  ;Save the return stack pointer as the value */
#endif
	/* note the return value is already in the PDL */
#ifndef HASDEC10
	return(setjmp());	/* use whatever we have */
#endif
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	callsys() - emulate SAIL 'CALL' to DEC10 UUOs		*/
/*	HINT: use UNIX system("..") calls as well as other	*/
/*	calls directly to the library and then string process	*/
/*	to do what we need to do.				*/
/*      NOTE: 10-4-87 - needs lots of work.                     */
/*								*/
INTEGER PROCEDURE callsys(acc,uuo)
	INTEGER acc;
	STRING uuo;
{ /* CALLSYS */
        INTEGER cpuTime=0;            /* in msec */
        INTEGER runTime=0;            /* in msec */
        STRING filename="BOGUS";
        long clock();        /* BSD system call - return CPU time 
			      *  since last call in milliseconds */
        long gettimeofday(); /* BSD system call - return  REAL time
                              * since last call in milliseconds */
        int execl();         /* BSDexecute named process */


	if(equ(GC_PUSHS(uuo),GC_PUSHS("EXIT"))==0)
	  PEXIT;

	else if(equ(GC_PUSHS(uuo),GC_PUSHS("RUNTIM"))==0)
	  { /* RUNTIM */
	   cpuTime = clock();
	   return(cpuTime);
	  } /* RUNTIM */

	else if(equ(GC_PUSHS(uuo),GC_PUSHS("MSTIME"))==0)
	  { /* MSTIME */
#ifdef ADD_THIS_CODE /* 10-9-87 */
	   struct timeval tp;         /* BSD: in <sys/time.h> */
           struct timezone tzp;
	   gettimeofday(&tp, &tzp);   /* get time kind of things */
           runTime = tp.tv_usec;      /* day time in milliseconds */
#endif
	   return(runTime);
	  } /* MSTIME */

	else if(equ(GC_PUSHS(uuo),GC_PUSHS("RUN"))==0)
	  { /* RUN */
           filename = PNULL;    /* NEED TO PASS FILENAME and ARGS !!!! */
	   execl(filename,0);
	  } /* RUN */

        else
	  { /* not implemented */
	   PSprintf("callsys(%s) not  fully implemented at this time!\n",
		    GC_PUSHS(uuo));
          } /* not implemented */
}


/* ---> END OF PSRUN2.C <--- */
