/* 	File: PSRUN3.C 						*/
/*								*/
/*	SAIL RUNTIME  STRING PROCEDURES				*/
/*								*/
/*								*/
/*	P. Lemkin						*/
/*	Image Processing Section				*/
/*	National Cancer Institute, DCBD, LTB			*/
/*	FCRF, Build 469 Room 150B				*/
/*	Frederick, MD 21701					*/
/*
*    COPYRIGHT NOTICE - Copyright (c) 1983-1987 by Peter F. Lemkin.
*                     All rights reserved.
*       Permission is granted to any individual or institution to
*       copy, use or modify this program, except for explicitly
*       commercial purposes, provided this copyright notice is
*       included in its entirety in all derived works.  No warranty
*       of the software or of the accuracy of any documentation
*       surrounding it is expressed or implied, and the author does
*       not acknowledge any liability resulting from program or
*       documentation errors.
*/
/*								*/
/*	Version: 57.10						*/
/*								*/
/*	Revised September 7, 1987				*/
/*	Revised June 30, 1987				        */
/*	Revised February 28, 1987				*/
/*	Revised August 14, 1986				*/
/*	Revised August 12, 1986				*/
/*	Revised August 11, 1986				*/
/*	Revised August 7, 1986				*/
/*	Revised August 6, 1986				*/
/*	Revised July 15, 1986				*/
/*	Revised July 8, 1986				*/
/*	Revised June 27, 1986				*/
/*	Revised June 22, 1986				*/
/*	Revised March 15, 1986				*/
/*	Revised March 8, 1986				*/
/*	Revised February 28, 1986				*/
/*	Revised December 18, 1985				*/
/*	Revised December 12, 1985				*/
/*	Revised December 6, 1985				*/
/*	Revised December 2, 1985				*/
/*	Revised November 26, 1985				*/
/*	Revised November 26, 1985 B.S. 				*/
/*	Revised November 5, 1985				*/
/*	Revised October 11, 1985				*/
/*	Revised August 8, 1985					*/
/*	Revised August 6, 1985					*/
/*	Revised June 7, 1985					*/
/*	Revised May 30, 1985					*/
/*	Revised April 26, 1985					*/
/*	Revised March 8, 1985					*/
/*	June 29, 1984						*/
/*								*/
/*								*/
/*	This procedure package includes the following standard 	*/
/*	runtime functions necessary for easy use of the PSAIL	*/
/*	compiler in a UNIX environment.				*/
/*								*/
/*	3.0 Auxillary functions used in library.		*/
/*      Patod(s) - return double value of string number s       */
/*      Patof(s) - return float value of string number s        */
/*      Patoi(s) - return int value of string number s          */
/*      Patol(s) - return long value of string number s         */
/*      Patoo(s) - return long octal value of string number s   */
/*      Patou(s) - return long unsigned value of string number s*/
/*      Patox(s) - return long hex value of string number s     */
/*								*/
/*  3. STRING procedures: sub-module <psrun3.c>			*/
/*	*catlist(ptr list) - concatenate list of string pointers */
/*	*cncat2(a,b) - return Pmalloc() allocated STRING of (a&b)*/
/*	*concat(a,b) - return SAIL style STRING  of (a&b)	*/
/*	*Pcpystr(s) - return dynamic string copy of string s.	*/
/*	length(s) - return length of string s.			*/
/*	*strsub(s,a,fn,b) - return STRING s[a,fn,b] substring	*/
/*	*subsc(s,a,b) - return STRING s[a For b] substring	*/
/*	*subst(s,a,b) - return STRING s[a To b] substring	*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    C O M P I L E   T I M E   M A C R O S              */
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/


#ifndef HASCONFIG
#include "config.h"	/* declare HAS... macros */
#endif

#ifndef PNULL
#include "psrdef.h"	/* need <sairun.c> & <psrung.h> compile-time macros */
#include "psrglb.h"	/* need <sairun.c> global variables */


/*	The following are definitions in PSRUN1.C		*/
STRING Pmalloc();    /* allocate nbyte storage from heap */
STRING salloc();     /* alloc nbyte string GC'able string space*/
#endif			/* end of code brought in if compiling alone */

/*	The following are definitions in PSRUN8.C		*/
float ppowff();	/* compute x**y floating		*/

/*	The following are FORWARD definitions in PSRUN3.C	*/
float Patof();       /* return float value of string # s */
STRING Pcpystr();    /* return copy of string s. */

/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    G L O B A L    V A R I A B L E S / M A C R O S	*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/

#ifndef TRUE
#include <psrdef.h>		/* <sairun.c> macros */
#endif


/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    S T R I N G  P R O C E D U R E S   		*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert double precision floating decimal SAIL string	*/
/*	number to double value. Return the double value.	*/
/*								*/
double PROCEDURE Patod(s)
	STRING s; 
{
	GC_POPS(s);	/* get arg from temp string var. PDL */
	return((double)Patof(s));
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert single precision floating decimal SAIL string	*/
/*	number to float value. Return the float value.		*/
/*								*/
float PROCEDURE Patof(s)
	STRING s; 
{
	int sawDP, digit, nbrDigits, expon, signExp, doingExp, i;
	char c;
	float f, t, fract;

	GC_POPS(s);	/* get arg from temp string var. PDL */

	if ((int)s==0 || *s=='\0') return(0.0);
	nbrDigits=0;
	expon=1;
	doingExp=FALSE;
	sawDP=0;
	f=0.0;
	fract=1.0;
	signExp=1;
	i=0;
	while ((*s != '\0') && (*s==' ' || *s=='\n' || *s=='\t') &&
		(i++ < 20)) 
			s++; /* trash white space */
	while ( (*s!='\0') || ((*s >= '0') && (*s <= '9')) ||
		(*s=='.') || 
		(! doingExp && (*s=='E' || *s=='e' || *s=='@') &&
		  nbrDigits==0) )
	   if (*s=='E' || *s=='e' || *s=='@') doingExp=TRUE;
	   else if (doingExp)
		{c = *(s++); /* compute the exponent */
		 if (c=='+') {signExp=1; continue;};
		 if (c=='-') {signExp= -1; continue;};
		 expon=(expon*10)+(c-'0');
		}
	   else	{t=f; /* compute number, get old value of f */
		 c = *s++;  /* extract character */
		 nbrDigits++; /* count characters in the number */
		 if (c=='.') {sawDP=1; continue;}
		 digit=c - '0'; /* compute value of digit in string */
		 if (sawDP==0) t=(10*t) + digit;
			else t=t + ((fract=fract/10)*digit);
		 if ((t >= 0) && (nbrDigits <= 0)) return (0.0);
		 f=t; /* save new value */
		}
	return(f*ppowff(10.0,(float)signExp*expon));
					   /* return float value */
}


/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert char array 's' to decimal INTEGER number */
/*							 */
INTEGER PROCEDURE Patoi(s)
	STRING s;
{	
	INTEGER i, sign;
	INTEGER n;

	GC_POPS(s);	/* get arg from temp string var. PDL */

	if ((int)s==0 || *s=='\0') return((INTEGER)0);
	i=0;
	while ((*s==' ' || *s=='\n' || *s=='\t') && (i++ <= 20))
		{s++; /* skip white space */}
	sign=1;
	if (*s == '+') {s++; sign = 1;}
	else if (*s == '-') {s++; sign = -1;}
	n=0;
	while ((*s >= '0' && *s <= '9'))
		{
		n=10*n + *s - '0';
		s++; i++;}
	return(sign*n);
}


/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert char array 's' to decimal INTEGER number */
/*							 */
INTEGER PROCEDURE Patol(s)
	STRING s;
{	
	INTEGER i, sign;
	INTEGER n;

	GC_POPS(s);	/* get arg from temp string var. PDL */

	if ((int)s==0 || *s=='\0') return((INTEGER)0);
	i=0;
	while ((*s==' ' || *s=='\n' || *s=='\t') && (i++ <=20))
		{s++; /* skip white space */}
	sign=1;
	if (*s == '+') {s++; sign = 1;}
	else if (*s == '-') {s++; sign = -1;}
	n=0;
	while ((*s >= '0' && *s <= '9'))
		{
		n=10*n + *s - '0';
		s++; i++;}
	return(sign*n);
}


/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert char array 's' to octal INTEGER number	 */
/*							 */
INTEGER PROCEDURE Patoo(s)
	STRING s;
{	
	INTEGER i, sign;
	INTEGER n;

	GC_POPS(s);	/* get arg from temp string var. PDL */

	if ((int)s==0 || *s=='\0') return((INTEGER)0);
	i=0;
	while ((*s==' ' || *s=='\n' || *s=='\t')&&(i++<=20))
		{s++; /* skip white space */}
	sign=1;
	if (*s == '+') {s++; sign = 1;}
	else if (*s == '-') {s++; sign = -1;}
	n=0;
	while ((*s >= '0' && *s <= '7'))
		{
		n=8*n + *s - '0';
		s++; i++;}
	return(sign*n);
}


/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert char array 's' to unsigned INTEGER number */
/*							 */
INTEGER PROCEDURE Patou(s)
	STRING s;
{	
	INTEGER i;
	INTEGER n;

	GC_POPS(s);	/* get arg from temp string var. PDL */

	if ((int)s==0 || *s=='\0') return((INTEGER)0);
	i=0;
	while ((*s==' ' || *s=='\n' || *s=='\t')&&(i++<=20))
		{s++; /* skip white space */}
	n=0;
	while ((*s >= '0' && *s <= '9'))
		{
		n=10*n + *s - '0';
		s++; i++;}
	return(n);
}


/*  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert char array 's' to hex INTEGER number      */
/*							 */
INTEGER PROCEDURE Patox(s)
	STRING s;
{	
	INTEGER i, ch;
	INTEGER n;

	GC_POPS(s);	/* get arg from temp string var. PDL */

	if ((int)s==0 || *s=='\0') return((INTEGER)0);
	i=0;
	while ((*s==' ' || *s=='\n' || *s=='\t')&&(i++<=20))
		{s++; /* skip white space */}
	n=0;
	while (((*s >= '0' && *s <= '9')) ||
	       ((*s >= 'A' && *s <='F')) ||
	       ((*s >= 'a' && *s <='f')))
		{
		ch = (*s >= '0' && *s <= '9') ? *s :
		       	(*s >= 'A' && *s <='F') ? (*s-'A'+10) :
		       	(*s >= 'a' && *s <='f') ? (*s-'a'+10) : 0;
/* @V.51.52 B.S. Need to detect illegal HEX #'s like NUMIN error! */
/* @V.51.102 Check other Pato?() conversions.! */
		n=16*n + ch - '0';
		s++; i++;}
	return(n);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Concatenate list of string pointers to 1st string*/
/*	Eg. catlist(nL, p1,p2,...,pn); computes:		 */
/*	q = p1 & p2 & ... & pn, where & is concat ptrs   */
/*	All strings are passed through the temp string 	 */
/*	varable PDL.			 		 */
/*							 */
/*	@V.50.65 Need to redo this where:		 */
/*		1) 1st pass compute total of lengths.	 */
/*		2) allocate new string of this length.	 */
/*		3) on 2nd pass copy all strings to new one*/
STRING PROCEDURE catlist(nArgs, ptr)
	INTEGER nArgs;		/* number of string args */
	STRING ptr;		/* ptr to rest of C arg PDL */
{
	STRING	*t;		/* ptr to list ptr */
	STRING	p,		/* holds current string arg */
		q,		/* result string pointer */
		qW;		/* working pointer into q string */
	INTEGER k,		/* tempoary variable */
		n,		/* working counter */
		totLength; 	/* total length of all strings */

/*	[1] Compute number of args and total length of all strings */
	t = &ptr;		/* set up to point to list of ptrs */
	totLength = n = 0;	/* init counter and size */
	while (n <= nArgs)
		{ /* get string arg and add its length */
		GC_GETARGS(p,n++);	/* get n'th temp string 
					 * variable PDL from the top
					 * of the PDL.
					*/
		totLength += length(GC_PUSHS(p));
		INC_ARG_PTR(t);	/* increment variable arg pointer */
		} /* get string arg and add its length */

/*		[2] Allocate new string 'q' of this length.	 */
		q = salloc(totLength+1);

/*	[3] On 2nd pass copy all strings to new one	*/
	qW = q;		/* set up working ptr to output string */
	for (n=1; n<=nArgs; n++)
		{ /* build up q as concat of {p1,p2,...pNarg} */
		GC_GETARGS(p,n);	/* get n'th temp string */
		p = (STRING)*p;		/* since stack off one level */
		while (*(qW++) = *(p++))
		        ;   /* concat including '\0' */
		qW--;			/* backup 1 char */
		}
	*qW = '\0';		/* finish off output string */
	GC_SADJN(nArgs);  /* remove string args from temp string PDL */
	return(GC_PUSHS(q));
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */
/*	Concatenate string q to string p and return the  */
/*	new string which is gotten from free string space*/
/*	using Pmalloc() NOT salloc()!!! This is so we do */
/*	not mess up the string allocation space for G.C. */
/*	If the string pointer is in the range of 0 to 177*/
/*	then it is assumed to be a single char of data	 */
/*	and is used directly instead of as a ptr to a    */
/*      string.                                         */
/*	NOTE: pass the string pointer back and DO NOT	*/
/*	USE the temporary string variable stack.	*/
/*							*/
STRING PROCEDURE cncat2(p,q)
	STRING p, q;
{
	char s1[2], s2[2];
	STRING r, oldR;

	if ((0<(int)p) && ((int)p<0177))
		{s1[0]=(int)p; s1[1]='\0'; p= &s1[0];} /* make string */
	if ((0<(int)q) && ((int)q<0177))
		{s2[0]=(int)q; s2[1]='\0'; q= &s2[0];} /* make string */
	if ((p==0 || *p=='\0') && (q==0 || *q=='\0'))
		return(PNULL);  /*test for special NULL cases*/
	else if (*p=='\0' && *q!='\0') 
		return(q); /* only 1 non-null string*/
	else if (*p!='\0' && *q=='\0') 
		return(p); /* only 1 non-null string */
	oldR = r = Pmalloc(length(GC_PUSHS(p))+length(GC_PUSHS(q))+1);
					 /* make long enough */
	while ((*r++ = *p++) != '\0') /* copy p to r */
		;
	--r;			/* back up over null */
	while ((*r++ = *q++) != '\0') /* copy q to r */
		;
	*r = '\0';
	return(oldR);	/* NOTE: don't use string PDL */
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Concatenate string q to string p and return the    */
/*	new string which is gotten from free string space  */
/*	If the string pointer is in the range of 0 to 177  */
/*	then it is assumed to be a single char of data and */
/*	is used directly instead of as a ptr to a string.   */
/*	NOTE: salloc() NOT Pmalloc() is used so we can put  */
/*	the string pointer into the active string ptr PDL   */
/*      						    */
STRING PROCEDURE concat(p,q)
	STRING p, q;
{
	char s1[2], s2[2];

	STRING r, oldR;
	GC_POPS(p);
	GC_POPS(q);
	GC_MARKSSTKFRAME;
	GC_NOTESTRING(p);
	GC_NOTESTRING(q);
	GC_NOTESTRING(r);
	GC_NOTESTRING(oldR);

	if ((0<(int)p) && ((int)p<0177))
		{s1[0]=(int)p; s1[1]='\0'; p= &s1[0];} /* make string */
	if ((0<(int)q) && ((int)q<0177))
		{s2[0]=(int)q; s2[1]='\0'; q= &s2[0];} /* make string */
	if ((p==0 || *p=='\0') && (q==0 || *q=='\0'))
		GC_SRETURN(PNULL)  /*test for special NULL cases*/
	else if (*p=='\0' && *q!='\0') 
		GC_SRETURN(Pcpystr(q)) /* only 1 non-null string*/
	else if (*p!='\0' && *q=='\0') 
		GC_SRETURN(Pcpystr(p)) /* only 1 non-null string */
	oldR = r = salloc(length(GC_PUSHS(p))+length(GC_PUSHS(q))+1);
					 /* make long enough */
	while ((*r++ = *p++) != '\0') /* copy p to r */
		;
	--r;			/* back up over null */
	while ((*r++ = *q++) != '\0') /* copy q to r */
		;
	*r = '\0';
	GC_SRETURN(oldR)
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Copy string p into a new string and return it.	*/
/*	New string is gotten from free string space.	*/
/*							*/
STRING PROCEDURE Pcpystr(p)
	STRING p;
{
	STRING r, oldR;

	GC_POPS(p);	/* get arg from temp string var. PDL */
	GC_MARKSSTKFRAME;
	GC_NOTESTRING(p);
	GC_NOTESTRING(r);
	GC_NOTESTRING(oldR);

	if ((int)p==0 || *p=='\0') GC_SRETURN(PNULL)
	oldR = r = salloc(length(GC_PUSHS(p))+1); /* make long enough */
	while ((*r++ = *p++) != '\0'); /* copy p to r */
	*r = '\0';	/* terminate string */
	GC_SRETURN(oldR)
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Return the length of string p.			*/
/*							*/
INTEGER PROCEDURE length(p)
	STRING p;
{
	INTEGER i;

	GC_POPS(p);	/* get arg from temp string var. PDL */

	if ((int)p==0 || *p=='\0')
		return((INTEGER)0);
	i = 0;
	while (*(p++) != '\0') /*find end of p */
		i++;
	return(i);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* 	Extract and return substring s[a fn b] where	*/
/*	fn ="for" or "to". Illegal (a,b) lengths are    */
/*	mapped according to the algorithm given in the   */
/*	SAIL manual [Reiser76] page 27. 		*/
/*							*/
STRING PROCEDURE strsub(s,a,fn,b)
	STRING s;
	STRING fn; /* Note: fn is either 'for' or 'to' */
	INTEGER a,b;
{
	STRING oldR, r;
	char op;
	INTEGER j, ls;

	GC_POPS(s);
	GC_POPS(fn);
	GC_MARKSSTKFRAME;
	GC_NOTESTRING(s);
	GC_NOTESTRING(fn);
	GC_NOTESTRING(oldR);
	GC_NOTESTRING(r);

	if ((int)s==0 || *s=='\0')
		GC_SRETURN(PNULL)
	op = *fn;			/* Get 1st char of function */
	if (op='f') {b=a+b-1; op='t';}	/* Map 'for' to 'to' */
	ls = length(GC_PUSHS(s));
	if (a<1) a=1;			/* Set left index to end */
	if (b>ls) b=ls;			/* Set right index to end */
	if (ls==0 || op!='t' || b<0 || a>b) 
		GC_SRETURN(PNULL)
	j=b;
	oldR = r = salloc(j+1);		/* make long enough */
	do *(r++) = *(s+(a++)-1); while ((a <= j) && (*r != '\0'));
	*r = '\0';
	GC_SRETURN(oldR)
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* 	Extract and return substring s[a FOR b] where	*/
/*	Illegal (a,b) lengths are mapped according to 	*/
/*	the algorithm given in the SAIL manual 		*/
/*	[Reiser76] page 27. 				*/
/*							*/
STRING PROCEDURE subsr(s,a,b)
	STRING s;
	INTEGER a,b;
{	
	STRING oldR, r;
	INTEGER j, ls;

	GC_POPS(s);	/* get arg from temp string var. PDL */
	GC_MARKSSTKFRAME;
	GC_NOTESTRING(s);
	GC_NOTESTRING(oldR);
	GC_NOTESTRING(r);

	if ((int)s==0 || *s=='\0')
		GC_SRETURN(PNULL)
	b=a+b-1;			/* Map 'for' to 'to' */
	ls = length(GC_PUSHS(s));
	if (a<1) a=1;			/* Set left index to end */
	if (b>ls) b=ls;			/* Set right index to end */
	if (ls==0 || b<0 || a>b) 
		GC_SRETURN(PNULL)
	j=b;
	oldR = r = salloc(j+1);		/* make long enough */
	do *(r++) = *(s+(a++)-1); while ((a <= j) && (*r != '\0'));
	*r = '\0';
	GC_SRETURN(oldR)
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* 	Extract and return substring s[a TO b] where	*/
/*	Illegal (a,b) lengths are mapped according to 	*/
/*	the algorithm given in the SAIL manual 		*/
/*	[Reiser76] page 27. 				*/
/*							*/
STRING PROCEDURE subst(s,a,b)
	STRING s;
	INTEGER a,b;
{	
	STRING oldR, r;
	INTEGER j, ls;

	GC_POPS(s);	/* get arg from temp string var. PDL */
	GC_MARKSSTKFRAME;
	GC_NOTESTRING(s);
	GC_NOTESTRING(oldR);
	GC_NOTESTRING(r);

	if (((int)s)==0 || *s=='\0')
		GC_SRETURN(PNULL)
	ls = length(GC_PUSHS(s));
	if (a<1) a=1;			/* Set left index to end */
	if (b>ls) b=ls;			/* Set right index to end */
	if (ls==0 || b<0 || a>b) 
		GC_SRETURN(PNULL)
	j=b;
	oldR = r = salloc(j+1);		/* make long enough */
	do *(r++) = *(s+(a++)-1); while ((a <= j) && (*r != '\0'));
	*r = '\0';
	GC_SRETURN(oldR)
}


/* ---> END OF PSRUN3.C <--- */
