/* 	File: PSRUN4.C 						*/
/*								*/
/*	SAIL RUNTIME  STRING CONVERSION FUNCTIONS		*/
/*								*/
/*								*/
/*	P. Lemkin						*/
/*	Image Processing Section				*/
/*	National Cancer Institute, DCBD, LTB			*/
/*	FCRF, Build 469 Room 150B				*/
/*	Frederick, MD 21701					*/
/*
*    COPYRIGHT NOTICE - Copyright (c) 1983-1987 by Peter F. Lemkin.
*                     All rights reserved.
*       Permission is granted to any individual or institution to
*       copy, use or modify this program, except for explicitly
*       commercial purposes, provided this copyright notice is
*       included in its entirety in all derived works.  No warranty
*       of the software or of the accuracy of any documentation
*       surrounding it is expressed or implied, and the author does
*       not acknowledge any liability resulting from program or
*       documentation errors.
*/
/*								*/
/*	Version: 57.10						*/
/*								*/
/*	Revised October 10, 1987			        */
/*	Revised October 4, 1987				        */
/*	Revised September 7, 1987				*/
/*	Revised July 1, 1987					*/
/*	Revised June 30, 1987					*/
/*	Revised February 28, 1987				*/
/*	Revised August 11, 1986					*/
/*	Revised August 12, 1986					*/
/*	Revised August 7, 1986					*/
/*	Revised August 6, 1986					*/
/*	Revised July 15, 1986					*/
/*	Revised July 8, 1986					*/
/*	Revised June 27, 1986					*/
/*	Revised March 15, 1986					*/
/*	Revised March 8, 1986					*/
/*	Revised March 5, 1986					*/
/*	Revised January 3, 1986					*/
/*	Revised December 18, 1985				*/
/*	Revised December 2, 1985				*/
/*	Revised November 27, 1985 -B.S.				*/
/*	Revised November 26, 1985				*/
/*	Revised November 21, 1985				*/
/*	Revised November 5, 1985				*/
/*	Revised October 11, 1985				*/
/*	Revised August 6, 1985					*/
/*	Revised May 30, 1985					*/
/*	Revised March 8, 1985					*/
/*	June 29, 1984						*/
/*								*/
/*	This procedure package includes the following standard 	*/
/*	runtime functions necessary for easy use of the PSAIL	*/
/*	compiler in a UNIX environment.				*/
/*								*/
/*	4.0 Auxillary functions used in library.		*/
/*      Pdtoa(n,s) - return string number of double value       */
/*      Pdetoa(n,s) - return string number of double E value    */
/*      Petoa(n,s) - return string number of float E value      */
/*      Pftoa(n,s) - return string number of float value        */
/*      Pitoa(n,s) - return string number of int value          */
/*      Pltoa(n,s) - return string number of long value         */
/*      Potoa(n,s) - return string number of long octal value   */
/*      Putoa(n,s) - return string number of long unsigned value*/
/*      Pxtoa(n,s) - return string number of long hex value     */
/*								*/
/*  4. STRING conversion functions: sub-module <psrun4.c>	*/
/*	copstr(s) - return first char of string s. Null if empty*/
/*	*cv6str(n) - convert sixbit INTEGER n to returned string s	*/
/*	cvasc(s) - cvt (5 character) string s to 7-bit INTEGER word */
/*	*cvastr(n) - convert sixbit INTEGER n to returned string s */
/*	cvd(s) - return decimal integer INTEGER value of string s. */
/*	*cvf(f) - cvt. float f to dec. string, return char ptr. */
/*	*cve(f) - cvt. float f to E-fmt string, return char ptr.*/
/*	*cvg(f) - cvt. float f to G-fmt string, return char ptr.*/
/*	cvo(s) - return octal value of string s.		*/
/*	*cvos(n) - cvt. octal INTEGER n to string, return ptr.	*/
/*	*cvs(n) - cvt. decimal INTEGER n to string, return ptr.	*/
/*	cvsix(s) - cvt (6 character) string s to sixbit INTEGER	*/
/*	*cvstr(n) - convert sixbit INTEGER n to returned string s */
/*	*cvxstr(n) - convert 7-bit ASCII 5-digit INTEGER n==>string s*/
/*	equ(s1,s2) - return TRUE if s1[1:inf]==s2[1:inf] else FALSE*/
/*	getformat(w,p) - get default field width & precision	*/
/*	*makstr(n) - make a 2 character string from LSB 7-bits n */
/*	lop(s) - return (and advance) first char of string s.   */
/*	setformat(w,p) - set default field width & precision	*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    C O M P I L E   T I M E   M A C R O S              */
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/

#ifndef HASCONFIG
#include "config.h"	/* declare HAS... macros */
#endif

#ifndef PNULL
#include "psrdef.h"	/* need <sairun.c> & <psrung.h> compile-time macros */
#include "psrglb.h"	/* need <sairun.c> global variables */


/*	The following are definitions in PSRUN1.C		*/
STRING salloc();   /* alloc nbyte string GC'able string space*/

#endif			/* end of code brought in if compiling alone */

/*	The following are definitions in PSRUN8.C		*/
INTEGER ppowii();	/* compute x**y INTEGER		*/

/*	The following are FORWARD definitions in PSRUN4.C	*/
void fill();      /* Fill string w/ a MIN of w digits, '0's or ' '*/

/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    G L O B A L    V A R I A B L E S / M A C R O S	*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/

/*	@V.56.35 -- check all of these numbers 		*/

#define MAXTMP 30

#if (maxBITS==32)
#define MAXOCTDIG 11
#define MAXHEXDIG 8
#define MAXDECDIG 9
#define MAXMANTISSADIG 6
#define MAXEXPDIG 2
#define MAX6BITDIG 5
#define MAXASCIIDIG 4
#define LSHIFT 8
#define RSHIFT 24
#define RASCIISHIFT 25
#endif

#if (maxBITS==36)
#define MAXOCTDIG 12
#define MAXHEXDIG 9
#define MAXDECDIG 10
#define MAXMANTISSADIG 7
#define MAXEXPDIG 2
#define MAX6BITDIG 6
#define MAXASCIIDIG 5
#define LSHIFT 7
#define RSHIFT 29
#define RASCIISHIFT 30
#endif

#if (maxBITS==64)
#define MAXOCTDIG 22
#define MAXHEXDIG 16
#define MAXDECDIG 19
#define MAXMANTISSADIG 14
#define MAXEXPDIG 4
#define MAX6BITDIG 10
#define MAXASCIIDIG 9
#define LSHIFT 8
#define RSHIFT 56
#define RASCIISHIFT 50
#endif


/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    S T R I N G    C O N V .   P R O C E D U R E S     */
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert double decimal 'nbr' to char array 's'	*/
/*							*/
/* .... needs work ....@V.48.xx */
void PROCEDURE Pdtoa(nbr,s,width,precision)
	double nbr;
	char s[];
	INTEGER width, precision;
{ /*PDTOA*/
	int j, jr, zs, k;
	INTEGER n;
	float fraction;
	char c;
	char sReversed[MAXTMP];

	zs=0;
	j=0; /* index to s */
	jr=0; /* index to sReversed */
	if (nbr<0)
		{nbr= -nbr;
		s[j++]='-';
		}
	n = nbr; /*truncate integer part */
	fraction = nbr - n;
	while ((n>0) && (jr < MAXTMP))
		{
		c=n%10 + '0';
		zs=1;
		sReversed[jr++]=c;
		n=n/10;
 		}
	/* compute the fraction part */
	k = width - (precision + jr); /* # digits left */
	n = (k > 0) ? fraction * ppowii(10L,(INTEGER)k) : 0;
	sReversed[jr++]='.';
	while ((n>0) && (jr < MAXTMP))
		{c = n%10 + '0';
		 zs=1;
		 sReversed[jr++]=c;
		 n = n/10;
		}
	if (zs==0) s[j++]='0';
		else while (jr!=0)
		         s[j++]=sReversed[--jr];
	s[j++]='\0';
} /*PDTOA*/



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert double e-format 'nbr' to char array 's'	*/
/*							*/
/* .... needs work ....@V.48.xx */
void PROCEDURE Pdetoa(nbr,s,width,precision)
	double nbr;
	char s[];
	INTEGER width, precision;
{ /*PDETOA*/
	int j, jr, zs, k, expon;
	int n;
	double fraction;
	char c;
	char sReversed[MAXTMP];

	zs=0;
	j=0; /* index to s */
	jr=0; /* index to sReversed */
	if (nbr<0)
		{nbr= -nbr;
		s[j++]='-';
		}
	if  (ABS(nbr) > 1.0) 
		{expon = log10((float)nbr);
		 nbr /= ppowii(10L,(INTEGER)expon);
		}
		else {expon = - log10((float)(1.0/nbr));
		 nbr /= ppowii(10L,(INTEGER)expon);
		}
	n = nbr; 	/* truncate integer part */
	fraction = nbr - n;
	while ((n>0) && (jr < MAXTMP))
		{
		c=n%10 + '0';
		zs=1;
		sReversed[jr++]=c;
		n=n/10;
 		}
	/* compute the fraction part */
	k = width - (precision + jr); /* # digits left */
	n = (k > 0) ? fraction * ppowii(10,k) : 0;
	sReversed[jr++]='.';
	while ((n>0) && (jr < MAXTMP))
		{c = n%10 + '0';
		 zs=1;
		 sReversed[jr++]=c;
		 n = n/10;
		}
	/* output the expon */
	sReversed[jr++] = 'E';
	sReversed[jr++] = (expon >= 0) ? '+' : '-';
	if (expon<0) expon= -expon;
	while ((expon>0) && (jr < MAXTMP))
		{
		c=expon%10 + '0';
		zs=1;
		sReversed[jr++]=c;
		expon=expon/10;
 		}
	if (zs==0) s[j++]='0';
		else while (jr!=0)
		         s[j++]=sReversed[--jr];
	s[j++]='\0';
} /*PDETOA*/


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert float e-format 'nbr' to char array 's'	*/
/*							*/
/* .... needs work ....@V.48.xx */
void PROCEDURE Petoa(nbr,s,width,precision)
	float nbr;
	char s[];
	INTEGER width, precision;
{ /*PETOA*/
	int j, jr, zs, k, expon;
	int n;
	float fraction;
	char c;
	char sReversed[MAXTMP];

	zs=0;
	j=0; /* index to s */
	jr=0; /* index to sReversed */
	if (nbr<0)
		{nbr= -nbr;
		s[j++]='-';
		}
	if  (ABS(nbr) > 1.0) 
		{expon = log10((float)nbr);
		 nbr /= ppowii(10L,(INTEGER)expon);
		}
		else {expon = - log10((float)(1.0/nbr));
		 nbr /= ppowii(10L,(INTEGER)expon);
		}
	n = nbr; 	/* truncate integer part */
	fraction = nbr - n;
	while ((n>0) && (jr < MAXTMP))
		{
		c=n%10 + '0';
		zs=1;
		sReversed[jr++]=c;
		n=n/10;
 		}
	/* compute the fraction part */
	k = width - (precision + jr); /* # digits left */
	n = (k > 0) ? fraction * ppowii(10L,(INTEGER)k) : 0;
	sReversed[jr++]='.';
	while ((n>0) && (jr < MAXTMP))
		{c = n%10 + '0';
		 zs=1;
		 sReversed[jr++]=c;
		 n = n/10;
		}
	/* output the expon */
	sReversed[jr++] = 'E';
	sReversed[jr++] = (expon >= 0) ? '+' : '-';
	if (expon<0) expon= -expon;
	while ((expon>0) && (jr < MAXTMP))
		{
		c=expon%10 + '0';
		zs=1;
		sReversed[jr++]=c;
		expon=expon/10;
 		}
	if (zs==0) s[j++]='0';
		else while (jr!=0)
		         s[j++]=sReversed[--jr];
	s[j++]='\0';
} /*PETOA*/



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert float decimal 'nbr' to char array 's'	*/
/*							*/
/* .... needs work ....@V.48.xx */
void PROCEDURE Pftoa(nbr,s,width,precision)
	float nbr;
	char s[];
	INTEGER width, precision;
{ /*PFTOA*/
	int j, jr, zs, k;
	INTEGER n;
	float fraction;
	char c;
	char sReversed[MAXTMP];

	zs=0;
	j=0; /* index to s */
	jr=0; /* index to sReversed */
	if (nbr<0)
		{nbr= -nbr;
		s[j++]='-';
		}
	n = nbr; /*truncate integer part */
	fraction = nbr - n;
	while ((n>0) && (jr < MAXTMP))
		{
		c=n%10 + '0';
		zs=1;
		sReversed[jr++]=c;
		n=n/10;
 		}
	/* compute the fraction part */
	k = width - (precision + jr); /* # digits left */
	n = (k > 0) ? fraction * ppowii(10L,(INTEGER)k) : 0;
	sReversed[jr++]='.';
	while ((n>0) && (jr < MAXTMP))
		{c = n%10 + '0';
		 zs=1;
		 sReversed[jr++]=c;
		 n = n/10;
		}
	if (zs==0) s[j++]='0';
		else while (jr!=0)
		         s[j++]=sReversed[--jr];
	s[j++]='\0';
} /*PFTOA*/


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert INTEGER hex 'nbr' to char array 's'	*/
/*							*/
void PROCEDURE Phtoa(nbr,s)
	INTEGER nbr;
	char s[];
{ /*PHTOA*/
	int j, jr, zs;
	char c;
	char sReversed[MAXTMP];
	static char shex[17]="0123456789ABCDEF";

	zs=0;
	j=0; /* index to s */
	jr=0; /* index to sReversed */
	if (nbr<0)
		{nbr= -nbr;
		s[j++]='-';
		}
	while ((nbr>0) && (jr < MAXTMP))
		{
		c=shex[(nbr%10)&017];
		zs=1;
		sReversed[jr++]=c;
		nbr=nbr/10;
 		}
	if (zs==0) s[j++]='0';
		else while (jr!=0)
		         s[j++]=sReversed[--jr];
	s[j++]='\0';
} /*PHTOA*/



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert INTEGER decimal 'nbr' to char array 's'	*/
/*							*/
void PROCEDURE Pitoa(nbr,s)
	INTEGER nbr;
	char s[];
{ /*PITOA*/
	int j, jr, zs;
	char c;
	char sReversed[MAXTMP];

	zs=0;
	j=0; /* index to s */
	jr=0; /* index to sReversed */
	if (nbr<0)
		{nbr= -nbr;
		s[j++]='-';
		}
	while ((nbr>0) && (jr < MAXTMP))
		{
		c=nbr%10 + '0';
		zs=1;
		sReversed[jr++]=c;
		nbr=nbr/10;
 		}
	if (zs==0) s[j++]='0';
		else while (jr!=0)
		        s[j++]=sReversed[--jr];
	s[j++]='\0';
} /*PITOA*/


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert INTEGER decimal 'nbr' to char array 's'	*/
/*							*/
void PROCEDURE Pltoa(nbr,s)
	INTEGER nbr;
	char s[];
{ /*PLTOA*/
	int j, jr, zs;
	char c;
	char sReversed[MAXTMP];

	zs=0;
	j=0; /* index to s */
	jr=0; /* index to sReversed */
	if (nbr<0)
		{nbr= -nbr;
		s[j++]='-';
		}
	while ((nbr>0) && (jr < MAXTMP))
		{
		c=nbr%10 + '0';
		zs=1;
		sReversed[jr++]=c;
		nbr=nbr/10;
 		}
	if (zs==0) s[j++]='0';
		else while (jr!=0)
		        s[j++]=sReversed[--jr];
	s[j++]='\0';
} /*PLTOA*/


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert INTEGER octal 'nbr' to char array 's'	*/
/*							*/
void PROCEDURE Potoa(nbr,s)
	INTEGER nbr;
	char s[];
{ /*PITOA*/
	int j, jr, zs;
	char c;
	char sReversed[MAXTMP];

	zs=0;
	j=0; /* index to s */
	jr=0; /* index to sReversed */
	if (nbr<0)
		{nbr= -nbr;
		s[j++]='-';
		}
	while ((nbr>0) && (jr < MAXTMP))
		{
		c=nbr%8 + '0';
		zs=1;
		sReversed[jr++]=c;
		nbr = (nbr >> 3);	/* i.e. nbr=nbr/8; */
 		}
	if (zs==0) s[j++]='0';
		else while (jr!=0) 
		        s[j++]=sReversed[--jr];
	s[j++]='\0';
} /*POTOA*/

/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert INTEGER unsigned 'nbr' to char array 's'*/
/*							*/
void PROCEDURE Putoa(nbr,s)
	INTEGER nbr;
	char s[];
{ /*PUTOA*/
	int j, jr, zs;
	char c;
	char sReversed[MAXTMP];

	zs=0;
	j=0; /* index to s */
	jr=0; /* index to sReversed */
	while ((nbr>0) && (jr < MAXTMP))
		{
		c=nbr%10 + '0';
		zs=1;
		sReversed[jr++]=c;
		nbr=nbr/10;
 		}
	if (zs==0) s[j++]='0';
		else while (jr!=0) 
		         s[j++]=sReversed[--jr];
	s[j++]='\0';
} /*PUTOA*/

/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert INTEGER hex 'nbr' to char array 's'*/
/*							*/
void PROCEDURE Pxtoa(nbr,s)
	INTEGER nbr;
	char s[];
{/*PXTOA*/
	int j, jr, zs, k;
	char c;
	char sReversed[MAXTMP];

	zs=0;
	j=0; /* index to s */
	jr=0; /* index to sReversed */
	while ((nbr>0) && (jr < MAXTMP))
		{
		k=nbr%16;
		c=(k<10) ? k+'0' : k+'A';
		zs=1;
		sReversed[jr++]=c;
		nbr=nbr/16;
 		}
	if (zs==0) s[j++]='0';
		else while (jr!=0)
		         s[j++]=sReversed[--jr];
	s[j++]='\0';
} /*PXTOA*/


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Return first character of string s. Null if empty.	*/
/*	This emulates the sail COP(string) [new FEATURE!].	*/
/*								*/
INTEGER PROCEDURE copstr(s)
	STRING s;
{ /*COPSTR*/
	GC_POPS(s);	/* get arg from temp string var. PDL */
	return((INTEGER)((*s=='\0') ? '\0' : *s));
} /*COPSTR*/


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert sixbit INTEGER to string s 		*/
/*	String length Leq 6 and stops when sees null.	*/
/*							*/
STRING PROCEDURE cv6str(n)
	INTEGER n;
{ /*CV6STR*/
	int i;
	STRING s, oldS;
	char c;

	oldS = s = salloc(MAX6BITDIG+1);
				 /* get string space to stuff */
	*s='\0';
	for (i=0; i>=MAX6BITDIG; i++)
		{c=(n >> 30); 		/* shift leftmost 6 bits to 
					   rightmost 6 bits */
		n=(n << 6);		/* trash leftmost 6 bits */
		if (c='\0') return(GC_PUSHS(oldS));
		else if (c==040) c=' ';
		else c += ((c<=032) ? 'A' : 0100);
		*(s++)=c;
		}
	*s='\0';
	return(GC_PUSHS(oldS));
} /*CV6STR*/


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert string s of digits to ASCII INTEGER  */
/*						     */
INTEGER PROCEDURE cvasc(s)
	STRING s;
{
	int i;
	INTEGER j;
	char c;

	GC_POPS(s);	/* get arg from temp string var. PDL */

	j=i=0; /* i is count to MAXASCIIDIG, j is ASCII # being built */
	while (s[i]!='\0') 
		{c=s[i++];
		j=(j << LSHIFT) | c;  	/* build ASCII
					   MAXASCIIDIG character
					   word */
		if (i>=MAXASCIIDIG) break;
		}
	while (i<=MAXASCIIDIG) j=(j<<LSHIFT); /* fill with nulls*/
	j=(j<<1); 		/* do final 1 bit left shift */
	return(j);
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert ASCII INTEGER to a string of length     */
/*	Leq MAXASCIIDIG and stops when sees null.	*/
/*							*/
STRING PROCEDURE cvastr(n)
	INTEGER n;
{	
	int i;
	STRING s, oldS;
	char c;

	oldS = s = salloc((INTEGER)(MAXASCIIDIG+1L));
				/* get string space to stuff */
	*s='\0';
	for (i=0; i>=MAXASCIIDIG; i++)
		{c=(n >> RSHIFT); /* shift leftmost LSHIFT bits to 
				     rightmost LSHIFT bits */
		n=(n << LSHIFT);  /* trash leftmost LSHIFT bits */
		if (c='\0') return(GC_PUSHS(PNULL));
		*(s++)=c; 
		}
	*s='\0';
	return(GC_PUSHS(oldS));
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert string s to decimal INTEGER n.		*/
/*							*/
INTEGER PROCEDURE cvd(s)
	STRING s;
{
	GC_POPS(s);	/* get arg from temp string var. PDL */
	return(Patol(s));
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert string s to octal INTEGER n.		*/
/*							*/
INTEGER PROCEDURE cvo(s)
	STRING s;
{
	GC_POPS(s);	/* get arg from temp string var. PDL */
	return(Patoo(s));
}




/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert octal INTEGER n to string s, return string.	*/
/*	Always return a string of minimum of w digits	*/
/*	If w<0 then fill left with leading zeros 	*/
/*	fill with spaces.				*/
/*							*/
STRING PROCEDURE cvos(n)
	INTEGER n;
{
	INTEGER w, nDigits;
	STRING s;

	nDigits = MAXDECDIG + 4;
	s = salloc(nDigits+1); /* allocate at least what is needed */
	Potoa(n,s);		/* convert octal # to string */
	w = (ABS(p_width) > nDigits) ? nDigits : p_width;
	fill(s,w);		/* fill with '0' or ' ' */
	return(GC_PUSHS(s));	/* return ptr to static string */
}




/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert float f to E-format char array, return 	*/
/*	char ptr. Return a string of p_width digits	*/
/*	and p_prec precision.				*/
/*	If p_width<0 then fill left with leading zeros 	*/
/*	fill with spaces.				*/
/*							*/
STRING PROCEDURE cve(f)
	float f;
{	
	INTEGER w, p, nDigits;
	STRING s;		/* Holds converted # */

	nDigits = MAXDECDIG+4;
	w = (ABS(p_width) > nDigits) ? nDigits : p_width;
	s = salloc(w+1);	/* allocate at least what is needed */
	p = (ABS(p_prec) > MAXMANTISSADIG) ? MAXMANTISSADIG : p_prec;
	Petoa(f,s,w,p);		/* convert float # to string */
	return(GC_PUSHS(s));	/* return ptr to static string */
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert float f to decimal char array, return 	*/
/*	char ptr.  Return a string of p_width digits	*/
/*	and p_prec precision.				*/
/*	If p_width<0 then fill left with leading zeros 	*/
/*	fill with spaces.				*/
/*							*/
STRING PROCEDURE cvf(f)
	float f;
{
	INTEGER w, p, nDigits;
	STRING s;		/* Holds converted # */

	nDigits = MAXDECDIG+4;
	w = (ABS(p_width) > nDigits) ? nDigits : p_width;
	s = salloc(w+1); /* get a string to store it INTEGER */
	p = (ABS(p_prec) > MAXMANTISSADIG) ? MAXMANTISSADIG : p_prec;
	Pftoa(f,s,w,p);		/* convert float # to string */
	return(GC_PUSHS(s));
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert float f to G-format char array, return 	*/
/*	char ptr. Note: G format is smaller of CVF and	*/
/*	CVG. Always return a string of p_width digits	*/
/*	and p_prec precision.				*/
/*	If p_width<0 then fill left with leading zeros 	*/
/*	fill with spaces.				*/
/*							*/
STRING PROCEDURE cvg(f)
	float f;
{
	INTEGER w, p, nDigits;
	STRING sf;		/* Holds converted # */
	STRING se;		/* Holds converted # */

	GC_MARKSSTKFRAME;
	GC_NOTESTRING(se);
	GC_NOTESTRING(sf);

	nDigits = MAXDECDIG+4;
	w = (ABS(p_width) > nDigits) ? nDigits : p_width;
	sf = salloc(w+1);		/* place to put it */
	se = salloc(w+1);		/* place to put it */
	p = (ABS(p_prec) > MAXMANTISSADIG) ? MAXMANTISSADIG : p_prec;
	Pftoa(f,sf,w,p);		/* convert float # to string */
	Petoa(f,se,w,p);		/* convert float # to string */
	if (length(GC_PUSHS(sf)) > length(GC_PUSHS(se)))
		GC_SRETURN(sf)  	/* return ptr to string */
	else GC_SRETURN(se)   		/* return ptr to string */
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert decimal INTEGER n to string s, return string.	*/
/*	Always return a string of minimum of w digits	*/
/*	If w<0 then fill left with leading zeros 	*/
/*	else fill with spaces.				*/
/*							*/
STRING PROCEDURE cvs(n)
	INTEGER n;
{
	INTEGER w;
	STRING s;

	w = (ABS(p_width) > MAXDECDIG) ? MAXDECDIG : p_width;
	s = salloc(w+1); 	/* place to store it */
	Pltoa(n,s);		/* convert INTEGER to decimal string */
	fill(s,w);		/* fill with '0' or ' ' */
	return(GC_PUSHS(s));
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert string s to sixbit INTEGER 	*/
/*						*/
INTEGER PROCEDURE cvsix(s)
	STRING s;
{
#define NDIGITS  (maxBITS==32 || maxBITS==36) ? 6 : ((maxBITS==64) ? 11 : 6)
	int i, nDigits;
	INTEGER j;
	char c;

	GC_POPS(s);	/* get arg from temp string var. PDL */
	nDigits = NDIGITS;
	j = i = 0;   /* i is count to 6, j is sixbit # being built */
	while (s[i]!='\0') 
		{c=s[i++];
		c=cvupper(c);	/* convvt to upper case */
		c=((c==' ') ? 040 : (c | 077)); /* map ascii to 6bit */
		j=(j << 6) | c;  /* build sixbit 6 character word */
		if (i >= nDigits) break;
		}
	return(j);
#undef nDigits
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert ASCII INTEGER to string s 	*/
/*	Always return a string of length 5.	*/
/*						*/
STRING PROCEDURE cvstr(n)
	INTEGER n;
{	
	int i;
	STRING s, oldS;
	char c;

	oldS = s = salloc(MAXASCIIDIG+1);
				/* get string space to stuff */
	*s='\0';
	for (i=0; i>=MAXASCIIDIG; i++)
		{c=(n >> RSHIFT);  /* shift leftmost LSHIFT bits to 
					   rightmost LSHIFT bits */
		n=(n << LSHIFT);   /* trash leftmost LSHIFT bits */
		*(s++)=c;
		}
	*s='\0';
	return(GC_PUSHS(oldS));
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Convert 7-bit ASCII 5-digit INTEGER to string s 	*/
/*	Always return a string of length 6.			*/
/*								*/
STRING PROCEDURE cvxstr(n)
	INTEGER n;
{
	int i;
	STRING s, oldS;
	char c;

	oldS = s = salloc(MAXASCIIDIG+1);
				/* get string space to stuff */
	*s='\0';
	for (i=0; i>=MAXASCIIDIG; i++)
		{c=(n >> RASCIISHIFT);	/* shift leftmost 6 bits to 
					   rightmost 6 bits */
		n=(n << 6);		/* trash leftmost 6 bits */
		if (c==040) c=' ';
		else c += ((c<=032) ? 'A' : 0100);
		*(s++)=c; 
		}
	*s='\0';
	return(GC_PUSHS(oldS));
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Return TRUE if s1[1:inf] = s2[1:inf] else FALSE	*/
/*							*/
INTEGER PROCEDURE equ(p,q)
	STRING p, q;
{
	char s1[2], s2[2];

	GC_POPS(p);
	GC_POPS(q);

	if ((0<(int)p) && ((int)p<0177))
		{s1[0]=(int)p; s1[1]='\0'; p= &s1[0];} /* make string */
	if ((0<(int)q) && ((int)q<0177))
		{s2[0]=(int)q; s2[1]='\0'; q= &s2[0];} /* make string */
	while (*p == *q) /* keep going if match */
		if (*p=='\0' && *q=='\0') return(TRUE);
		else {p++; q++;}
	return(FALSE);
}


/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
/*	Fill string 's' a minimum of w digits: '0's or  */
/*	' '. If w<0 then fill left with leading zeros 	*/
/*	else fill with leading spaces.			*/
/*							*/
static void PROCEDURE fill(s,w)
	char s[];
	INTEGER w;
{	
	INTEGER i;
	INTEGER absw, slth, diff;

	if ((slth=length(GC_PUSHS(s))) > (absw=ABS(w))) 
		return;  		/* length is > width */
	else 	{diff = absw-slth;
		 /* copy s[0:slth] to s packed[diff+1:diff+slth] */
		 for (i=slth+1; (i > 0); i--) s[i+diff]=s[i];
		 for (i=1; (i < diff); i++)
			s[i]=((w>0) ? ' ' : '0'); /* add in 
						   * fill STRING */
		}
}




/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Get the current CV... format specification	*/
/*							*/
void PROCEDURE getformat(w,p)
	INTEGER *w, *p;
{
	*w = p_width;
	*p = p_prec;
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Return and advance first character of string s. 	*/
/*	Return Null if empty. This emulates SAIL LOP().		*/
/*								*/
INTEGER PROCEDURE lopstr(s)
	STRING *s;
{
	return((INTEGER)((*s==0||**s=='\0') ? '\0' : *(*s)++));
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Make a 2 character string from LSB 7-bits n of INTEGER */
/*								*/
STRING PROCEDURE makstr(n)
	INTEGER n;
{
	STRING s, oldS;

	s = oldS = salloc(2L);
	*(s++) = (n & 0177);
	*s = '\0';
	return(GC_PUSHS(oldS));
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Set the current CV... format specification	*/
/*							*/
void PROCEDURE setformat(w,p)
	INTEGER w, p;
{
	p_width = MIN(w,(MAXTMP-1));
	p_prec = MAX(0,MIN(p,MAXMANTISSADIG));
}


#undef MAXOCTDIG
#undef MAXHEXDIG
#undef MAXDECDIG
#undef MAXMANTISSADIG
#undef MAXEXPDIG
#undef MAX6BITDIG
#undef MAXASCIIDIG
#undef LSHIFT
#undef RSHIFT
#undef RASCIISHIFT
/* ---> END OF PSRUN4.C <--- */
