/* 	File: PSRUN5.C 						*/
/*								*/
/*	SAIL RUNTIME  SCAN PROCEDURES				*/
/*								*/
/*								*/
/*	P. Lemkin						*/
/*	Image Processing Section				*/
/*	National Cancer Institute, DCBD, LTB			*/
/*	FCRF, Build 469 Room 150B				*/
/*	Frederick, MD 21701					*/
/*
*    COPYRIGHT NOTICE - Copyright (c) 1983-1987 by Peter F. Lemkin.
*                     All rights reserved.
*       Permission is granted to any individual or institution to
*       copy, use or modify this program, except for explicitly
*       commercial purposes, provided this copyright notice is
*       included in its entirety in all derived works.  No warranty
*       of the software or of the accuracy of any documentation
*       surrounding it is expressed or implied, and the author does
*       not acknowledge any liability resulting from program or
*       documentation errors.
*/
/*								*/
/*	Version: 57.10						*/
/*								*/
/*	Revised October 10, 1987				*/
/*	Revised September 7, 1987				*/
/*	Revised July 1, 1987					*/
/*	Revised June 30, 1987					*/
/*	Revised February 28, 1987				*/
/*	Revised August 12, 1986					*/
/*	Revised August 11, 1986					*/
/*	Revised August 7, 1986					*/
/*	Revised July 15, 1986					*/
/*	Revised July 8, 1986					*/
/*	Revised June 27, 1986					*/
/*	Revised April 28, 1986					*/
/*	Revised April 22, 1986					*/
/*	Revised April 6, 1986					*/
/*	Revised March 29, 1986					*/
/*	Revised March 15, 1986					*/
/*	Revised March 8, 1986					*/
/*	Revised February 28, 1986				*/
/*	Revised January 17, 1986				*/
/*	Revised December 19, 1985				*/
/*	Revised December 13, 1985				*/
/*	Revised December 2, 1985				*/
/*	Revised November 21, 1985 - B.S.			*/
/*	Revised November 21, 1985				*/
/*	Revised November 20, 1985 B.S.				*/
/*	Revised November 5, 1985				*/
/*	Revised October 11, 1985				*/
/*	Revised September 9, 1985				*/
/*	Revised August 6, 1985					*/
/*	Revised June 7, 1985					*/
/*	Revised April 26, 1985					*/
/*	Revised March 8, 1985					*/
/*	June 29, 1984						*/
/*								*/
/*								*/
/*	This procedure package includes the following standard 	*/
/*	runtime functions necessary for easy use of the PSAIL	*/
/*	compiler in a UNIX environment.				*/
/*								*/
/*  5. SCAN procedures: sub-module <psrun5.c>			*/
/*	breakset(tbl,brkchars,mode) - modify break table	*/
/*	getbreak() - get a free break table INTEGER number 1 to 54  */
/*	PgetBrkBit() - test if (brkTbl,chr) chr In brkTbl */
/*	PgetOmitBit() - test if (omitTbl,chr) chr In omitTbl */
/*	intscan(s,*brk) - scan INTEGER # from *s put brk in INTEGER *brk */
/*	realscan(s,*brk) - scan real # from *s put brk in INTEGER *brk */
/*	relbreak(tbl) - release break table number 1 to 54	  */
/*	*scan(*s,tbl,*bChr) -rtn scan'd string *s w/brk tbl, set *bChr*/
/*	*scanc(s,bStr,oStr,mode) - rtn scan'd string explicit table */
/*	setbreak(tbl,brkStr,omitStr,mode) - define break table	*/
/*	stdbrk(tbl) - set break tbl (1 to 18) as in pg 38 [REISER]*/
/*								*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    C O M P I L E   T I M E   M A C R O S              */
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/

#ifndef HASCONFIG
#include "config.h"	/* declare HAS... macros */
#endif


#ifndef PNULL
#include "psrdef.h"	/* need <sairun.c> & <psrung.h> compile-time macros */
#include "psrglb.h"	/* need <sairun.c> global variables */


/*	The following are external definitions in PSRUN1.C	*/
STRING salloc();	  /* return string from dynamic GC storage */

/*	The following are external definitions in PSRUN3.C	*/
float Patof();     /* return float value of string number */
INTEGER Patol();   /* return INTEGER value of string number */
STRING cncat2();   /* return Pmalloc() alloc STRING of (a&b)*/
INTEGER length();  /* return length of string s. */
#endif			/* end of code brought in if compiling alone */

/*	Following are FORWARD references in this module <PSRUN5.c> */
STRING scan();      /* rtn scan'd string *s w/brk tbl, set *bChr*/
void setbreak();    /* define SCAN break table */
void setbrkbit();   /* Set bit (0|1)<==break table t for char c*/
void setomitbit();  /* Set bit (0|1)<==omit table t for char c*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    G L O B A L    V A R I A B L E S / M A C R O S	*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/

static INTEGER  bChar[60*4]={0};  /* Bit map of 128 chars to break on */
INTEGER  PmodeTbl[60]={0};   /* BREAK mode-bits table. Where:
				   * Amode is bit 0 (from right)
				   * Bmode is bit 1 (from right)
					etc.
				   */
static INTEGER  freeBtbl[60]={0};  /* free break tables=0 */
static INTEGER  oChar[60*4]={0};  /* Bit map of 128 chars to omit */
static INTEGER  
	Amode, Fmode, Imode, Kmode, Lmode, Omode, Rmode, Smode,
	Xmode, Zmode;
static INTEGER	fBrkTable=0;	/* internal REALSCAN # break table */
static INTEGER 	iBrkTable=0;	/* Internal INTSCAN # break table */


/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    S C A N        P R O C E D U R E S   		*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	breakset(tbl,brkchars,mode) - modify EXISTING break table */
/*	Simulate the SAIL SETBREAK library function.		  */
/*	It is described in the SAIL manual pages 37-38.		  */
/*								  */
/*	GROUP 0 = Conversion specification			  */
/*		"K" = cvt (a-z) to (A-Z)			  */
/*		"F" = do NOT cvt (a-z) to (A-Z)			  */
/*		"Z" = accept a byte w/value zero ~"I", ~"X")	  */
/*		"L" = cvt (A-Z) to (a-z), PSAIL EXTENSION (B.L.)  */
/*								  */
/*	GROUP 1 = break character specification			  */
/*		"I" = specify by Inclusion			  */
/*		"X" = specify by eXclusion			  */
/*		"O" = specify by Omission			  */
/*								  */
/*	GROUP 2 = break character disposition			  */
/*		"S" = skip(default), put into breakChar variable  */
/*		"A" = append to end of output string.		  */
/*		"R" = retain in front of remaining string	  */
/*								  */
/*	GROUP 3 - handling TOPS10 line #s. NOT portable. ignored! */
/*								  */
void PROCEDURE breakset(tbl,brkchars,mode)
	INTEGER tbl;
	STRING brkchars;
	INTEGER mode;
{
	int i;
	char c;

	GC_POPS(brkchars);	/* get arg from temp string var. PDL */
	if (tbl<1 || tbl>54) return;  /*no-op */
	mode = cvupper(mode);	/* easier to test UPPER case letter */
	if (mode!='X' && mode!='I' && mode!='O') mode='I'; /* default */
	if (mode=='X') for(i=0;i<128;i++)
			 setbrkbit(tbl,i);   /* turn it all on! */
	while ((c = *(brkchars++))!='\0')
		if (mode=='I')  setbrkbit(tbl,c);
		else if (mode=='O') setomitbit(tbl,c);
/*		else if (mode=='X') clearbrkbit(tbl,c); */
}



/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
/*	Clear bit 'c' from break table tabIdx 			*/
/*								*/
static void PROCEDURE clrbrkbit(tabIdx,c)
	INTEGER tabIdx;
	char c;
{
	int wAddr, bAddr;
	INTEGER *bCharP;

	wAddr=(c >> 5) & 03;
	bAddr=c & 037;
	bCharP=(&bChar[0]) + ((tabIdx-1)<<2) + wAddr;
	*bCharP &= ~(01 << bAddr);
}


/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
/*	Clear bit 'c' from omit table tabIdx 			*/
/*								*/
static void PROCEDURE clromitbit(tabIdx,c)
	INTEGER tabIdx;
	char c;
{
	INTEGER wAddr, bAddr, *oCharP;

	wAddr=(c >> 5) & 03;
	bAddr=c & 037;
	oCharP=(&oChar[0]) + ((tabIdx-1)<<2) + wAddr;
	*oCharP &= ~(01 << bAddr);
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	INTEGER getbreak() - get a free break table number 1 to 54 */
/*	If a free table is found, then clear its setbreak entries. */
/*	Return -18 if there are no more free break table numbers*/
/*								*/
INTEGER PROCEDURE getbreak()
{
	INTEGER i, j, k, *pB, *pO;

	for(i=1;i<=54;i++)
		if (freeBtbl[i]==FALSE)
			{freeBtbl[i]=TRUE;
			 pB = &bChar[0] + (k = ((i-1)<<2));
			 pO = &oChar[0] + k;
			 for (j=0; j<4; j++) {*(pB++)=0; *(pO++)=0;}
			 PmodeTbl[i]=0;
			 return(i);
			}
	return(-18);		/* none free. See [Reiser] */
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Get bit 'c' from brk table tabIdx 			*/
/*								*/
int PROCEDURE PgetBrkBit(tabIdx,c)
	INTEGER tabIdx;
	char c;
{
	int wAddr, bAddr;

	wAddr=(c >> 5) & 03;
	bAddr=c & 037;
	return((bChar[((tabIdx-1)<<2)+wAddr] >> bAddr) & 01);
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Get bit 'c' from omit table tabIdx 			*/
/*								*/
int PROCEDURE PgetOmitBit(tabIdx,c)
	INTEGER tabIdx;
	char c;
{
	int wAddr, bAddr;

	wAddr=(c >> 5) & 03;
	bAddr=c & 037;
	return((oChar[((tabIdx-1)<<2)+wAddr] >> bAddr) & 01);
}


/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
/*	Set the global -mode BREAK switches from 'mBits' 	*/
/*	Non-zero is the same as TRUE...				*/
/*								*/
static void PROCEDURE getScanMode(mBits)
	INTEGER mBits;
{
	Amode = mBits & Abit;
	Fmode = mBits & Fbit;
	Imode = mBits & Ibit;
	Kmode = mBits & Kbit;
	Lmode = mBits & Kbit;
	Omode = mBits & Obit;
	Rmode = mBits & Rbit;
	Smode = mBits & Sbit;
	Xmode = mBits & Xbit;
	Zmode = mBits & Zbit;
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Scan an INTEGER from string s, return brk character	*/
/*								*/
INTEGER PROCEDURE intscan(s,brk)
	STRING *s;
	INTEGER *brk;
{
	INTEGER n;
	STRING nbr;

	if (iBrkTable==0)
		setbreak((iBrkTable=56),"0123456789+-",
			" \t\r","XNR");	/* to scan an integer */
	nbr = scan(s,iBrkTable,brk);   /* scan until non integer,
					remember sign, shorten s */
	n = Patol(nbr);		/* scan() returns a dynamic string
				  * which is going to be G.C.ed when 
				  * we get back to the user. If we call
				  * the G.C. in scan() we are ok, it 
				  * will not be called here. So we are 
				  * ok. THE WINDOW OF DESTRUCTION! */
	return(n);
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Scan an integer from string s, return brk character.	*/
/*	Return the BREAK mode bit pattern.			*/
/*								*/
static INTEGER PROCEDURE parseScanMode(cond)
	STRING cond;
{
	char ch;
	INTEGER mBits;

	Amode=Fmode=Imode=Kmode=Omode=Rmode=Smode=Xmode=Zmode=0;
	while ((ch = *(cond++)) != '\0')
	     {if (islower(ch)) ch=cvupper(ch);
	      if (ch=='A') {Amode=Abit; Smode=FALSE; Rmode=FALSE;}
	      else if (ch=='F') {Fmode=Fbit; Kmode=0; Lmode=0;}
	      else if (ch=='I') {Imode=Ibit; Xmode=0; Omode=0;}
	      else if (ch=='K') {Kmode=Kbit; Fmode=0; Lmode=0;}
	      else if (ch=='L') {Lmode=Lbit; Fmode=0; Kmode=0;}
	      else if (ch=='O') {Omode=Obit; Xmode=0; Imode=0;}
	      else if (ch=='R') {Rmode=Rbit; Amode=0; Smode=0;}
	      else if (ch=='S') {Smode=Sbit; Amode=0; Rmode=0;}
	      else if (ch=='X') {Xmode=Xbit; Omode=0; Imode=0;}
	      else if (ch=='Z') Zmode=Zbit;
	     }
	mBits = (Amode | Fmode | Imode | Kmode | Omode | Rmode |
		 Smode | Xmode | Zmode);
	return(mBits);
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Scan a real from string s, return brk character		*/
/*								*/
float PROCEDURE realscan(s,brk)
	STRING *s;
	INTEGER *brk;
{
	float n;
	STRING nbr;

	if (fBrkTable==0)
		setbreak((fBrkTable=56),"0123456789+-.@Ee",
			" \t\r","XNR");	/* to scan an real */
	nbr = scan(s,fBrkTable,brk);   /* scan until non real,
					remember sign, shorten s */
	n = Patof(nbr);
	return(n);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	relbreak() - Release break table number 1 to 54		*/
/*								*/
void PROCEDURE relbreak(tbl)
	INTEGER tbl;
{
	char i;
	if (tbl<1 || tbl>54) return;

	for (i=0; i<=127; i++)
		{clrbrkbit(tbl,i);
		 clromitbit(tbl,i);
		}
	freeBtbl[tbl]=FALSE;
	PmodeTbl[tbl]=0;    /* extern array if 0 means unassigned */
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	scan(*iStr,tbl,*brk) - scan STRING  iStr string w/break */
/*	table (as in SAIL SCAN library function).		*/
/*	set the break character on termination.			*/
/*	NOTE: Group 0 mode Z is not implemented.		*/
/*	@V.46.191 --- NEEDS some WORK ---			*/
/*	NOTE: edit 'R' mode to optimize so may not need salloc()*/
/*	NOTE: tables 1-54 are users, 55-59 are <sairun.c>'s	*/
/*								*/
/*	NOTE: @V.49.89 - if mode is 'S' i.e. skip character	*/
/*	then we can use the original string for the output	*/
/*	string rather than going to salloc() to get more	*/
/*	space - this can be a drastic optimization!!!		*/
/*	For mode 'S' and 'iStr' is				*/
/*		"AB+CD\0"					*/
/*	and the break set is "+", then after scanning it is	*/
/*		"AB\0CD\0"					*/
/*		^    ^						*/
/*		|    |						*/
/*		oPtr iPtr   brk=='+'				*/
/*								*/
/*	@V.53.66 NOTE: If we see a '\<code>' we must convert it	*/
/*	to an ASCII character. [Potential problem of multple 	*/
/*	conversions.]						*/
/*								*/
STRING PROCEDURE scan(iStr,tbl,brk)
	STRING *iStr;
	INTEGER tbl;
	INTEGER *brk;
{
	STRING oStr, oldS;
	int b, n;
	char c;

	GC_MARKSSTKFRAME;
	GC_NOTESTRING(oStr);
	GC_NOTESTRING(oldS);

	*brk='\0';			/* default is null break char */
	if (**iStr=='\0' || tbl<1 || tbl>59) 
		 GC_SRETURN(PNULL)  		/*no-op */
	getScanMode(PmodeTbl[tbl]);	/* set up global mode flags */

	if (Smode) oStr = *iStr; 		/* scan it in place */
	else oStr=salloc(length(GC_PUSHS(*iStr))+1);   /* enough to 
							* copy iStr */
	oldS=oStr;
	while (**iStr != '\0')
		{ /* check next character */
		c = *((*iStr)++);	/* get next character */

		/* If c='\' then expand it to single ASCII char */
		if (c=='\\')
			{ /* '\<code>' to ASCII character */
			c = *((*iStr)++);	/* get next char */
			if (isoctal(c))
				{ /*get octal number*/
				n = c -'0';
				while (isoctal(**iStr))
					n = n*8 + (*((*iStr)++) - '0');
				c = n;   /* use actual character */
				} /*get octal number*/
			else if (c=='n') c=012;
			else if (c=='r') c=015;
			else if (c=='f') c=014;
			else if (c=='t') c=011;
			} /* '\<code>' to ASCII character */

		*brk = c;	/* copy break char in case breaks */

		if ( ((b=PgetBrkBit(tbl,c)) == 0) && Imode ||
	             (Xmode && (b != 0)) ) break;   /* done */

		if (PgetOmitBit(tbl,c) == 0)
			{ /* do NOT omit this character */
			*(oStr++)=((Kmode) ? cvupper(c) 
					: ((Lmode) ? cvlower(c) 
					   : c)); /* build string */
			} /* do NOT omit this character */
		if (Amode)
			*(oStr++) = c;  	/* append break to
						 * end of output str*/
		else if (Rmode)
			{(*iStr)--;		/* retain break in
						 * front of input str*/
			*brk='\0';
			}

		*oStr = '\0'; 		/* end string */
		} /* check next character */

	GC_SRETURN(oldS)	/* front of new string,
				 * note that we advanced iStr */
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	*scanc(s,bStr,oStr,mode) - rtn scan'd string explicit table */
/*	Do SCANC function described in [ReisJ76] page 40.	  */
/*								*/
STRING PROCEDURE scanc(s, bStr, oStr, mode)
	STRING s, bStr, oStr, mode;
{
	INTEGER tbl, brchar;
	STRING rslt;

	GC_POPS(s);
	GC_POPS(bStr);
	GC_POPS(oStr);
	GC_POPS(mode);
	GC_MARKSSTKFRAME;
	GC_NOTESTRING(bStr);
	GC_NOTESTRING(oStr);
	GC_NOTESTRING(mode);
	GC_NOTESTRING(rslt);

	tbl = getbreak();
	setbreak(tbl,bStr,oStr,mode);
	rslt = scan(&s,tbl,&brchar);
	relbreak(tbl);
	GC_SRETURN(rslt)
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	setbreak(tbl,brkStr,omitStr,cond) - define break table	*/
/*	Simulate the SAIL SETBREAK library function.		*/
/*	The condition string is saved and used later in scan()	*/
/*	It is described in the SAIL manual pages 37-38.		*/
/*	NOTE: tables 1-54 are users, 55-59 are <sairun.c>'s	*/
/*								*/
void PROCEDURE setbreak(tbl,brkStr,omitStr,cond)
	INTEGER tbl;
	STRING brkStr, omitStr, cond;
{
	int i;
	INTEGER j;

	GC_POPS(brkStr);
	GC_POPS(omitStr);
	GC_POPS(cond);

	if (tbl<1 || tbl>59) return;  /*no-op */
	PmodeTbl[tbl] = parseScanMode(cond);	/* set -Mode flags
						 * and mBits returned */
	for (i=0; i<=3; i++) 	/* clear omit and break tables */
		{j=((tbl-1)<<2) + i;
		 oChar[j] = 0;
		 bChar[j] = (Xmode) ? -1 : 0;
		}
	while (*brkStr) 
		{if (Imode) setbrkbit(tbl,*brkStr);
		 if (Xmode) clrbrkbit(tbl,*brkStr);
		 brkStr++;
		}
	while (*omitStr) setomitbit(tbl,*(omitStr++));
}



/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
/*	Set bit (0 or 1) from break table tabIdx for char c	*/
/*								*/
static void PROCEDURE setbrkbit(tabIdx,c)
	INTEGER tabIdx;
	char c;
{
	int wAddr,bAddr;
	INTEGER *bCharP;

	wAddr=(c >> 5) & 03;
	bAddr=c & 037;
	bCharP = (&bChar[0]) + ((tabIdx-1)<<2) + wAddr;
	*bCharP |= (01 << bAddr);
}


/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
/*	Set bit (0 or 1) from omit table tabIdx for char c	*/
/*								*/
static void PROCEDURE setomitbit(tabIdx,c)
	INTEGER tabIdx;
	char c;
{
	int wAddr,bAddr;
	INTEGER *oCharP;

	wAddr=(c >> 5) & 03;
	bAddr=c & 037;
	oCharP = (&oChar[0]) + ((tabIdx-1)<<2) + wAddr;
	*oCharP |= (01 << bAddr);
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Set break table number 1 to 12 to std table defined in  */
/*	[Reiser] page 39.		  			*/
/*								*/
void PROCEDURE stdbrk(tbl)
	INTEGER tbl;
{
	int i, k;
	INTEGER *pB, *pO;
	STRING sailID, letts, digs, realDigs;

	if (tbl<0 || tbl>12) return; 	/* forget it! */
	PmodeTbl[tbl]=0;
	pB = &bChar[0] + (k = ((tbl-1)<<4));
	pO = &oChar[0] + k;
	for(i=0;i<=3;i++) 
		{*(pB++) = 0;
		 *(pO++) = 0;
		}

	letts="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
	digs="0123456789";
	realDigs="0123456789+-@Ee.";
	sailID=cncat2(letts,digs);
	switch (tbl){
	case 1: setbreak((INTEGER)1,GC_PUSHS("\012"),GC_PUSHS("\015"),
					GC_PUSHS("INS")); break;
	case 2: setbreak((INTEGER)2,GC_PUSHS("\012"),GC_PUSHS(PNULL),
					GC_PUSHS("INA")); break;
	case 3: setbreak((INTEGER)3,GC_PUSHS("\012\015\040\011\014"),
				GC_PUSHS(PNULL),GC_PUSHS("XNR")); break;
	case 4: setbreak((INTEGER)4,GC_PUSHS(sailID),GC_PUSHS(PNULL),
					GC_PUSHS("INS")); break;
	case 5: setbreak((INTEGER)5,GC_PUSHS(sailID),GC_PUSHS(PNULL),
					GC_PUSHS("INR")); break;
	case 6: setbreak((INTEGER)6,GC_PUSHS(letts),GC_PUSHS(PNULL),
					GC_PUSHS("INR")); break;
	case 7: setbreak((INTEGER)7,GC_PUSHS(digs),GC_PUSHS(PNULL),
					GC_PUSHS("XNR")); break;
	case 8: setbreak((INTEGER)8,GC_PUSHS(digs),GC_PUSHS(PNULL),
					GC_PUSHS("INS")); break;
	case 9: setbreak((INTEGER)9,GC_PUSHS(digs),GC_PUSHS(PNULL),
					GC_PUSHS("INR")); break;
	case 10: setbreak((INTEGER)10,GC_PUSHS(realDigs),
				GC_PUSHS(PNULL),GC_PUSHS("XNR")); break;
	case 11: setbreak((INTEGER)11,GC_PUSHS(realDigs),
			   GC_PUSHS(PNULL),GC_PUSHS("INS")); break;
	case 12: setbreak((INTEGER)12,GC_PUSHS(realDigs),
				GC_PUSHS(PNULL),GC_PUSHS("INR")); break;
	}
}

/* ---> END OF PSRUN5.C <--- */
