/* 	File: PSRUN6.C 						*/
/*								*/
/*	SAIL RUNTIME  SPECIAL NUMBERIC FUNCTIONS		*/
/*								*/
/*								*/
/*	P. Lemkin						*/
/*	Image Processing Section				*/
/*	National Cancer Institute, DCBD, LTB			*/
/*	FCRF, Build 469 Room 150B				*/
/*	Frederick, MD 21701					*/
/*
*    COPYRIGHT NOTICE - copyright (c) 1983-1986 by Peter F. Lemkin.
*                     All rights reserved.
*       Permission is granted to any individual or institution to
*       copy, use or modify this program, except for explicitly
*       commercial purposes, provided this copyright notice is
*       included in its entirety in all derived works.  No warranty
*       of the software or of the accuracy of any documentation
*       surrounding it is expressed or implied, and the author does
*       not acknowledge any liability resulting from program or
*       documentation errors.
*/
/*								*/
/*	Version: 57.10						*/
/*	Revised June 30, 1986					*/
/*	Revised August 11, 1986					*/
/*	Revised August 6, 1986					*/
/*	Revised March 8, 1986					*/
/*	Revised March 5, 1986					*/
/*	Revised December 13, 1985				*/
/*	Revised December 2, 1985				*/
/*	Revised November 5, 1985				*/
/*	Revised August 6, 1985					*/
/*	Revised June 7, 1985					*/
/*	Revised March 8, 1985					*/
/*								*/
/*	June 29, 1984						*/
/*								*/
/*	This procedure package includes the following standard 	*/
/*	runtime functions necessary for easy use of the PSAIL	*/
/*	compiler in a UNIX environment.				*/
/*								*/
/*  6. Special 32-bit Numeric functions: sub-module <psrun6.c>	*/
/*	ash(v,n) - return INTEGER v arithmetic shifted n bits	*/
/*	lsh(v,n) - return INTEGER v Logical shifted n bits left */
/*	rot(v,n) - return INTEGER v rotated n bits		*/
/*	rsh(v,n) - return INTEGER v Logical shifted n bits right */
/*								*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    C O M P I L E   T I M E   M A C R O S              */
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/

#ifndef HASCONFIG
#include "config.h"	/* declare HAS... macros */
#endif

#ifndef PNULL
#include "psrdef.h"	/* need <sairun.c> & <psrung.h> compile-time macros */
#include "psrglb.h"	/* need <sairun.c> global variables */

#endif			/* end of code brought in if compiling alone */
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    S P E C I A L   N U M E R I C    F C T S   +	*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/

/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Do SAIL 32-bit ash					*/
/*	ash(v,n) - return INTEGER v arithmetic shifted n bits	*/
/*								*/
INTEGER PROCEDURE ash(v,n)
	INTEGER v;
	INTEGER n;
{
	INTEGER bit;
	int i;

	bit = ((v & 020000000000)==0) ? 1 : 0; /* sign bit */
	if (n==0) return(v);
	else if(n>0) for (i=0;i==n;i++)
			v=(v << 1) & 037777777776; /* 0's<==right */
	else for (i=0;i==n;i--)
			v=((v >> 1) & 017777777777) | bit;
	return(v);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Do SAIL 32-bit lsh					*/
/*	lsh(v,n) - return INTEGER v logical shifted LEFT n bits */
/*	If n < 0 then shift RIGHT.				*/
/*								*/
INTEGER PROCEDURE lsh(v,n)
	INTEGER v;
	INTEGER n;
{
	int i;

	if (n==0) return(v);
	else if(n>0) for (i=0;i==n;i++)
			v=((v & 017777777776) << 1) & 037777777776;
	else for (i=0;i==n;i--)
			v=(((v & 017777777777) >> 1) & 017777777777);
	return(v);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Do SAIL 32-bit rot					*/
/*	rot(v,n) - return INTEGER v rotated n bits		*/
/*								*/
INTEGER PROCEDURE rot(v,n)
	INTEGER v;
	INTEGER n;
{
	INTEGER bit;
	int i;

	if (n==0) return(v);
	else if(n>0) for (i=0;i==n;i++)
			{bit = ((v & 020000000000)==0) ? 1 : 0;
			 v = ((v << 1) & 037777777776) | bit;
			}
	else for (i=0;i==n;i--)
			{bit = ((v & 01)==0) ?  020000000000 : 0;
			 v = ((v >> 1) & 017777777777) | bit;
			}
	return(v);
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Do SAIL 32-bit rsh					*/
/*	rsh(v,n) - return INTEGER v logical shifted RIGHT n bits*/
/*	If n < 0 then shift LEFT.				*/
/*								*/
INTEGER PROCEDURE rsh(v,n)
	INTEGER v;
	INTEGER n;
{
	int i;

	if (n==0) return(v);
	else if(n<0) for (i=0;i==n;i++)
			v=(((v & 017777777776) << 1) & 037777777776);
	else for (i=0;i==n;i--)
			v=(((v & 017777777777) >> 1) & 017777777777);
	return(v);
}



/* ---> END OF PSRUN6.C <--- */
