/* 	File: PSRUN8.C 						*/
/*								*/
/*	PSAIL RUNTIME  NUMERIC FUNCTIONS			*/
/*								*/
/*								*/
/*	P. Lemkin						*/
/*	Image Processing Section				*/
/*	National Cancer Institute, DCBD, LTB			*/
/*	FCRF, Build 469 Room 150B				*/
/*	Frederick, MD 21701					*/
/*
*    COPYRIGHT NOTICE - Copyright (c) 1983-1987 by Peter F. Lemkin.
*                     All rights reserved.
*       Permission is granted to any individual or institution to
*       copy, use or modify this program, except for explicitly
*       commercial purposes, provided this copyright notice is
*       included in its entirety in all derived works.  No warranty
*       of the software or of the accuracy of any documentation
*       surrounding it is expressed or implied, and the author does
*       not acknowledge any liability resulting from program or
*       documentation errors.
*/
/*								*/
/*								*/
/*	Version: 57.10						*/
/*                                                              */
/*      Revised September 7, 1987                               */
/*      Revised August 12, 1987                                 */
/*      Revised July 23, 1987                                   */
/*	Revised July 1, 1987					*/
/*	Revised June 30, 1987					*/
/*	Revised February 28, 1987				*/
/*	Revised September 26, 1986				*/
/*	Revised September 19, 1986				*/
/*	Revised September 11, 1986				*/
/*	Revised August 15, 1986					*/
/*	Revised August 14, 1986					*/
/*	Revised August 12, 1986					*/
/*	Revised August 11, 1986 - moved from pmath.c to psrun8.c */
/*	Revised August 7, 1986 					*/
/*	Revised July 18, 1986 					*/
/*	Revised June 26, 1986 					*/
/*	Revised May 15, 1986 					*/
/*	Revised April 22, 1986 					*/
/*	Revised March 29, 1986 					*/
/*	Revised March 21, 1986 					*/
/*	Revised March 19, 1986 					*/
/*	Revised March 8, 1986 					*/
/*	Revised March 5, 1986 					*/
/*	Revised February 28, 1986 				*/
/*	Revised January 18, 1986 				*/
/*	Revised December 15, 1985 				*/
/*	Revised August 5, 1984 					*/
/*	Revised May 30, 1984 					*/
/*	May 16, 1984 - pmath.c repackaging			*/
/*	September 28, 1984 - pmath.c repackaging		*/
/*	November 11, 1983  vrsn of midiC stdio.c fcts)		*/
/*	October 21, 1983  vrsn of midiC manexp.c)		*/
/*								*/
/*								*/
/*	This procedure package includes the following standard 	*/
/*	runtime functions necessary for easy use of the PSAIL	*/
/*	compiler in a UNIX environment.				*/
/*								*/
/*	INTRODUCTION						*/
/*	------------						*/
/*	In many applications one does not need to have trig or  */
/*	math functions computed in double precision. This may   */
/*	be important if double precision is not supported in 	*/
/*	hardware and accuracy is not important but quick	*/
/*	calculation is.						*/
/*								*/
/*	This package could be used to advantage in a situation	*/
/*	when the C compiler supports passing float variables as	*/
/*	floats rather than coercing them to doubles. Otherwise	*/
/*	one would be inclined to use the <math.c> package	*/
/*	supplied with your C compiler.				*/
/*								*/
/*	This package includes most of the standard trig and 	*/
/*	math functions normally found in math packages.		*/
/*	Many of the trig functions were transliterated from	*/
/*	the original SAIL MACRO-10 (actually the FAIL assembler)*/
/*	assembly language files 'TRIGS' and 'GOGOL' of 4-3-75.	*/
/*	These routines extract the exponent and mantissa and	*/
/*	then compute the function over a range where the 	*/
/*	approximation is accurate. Additional functions		*/
/*	obtained from various other public domain sources were	*/
/*	added. The original SAIL library polynomial estimation   */
/*	constants were for 36-bits and should be changed for	 */
/*	machines with more precision.				 */
/*								*/
/*	GENERAL REFERENCES TO MATH PACKAGES OR METHODS		*/
/*	----------------------------------------------		*/
/*	A few references to general math packages or methods of	*/
/*	calculating these functions are listed here. In 	*/
/*	addition, particular references to particular papers 	*/
/*	used in computing particular functions are given in 	*/
/*	the function themselves.				*/
/*	1. Abramowitz & Stegun, NBS APS#55, Handbook of		*/
/*	   Mathematical Functions, Nov, 1967. Dover Publ.	*/
/*	   Inc., New York.					*/
/*	2. Rectory, K. (ed), Survey of Applicable Mathematics,	*/
/*	   MIT Press, Cambridge Mass., 1969.			*/
/*	3. An article on Unix C runtimes in BYTE ("The UNIX C	*/
/*	   Compiler in a CP/M Environment, by M. Halfont, BYTE,	*/
/*	   Aug83, pp243-267) gives two references which were 	*/
/*	   used in the development of RELIABLE trig libraries 	*/
/*	   for UNIX.						*/
/*								*/
/*								*/
/*  8. Program control procedures: sub-module <PSRUN8.c>	*/
/*	   EXPONENT MANIPULATION				*/
/*	paprovf(a,b) - compute (a/b) return 0 OK, 1 OVF, 2 UNDF	*/
/*	pflexp(x) - return the INTEGER exponent e in x=f*(2^e)	*/
/*	pflfract(x) - return the float fraction f in x=f*(2^e)	*/
/*	pflval(e,f) - return the float value f*(2^e)		*/
/*	    NUMERIC procedures					*/
/*	pabs(n) - convert INTEGER n absolute integer 		*/
/*	pabsf(f) - convert float f absolute value 		*/
/*	pexp(x) - return exponential (e) to float x  power	*/
/*	plog(x) - return log (base e) of float x		*/
/*	plog10(x) - return log (base 10) of float x		*/
/*	ppowii(i,n) - raise INTEGER i to n-th power. n >= 0	*/
/*	ppowif(f,n) - raise INTEGER i to float f-th power.   */
/*	ppowfi(f,n) - raise float f to INTEGER n-th power. n>=0 */
/*	ppowff(x,y) - raise float x to float y power		*/
/*	prand(x) - return float random number of x		*/
/*	psqrt(x) - return float square root of x		*/
/*								*/
/*	    DEGREE ORIENTED TRIG FUNCTIONS			*/
/*	pdegtorad(d) - return float radian value of degrees	*/
/*	pcosd(d) - return float value of cos of d (degrees)	*/
/*	pradtodeg(r) - return float degree value of radians	*/
/*	psind(d) - return float value of sin of d (degrees)	*/
/*	ptand(d) - return float value of tan of d (degrees)	*/
/*								*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    C O M P I L E   T I M E   M A C R O S              */
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/

#ifndef HASCONFIG
#include "config.h"	/* declare HAS... macros */
#include "confg2.h"	/* declare 2ndary HAS... macros */
#endif

#ifndef PNULL
#include "psrdef.h"	/* need <sairun.c> & <psrung.h> compile-time macros */
#include "psrglb.h"	/* SAIRUN global variables */


/*	The following are found in PSRUN7.C			*/
#ifndef __SABER__
void PSprintf();				/* in PSRUN7.C */
#else
void PSprintf(char *, ...);
#endif

/*	The following are references in PSRUN9.C or <sairun.h>*/
#ifdef HASPMATH
float pcos(); /* return float value of cos of r (radians) */
float psin(); /* return float value of sin of r (radians) */
float ptan(); /* return float value of tan of r (radians) */
#endif
#ifndef HASPMATH	/* to define trig fcts in terms of <math.h> */
#include <math.h>
#define pcos(r) ( (float)cos((double)r) )
#define psin(r) ( (float)sin((double)r) )
#define ptan(r) ( (float)tan((double)r) )
#endif


#endif			/* end of code brought in if compiling alone */

/*	The following are FORWARD references in PSRUN8.C	*/
INTEGER paprovf();   /* (a,b) compute (a/b) return 0=OK 1=OVF 2=UNDF*/
INTEGER pflexp();    /* (x), return INTEGER exponent e in x=f*(2^e) */
FAKEFLOAT pflfract(); /* (x), return float fraction f in x=f*(2^e) */
FAKEFLOAT pflval();   /* (e,f), return float value f*(2^e) */
float pdegtorad();		/* convert radians to degrees */

/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    E X P O N.  M A N I P.  F C T S.		+	*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/

/*								*/
/*	The trig package and other math functions depends	*/
/* on disecting a double number x into an INTEGER exponent	*/
/* e and a float fraction in the range of (-1.0:+1.0).		*/
/* From Gary Knott, extract INTEGER exponent of x:		*/
/*	If x=0 then return excess code 				*/
/* 	else return INTEGER e such that 			*/
/*		x=f*(2^e), 0.5 leq f < 1.0. 			*/
/*								*/
/*	Floating point formats handled				*/
/*	Switch		format					*/
/*	------		------					*/
/*	HASDEC10	S,8-bit exp-128, 27-bit mantissa	*/
/*	HASIEEEFLOAT	S,8-bit exp-127, 23-bit mantissa	*/
/*	HASSKYBOARDFP	S,8-bit exp-127, 23-bit mantissa	*/
/*	HASVAX		S,8-bit exp-128, 23-bit mantissa	*/
/*	HASIBM370	S,7-bit exp-64(hex), 24-bit mantissa	*/
/*	HASMOTOROLAFP	24-bit mantissa, S, 7-bit exp-64(hex)	*/
/*	HASCRAY		S,15-bit exp-16384, 48-bit mantissa	*/


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	paprovf(a,b) - compute (a/b) return 1 OVF, 2 UNDF, 0 OK	*/
/*								*/
INTEGER PROCEDURE paprovf(a,b)
	float a,b;
{
	static povfp;		/* overflow variable */
	povfp = 0;		/* default is OK */


#ifdef HASSARGASSOC
/* #m	JFOV	.+1		; clear PDP10 processor flags */

	(a/b);			/* do trial division */
/* #m	JFOV	.+2		; Skip if over or under flow */
/* #m	  JRST	.APOV1		;No, all is ok */
/* #m	JSP	.TMP1, .+1	;Put processor flags in LH of .TMP1 */
/* #m	TLNN	.TMP1,100	;Don't skip if underflow */
/* #m	  AOS	povfp		;+2, underflow */
/* #m	AOS	povfp		;+1, overflow */
/* #m .APOV1: */
*/
#endif

#ifndef HASSARGASSO
	povfp = ((b == 0.0) && (a != 0.0) ? 1
		: (b == 0.0) && (a == 0.0) ? 2
		: 0);
#endif

/*
 *	+ + +  YOU CAN ADD YOUR OWN CODE IF POSSIBLE. + + +
 *
*/
	return(povfp);
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pflexp(x) - return the INTEGER exponent e in x=f*(2^e)	*/
/*                                                              */
INTEGER PROCEDURE pflexp(x)
	FAKEFLOAT x;
{
	UINTEGER i;

#ifdef HASDEC10
	i = ((x<0) ? (x^0400777777777) : x);	/*  i = abs(x) */
	return( ((((0377000000000 & i) >> -27) & 0377) - 0200) );
#endif
#ifdef HASIEEEFLOAT
	i = ((x<0) ? (x^027737777777) : x);	/*  i = abs(x) */
	return( ((((017740000000 & i) >> -23) & 0377) - 0177) );
#endif
#ifdef HASSKYBOARDFP
	i = ((x<0) ? (x^027737777777) : x);	/*  i = abs(x) */
	return( ((((017740000000 & i) >> -23) & 0377) - 0177) );
#endif
#ifdef HASVAX
	i = ((x<0) ? (x^02003777777) : x);	/*  i = abs(x) */
	return( ((((017740000000 & i) >> -23) & 0377) - 0200) );
#endif
#ifdef HASIBM370
	i = ((x<0) ? (x^020077777777) : x);	/*  i = abs(x) */
	return( ((((017700000000 & i) >> -24) & 0177) - 077) );
#endif
#ifdef HASMOTOROLAFP
	i = ((x<0) ? (x^037777777000) : x);	/*  i = abs(x) */
	return((i & 0177) - 077);
#endif
#ifdef HASCRAY
	i = ((x<0) ? (x^0200001777777777777777) : x); /* i=abs(x) */
	return( ((((0377774000000000000000 & i) >> -48) & 077777) 
							- 077777) );
#endif

PSprintf("\nDRYROT pflexp() - never defined HAS<float-point-type>.\n");
PEXIT;			/* return to system ...*/
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pflfract(x) - return the float fraction f in x=f*(2^e)	*/
/*                                                              */
FAKEFLOAT PROCEDURE pflfract(x)
	FAKEFLOAT x;
{
	UINTEGER i;
	float *fi;		/* so we can synth. float # */

#ifdef HASDEC10
	i = ((x<0) ? (x^0400777777777) : x);	/*  i = abs(x) */
	i = (i & 0400777777777) | 0200000000000;
	fi = &i;	/* put data in float return variable */
	return(*fi);
#endif
#ifdef HASIEEEFLOAT
	i = ((x<0) ? (x^020037777777) : x);	/*  i = abs(x) */
	i = (i & 020037777777) | 010000000000;
	fi = &i;	/* put data in float return variable */
	return(*fi);
#endif
#ifdef HASSKYBOARDFP
	i = ((x<0) ? (x^020037777777) : x);	/*  i = abs(x) */
	i = (i & 020037777777) | 010000000000;
	fi = &i;	/* put data in float return variable */
	return(*fi);
#endif
#ifdef HASVAX
	i = ((x<0) ? (x^020037777777) : x);	/*  i = abs(x) */
	i = (i & 020037777777) | 0100000000000;
	fi = &i;	/* put data in float return variable */
	return(*fi);
#endif
#ifdef HASIBM370
	i = ((x<0) ? (x^020077777777) : x);	/*  i = abs(x) */
	i = (i & 020077777777) | 0100000000000;
	fi = &i;	/* put data in float return variable */
	return(*fi);
#endif
#ifdef HASMOTOROLAFP
	i = ((x<0) ? (x^037777777000) : x);	/*  i = abs(x) */
	i = (i & 037777777000) | 0400;
	fi = &i;	/* put data in float return variable */
	return(*fi);
#endif
#ifdef HASCRAY
	i = ((x<0) ? (x^0200001777777777777777) : x); /* i=abs(x) */
	i = (i & 0377774000000000000000) | 0400000000000000000000;
	fi = &i;	/* put data in float return variable */
	return(*fi);
#endif

PSprintf("\nDRYROT pflfract() - never defined HAS<float-point-type>.\n");
PEXIT;			/* return to system ...*/
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pflval(e,f) - return the float value f*(2^e)		*/
/*                                                              */
FAKEFLOAT PROCEDURE pflval(e,f)
	INTEGER e;
	FAKEFLOAT f;
{	
	FAKEFLOAT x;
	UINTEGER i, ffract, fexp;

#ifdef HASDEC10
	i = ((f<0) ? (f^0400777777777) : f);	/*  i = abs(f) */
	ffract = i & 0400777777777;
	fexp = ((e+0200) << 27);
	x = ffract | fexp;
	return((f>0) ? x : (x | 0400000000000));
#endif
#ifdef HASIEEEFORMAT
	i = ((f<0) ? (f^020037777777) : f);	/*  i = abs(f) */
	ffract = i & 020037777777;
	fexp = ((e+0177) << 23);
	x = ffract | fexp;
	return((f>0) ? x : (x | 0200000000000));
#endif
#ifdef HASSKYBOARDFP
	i = ((f<0) ? (f^020037777777) : f);	/*  i = abs(f) */
	ffract = i & 020037777777;
	fexp = ((e+0177) << 23);
	x = ffract | fexp;
	return((f>0) ? x : (x | 0200000000000));
#endif
#ifdef HASVAX
	i = ((f<0) ? (f^020037777777) : f);	/*  i = abs(f) */
	ffract = i & 020037777777;
	fexp = ((e+0200) << 23);
	x = ffract | fexp;
	return((f>0) ? x : (x | 020000000000));
#endif
#ifdef HASIBM370
	i = ((f<0) ? (f^020077777777) : f);	/*  i = abs(f) */
	ffract = i & 020077777777;
	fexp = ((e+077) << 24);
	x = ffract | fexp;
	return((f>0) ? x : (x | 0200000000000));
#endif
#ifdef HASMOTOROLAFP
	i = ((f<0) ? (f^037777777000) : f);	/*  i = abs(f) */
	ffract = i & 037777777000;
	fexp = ((e+077) << 24);
	x = ffract | fexp;
	return((f>0) ? x : (x | 0200000000000));
#endif
#ifdef HASCRAY
	i = ((f<0) ? (f^0200001777777777777777) : f); /* i=abs(f) */
	ffract = i & 0200001777777777777777;
	fexp = ((e+040000) << 48);
	x = ffract | fexp;
	return((f>0) ? x : (x | 0400000000000000000000));
#endif

PSprintf("\nDRYROT pflval() - never defined HAS<float-point-type>.\n");
PEXIT;			/* return to system ...*/
}

/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    N U M E R I C   F U N C T I O N S		+	*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pabs(n) - Return the absolute integer value of 'n'.	*/
/*								*/
INTEGER PROCEDURE pabs(n)
	INTEGER n;
{
	if (n<0) return( - n);
	else return (n);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pabsf(f) - Return the absolute float value of 'f'.	*/
/*								*/
float PROCEDURE pabsf(f)
	float f;
{
	if (f<0.0) return( - f);
	else return (f);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pexp(x)  -  return exponential (e) to float f power	*/
/*	(From SAIL - GOGOL(pg 63) .EXP runtime for reals)	*/
/*	Range of x is [ - 88.028:+88.028]			*/
/*	If x < - 88.028 then it returns 0.			*/
/*	If x > +88.028 then it returns posINF.			*/
/*	Reduce the range of arg by: exp(x)=2**(x*Log(e) base 2) */
/*			 	= 2**(m+f)			*/
/*	2**m is calculated by algebraically adding m to exponent */
/*	of the result of 2**f. 2**f is calculated as		*/
/*	2**f = 2*(0.5+f*(a+b*f^2 - f-c*(f^2 + d)**-1)**-1	*/
/*	Alternatively, from Rectory...				*/
/*	exp(x) = 1 +x/1! + x^2/2! + x^3/3! + ...		*/
/*	Where: x in [ - inf:+inf]				*/
/*								*/
#define	pa	9.95459578
#define	pb	0.03465735903
#define	pc	617.97226953
#define	pd	87.417497202
float PROCEDURE pexp(x)
	float x;
{ 	
	INTEGER m;
	float expX, f, fSq, twoTof, xLogE;

	if (x==0) expX = 1.0; 
	else if (x< - 88.028) expX = 0.0; 
	else if (x>88.028) expX = posINF; 
	else	{ /*compute it */
		m = pflexp(x);		/* extract exponent */
		xLogE = x*LOGe2;	/* x times LOG(e) base 2 */
		f = pflfract(xLogE);	/* extract exponent */
		fSq = f*f;
		twoTof = 2.0/(0.5 + f*(pa + pb*fSq -f -pc/(fSq + pd)));
		expX = pflval(m,twoTof);
		} /*compute it */
	return(expX);
#undef pa
#undef pb
#undef pc
#undef pd
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	plog(x)  -  return logorithm (base e) of float x 	*/
/*	(From SAIL - GOGOL(pg 63) .LOG runtime for reals)	*/
/*	log(absf(x)) is calcuated by this subroutine and 	*/
/*	returns negINF if x is 0.				*/
/*	log(X) = (E + log2(F))*log(2)				*/
/*	Where X = (F/2)*2^(E+1), and log2(F) is given by	*/
/*	log2(F) = C1*Z + C3*Z^3 + C5*Z^5  -  1/2		*/
/*	and Z = (F - sqrt(2))/(F+sqrt(2))			*/
/*	Alternatively, S&R #4.1.44 for x in [0:1.0] has 	*/
/*	error < 3.0E-8						*/
/*								*/
float PROCEDURE plog(x)
	float x;
{ 	
	float logX, e, f, z, z2, z3, log2f;

	if (x==0) logX = negINF; 
	else if (x==1.0) logX = 0.0;
	else	{ /* compute it*/
		if (x<0) x = -x;	/* Faster than ABS(x) */
	 	e = pflexp(x);		/* extract the exponent */
		f = pflfract(x);		/* extract the fraction */
		z = (f - SQRT2) / (f + SQRT2);
		z2 = z*z;
		log2f = (0.5989786496 * z) + (0.9614706323 * 
			(z3 = z * z2));
		log2f += (2.8853912903 * (z3 * z2)) - 0.5;
		logX = (e + log2f)*LOGe2;
		} /* compute it*/
	return(logX);

}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	plog10(x)  -  return logorithm (base 10) of float x 	*/
/*	Compute as:	log10(x) = log(x)*log(10)		*/
/*								*/
float PROCEDURE plog10(x)
	float x;
{	
	return(LOGe2 * plog(x));
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	ppowii(i,n) - Raise INTEGER i to n'th power for n >= 0	*/
/*								*/
INTEGER PROCEDURE ppowii(i,n)
	INTEGER i,n;
{	
	INTEGER j, p;

	for (p=1,j=1; j<=n; ++j)
	  p *= i;
	return(p);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	ppowif(i,y) - Raise INTEGER i to float x power. 	*/
/*								*/
float PROCEDURE ppowif(i,y)
	INTEGER i;
	float y;
{
	if (y==0.0) 
		return(1.0);
	else return(pexp(y*plog((float)i)));
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	ppowfi(x,n) - Raise float f to INTEGER n'th power for n >= 0 */
/*								     */
float PROCEDURE ppowfi(x,n)
	float x;
	INTEGER n;
{	
	INTEGER j;
	float f;

	for (f=1.0,j=1; j<=n; ++j)
	  f *= x;
	return(f);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	ppowff(x,y) - Raise float x to float y power.	*/
/*							*/
float PROCEDURE ppowff(x,y)
	float x,y;
{	
	if (y==0.0) 
		return(1.0);
	else return(pexp(y*plog(x)));
}




/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	prand(r) - return float random # if r=0. If r is not 	*/
/*	zero the reset the seed to r.				*/
/*	Derived from random number generator routine in BYTE	*/
/*	August, 1983, page 90.					*/
/*	This version returns a positive random number in the	*/
/*	the non-closed interval (0.0 : 1.0). If the seed is	*/
/*	zero then use the previous value. Otherwise, if it	*/
/*	is > 0.0 but < 1.0, use the new seed value.		*/
/*								*/
/*	NOTE: @V.53.05 An alternate random number generator with*/
/*	possibly better uniform distribution in CRAY C-library	*/
/*	manual. The number is computed as a 48-bit integer X	*/
/*	and 31-bits are extracted. Xn+1=(a*Xn + c) Mod m for n>0. */
/*	Where: a =273673163155(8) and c=13(8). Take to 31-bits. */
/*	Problem: slow in emulating 48-bit arithmetic MOD 	*/
/*	function on many machines. But you could change it if	*/
/*	you want.						*/
/*								*/
float PROCEDURE prand(seed)
	float seed;
{
#define pRndA	13849
#define pRndC	25173
	float v;
	INTEGER k;
	static INTEGER oldRndSeed;   /* 32-bit seed for random 
					* number generator */
	if (0.0 < seed && seed < 1.0)
		oldRndSeed = seed * 100000; /* multiply since we will
					     * work with a 32-bit 
					     * integer */
	do { /* COMPUTE non zero v */
		do {oldRndSeed = (oldRndSeed * pRndA + pRndC); 
		    if (oldRndSeed < 0) oldRndSeed = - oldRndSeed;
		   }
		while (oldRndSeed == 0);
	   /* Compute:   v=(oldRndSeed MOD 1.0)		*/
	   k = v = oldRndSeed;	/* compute copy and Integer part */
	   v = v - k;
	   } /* COMPUTE non zero v */
	while (v == 0.0) ;
	return(v);
#undef PRndA
#undef PRndC
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	psqrt(r) - return float value of sqrt of r		*/
/*	From power series of SQRT(1+x) (x in range [-1:+1]	*/
/*	in Rectory, page 683.)					*/
/*								*/
/*								*/
#define q0  1.000000000
#define q1  0.500000000
#define q2  0.125000000
#define q3 (-0.062500000)
#define q4  0.039062500
#define q5 (-0.027343750)
#define q6  0.020507812
#define q7 (-0.016113281)
float PROCEDURE psqrt(r)
	float r;
{
	INTEGER e;
	float f, x, v, oddPower;

	if (r<0.0)  
		{PSprintf("Can not take SQRT of negative number %f",r);
		return(0.0);
		}
	if (r==0.0)
	  return(0.0);
	e = pflexp(r);		/* get integer binary exponent */
	f = pflfract(r);	/* get fraction 0 to 1.0 */
	if (e & 01)
	  {e--; oddPower=SQRT10;}
	else oddPower=1.0;
	x = 1.0 - f;		/* map so can use power series */
	v = q0+x*(q1+x*(q2+x*(q3+x*(q4+x*(q5+x*(q6+x*q7))))));
	return(pflval(e/2,v)*oddPower);
#undef q0
#undef q1
#undef q2
#undef q3
#undef q4
#undef q5
#undef q6
#undef q7
}


/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    D E G R E E   T R I G   F U N C T.		+	*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pcosd(d) - return float value of cos of d (degrees)	*/
/*								*/
float PROCEDURE pcosd(d)
	float d;
{	
	float pdegtorad();		/* forward reference */
	return(pcos(pdegtorad(d)));
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pdegtorad(d) - return float radian value of degrees	*/
/*								*/
float PROCEDURE pdegtorad(d)
	float d;
{	return(d/57.295779);
}




/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pradtodeg(r) - return float degree value of radians	*/
/*								*/
float PROCEDURE pradtodeg(r)
	float r;
{	return(r*57.295779);
}






/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	psind(d) - return float value of sin of d (degrees)	*/
/*								*/
float PROCEDURE psind(d)
	float d;
{	return(psin(pdegtorad(d)));
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	ptand(r) - return float value of tand of r (radians)	*/
/*								*/
float PROCEDURE ptand(r)
	float r;
{	return(ptan(pdegtorad(r)));
}



/* ---> END OF PSRUN8.C <--- */
