/* 	File: PSRUN9.C 						*/
/*								*/
/*	SAIL RUNTIME  PORTABLE TRIG PROCEDURES			*/
/*								*/
/*								*/
/*	P. Lemkin						*/
/*	Image Processing Section				*/
/*	National Cancer Institute, DCBD, LTB			*/
/*	FCRF, Build 469 Room 150B				*/
/*	Frederick, MD 21701					*/
/*
*    COPYRIGHT NOTICE - copyright (c) 1983-1986 by Peter F. Lemkin.
*                     All rights reserved.
*       Permission is granted to any individual or institution to
*       copy, use or modify this program, except for explicitly
*       commercial purposes, provided this copyright notice is
*       included in its entirety in all derived works.  No warranty
*       of the software or of the accuracy of any documentation
*       surrounding it is expressed or implied, and the author does
*       not acknowledge any liability resulting from program or
*       documentation errors.
*/
/*								*/
/*	Version: 57.10						*/
/*								*/
/*	Revised September 7, 1987  			       	*/
/*	Revised August 12, 1987  			       	*/
/*	Revised July 1, 1987  			        	*/
/*	Revised June 30, 1987  			        	*/
/*	Revised September 11, 1986				*/
/*	Revised August 15, 1986					*/
/*	Revised August 14, 1986					*/
/*	Revised August 12, 1986					*/
/*	Revised August 11, 1986 - moved from pmath.c to psrun9.c*/
/*	Revised August 7, 1986 					*/
/*	Revised July 18, 1986 					*/
/*	Revised June 26, 1986 					*/
/*	Revised May 15, 1986 					*/
/*	Revised April 22, 1986 					*/
/*	Revised March 29, 1986 					*/
/*	Revised March 21, 1986 					*/
/*	Revised March 19, 1986 					*/
/*	Revised March 8, 1986 					*/
/*	Revised March 5, 1986 					*/
/*	Revised February 28, 1986 				*/
/*	Revised January 18, 1986 				*/
/*	Revised December 15, 1985 				*/
/*	Revised August 5, 1984 					*/
/*	Revised May 30, 1984 					*/
/*	May 16, 1984 - pmath.c repackaging			*/
/*	September 28, 1984 - pmath.c repackaging		*/
/*	November 11, 1983  vrsn of midiC stdio.c fcts)		*/
/*	October 21, 1983  vrsn of midiC manexp.c)		*/
/*								*/
/*								*/
/*	This procedure package includes the following standard 	*/
/*	runtime functions necessary for easy use of the PSAIL	*/
/*	compiler in a UNIX environment.				*/

/*								*/
/*	INTRODUCTION						*/
/*	------------						*/
/*	In many applications one does not need to have trig	*/
/*	or math functions computed in double precision. This	*/
/*	be important if double precision is not supported in 	*/
/*	hardware and accuracy is not important but quick	*/
/*	calculation is.						*/
/*								*/
/*	This package could be used to advantage in a situation	*/
/*	when the C compiler supports passing float variables as	*/
/*	floats rather than coercing them to doubles. Otherwise	*/
/*	one would be inclined to use the <math.c> package	*/
/*	supplied with your C compiler.				*/
/*								*/
/*	This package includes most of the standard trig and 	*/
/*	math functions normally found in math packages.		*/
/*	Many of the trig functions were transliterated from	*/
/*	the original SAIL MACRO-10 (actually the FAIL assembler)*/
/*	assembly language files 'TRIGS' and 'GOGOL' of 4-3-75.	*/
/*	These routines extract the exponent and mantissa and	*/
/*	then compute the function over a range where the 	*/
/*	approximation is accurate. Additional functions		*/
/*	obtained from various other public domain sources were	*/
/*	added.							*/
/*								*/
/*	GENERAL REFERENCES TO MATH PACKAGES OR METHODS		*/
/*	----------------------------------------------		*/
/*	A few references to general math packages or methods of	*/
/*	calculating these functions are listed here. In 	*/
/*	addition, particular references to particular papers 	*/
/*	used in computing particular functions are given in 	*/
/*	the function themselves.				*/
/*	1. Abramowitz & Stegun, NBS APS#55, Handbook of		*/
/*	   Mathematical Functions, Nov, 1967. Dover Publ.	*/
/*	   Inc., New York.					*/
/*	2. Rectory, K. (ed), Survey of Applicable Mathematics,	*/
/*	   MIT Press, Cambridge Mass., 1969.			*/
/*	3. An article on Unix C runtimes in BYTE ("The UNIX C	*/
/*	   Compiler in a CP/M Environment, by M. Halfont, BYTE,	*/
/*	   Aug83, pp243-267) gives two references which were 	*/
/*	   used in the development of RELIABLE trig libraries 	*/
/*	   for UNIX.						*/
/*								*/
/*  9. Portable trig procedures: sub-module <psrun9.c>		*/
/*	pacos(r) - return float value of acos of r (radians)	*/
/*	pasin(r) - return float value of asin of r (radians)	*/
/*	patan(r) - return float value of atan of r (radians)	*/
/*	patan2(a,b) - return float value of atan of a/b (radians)*/
/*	pcos(r) - return float value of cos of r (radians)	*/
/*	pcosh(x) - return float value of cosh(x)		*/
/*	psin(r) - return float value of sin of r (radians)	*/
/*	psinh(x) - return float value of sinh(x)		*/
/*	ptan(r) - return float value of tan of r (radians)	*/
/*	ptanh(x) - return float value of tanh(x)		*/
/*								*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    C O M P I L E   T I M E   M A C R O S              */
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/

#ifndef HASCONFIG
#include "config.h"	/* declare HAS... macros */
#include "confg2.h"	/* declare 2ndary HAS... macros */
#endif

#ifndef PNULL
#include "psrdef.h"	/* need <sairun.c> & <psrung.h> compile-time macros */
#include "psrglb.h"	/* SAIRUN global variables */


/*	The following are found in PSRUN8.C			*/
/* NOTE: the typedef 'FAKEFLOAT' is used to bit-pick the exp/mantissa 
 * where the assumption is that a float is 32-bits and an INTEGER is
 * 32-bits.
*/
INTEGER paprovf(); /* compute (a/b) return 0 OK, 1 OVF, 2 UNDF*/
INTEGER pflexp(); /* return INTEGER exponent e in x=f*(2^e) */
FAKEFLOAT pflfract(); /* return float fraction f in x=f*(2^e)*/
FAKEFLOAT pflval(); /* return float value f*(2^e)	*/

#endif			/* end of code brought in if compiling alone */
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    R A D I A N    T R I G   F U N C T.	+	*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pacos(r) - return float value of acos of r (radians)	*/
/*	The range of r is [-1.0:+1.0]				*/
/*								*/
float PROCEDURE pacos(r)
	float r;
{
	float patan();	/* forward reference */
	float pacosX;

	if (r>0)
	  pacosX = patan(psqrt(1.0-(r*r))/r);
	else if (r<0) 
	  pacosX = PI + patan(psqrt(1.0-(r*r))/r);
	else pacosX = PIover2;
	return(pacosX);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pasin(r) - return float value of asin of r (radians)	*/
/*	The range of r is [-1.0:+1.0]				*/
/*								*/
float PROCEDURE pasin(r)
	float r;
{	
	float patan();	/* forward reference */

	return(patan(r/psqrt(1.0-(r*r))));
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	patan(r) - return float value of atan of r (radians)	*/
/*	From A&S page 81, formula #4.4.49			*/
/*	Absolute error < 1.0E-5					*/
/*								*/
#define t1   1.0000000000
#define t3  (-0.3333314528)
#define t5   0.1999355085
#define t7  (-0.1420889944)
#define t9   0.1065626393
#define t11 (-0.0752896400)
#define t13  0.0429096138
#define t15 (-0.0161657367)
#define t17  0.0028662257

float PROCEDURE patan(r)
	float r;
{

	float v, v1, x, xSq;

	if (r>1.0) 
	    v = patan(PIover2 - patan(1/r));  /* special case */
	else if (r < 0) v = -patan(-r);	/* recursive call */
	else	{ /* compute it */
		x = ABS(r);	/* abs(r) */
		xSq = x * x;
		v1 = (t11+xSq*(t13+xSq*(t15+xSq*t17)));
		v = x*(t1+xSq*(t3+xSq*(t5+xSq*(t7+xSq*(t9+xSq*(v1))))));
		if (r>0)
		  v = -v;
		} /* compute it */
	return(v);
#undef t1
#undef t3
#undef t5
#undef t7
#undef t9
#undef t11
#undef t13
#undef t15
#undef t17
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	patan2(a,b) - return float value of atan of a/b (radians) */
/*								*/
float PROCEDURE patan2(a,b)
	float a,b;
{
	float v, r;
	INTEGER overFlow;

	v = patan(a/b);
	overFlow = paprovf(a,b); /* test a/b over or underflow flag */

	if (overFlow==0 && b < 0)
	  r=(PI+v);
	else if (overFlow==0 && b>0)
	  r=(v-PI);
	else if (overFlow==1)
	  r=((a >= 0) ? PIover2 : -PIover2);
	else if (overFlow==2) 
		{ /* compute it */
		if (b>=0)
		  r=0.0;
		else r=((a >= 0) ? PI : -PI);
		} /*compute it*/
	else r=v;
	return(r);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pcos(r) - return float value of cos of r (radians)	*/
/*								*/
float PROCEDURE pcos(r)
	float r;
{	float psin();

	return(psin(r+1.5707963));   /*cvt to sin by adding PI/2 */
}




/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pcosh(x) - return float value of cosh of x (radians)	*/
/*	if |f| <= 88.029,					*/
/* 		cosh(x) = 1/2(exp(x) + 1.0/exp(x))		*/
/* 	if |f| > 88.029 and (|f|-ln(2)) <= 88.029,		*/
/* 		cosh(x) = exp(|f|-ln(2))			*/
/* 	if (|f|-ln(2)) > 88.029,				*/
/* 		cosh(x)=posINF					*/
/*								*/
float PROCEDURE pcosh(x)
	float x;
{
	float absX, expX, pcoshX;

	absX = (x<0.0) ? -x : x;

	if (absX <= 88.029)
		{expX=pexp(x);
		pcoshX = 0.5*(expX + (1.0/expX));
		}
	else if ((absX > 88.029) && ((absX-LN2) <= 88.029))
		pcoshX = pexp(absX-LN2);
	else if ((absX-LN2) > 88.029)
		pcoshX = posINF;
	return(pcoshX);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	psin(r) - return float value of sin of r (radians)	*/
/*	From A&S page 76, formula #4.3.97			*/
/*	Absolute error < 2.0E-9					*/
/*								*/
#define s1   1.0000000000
#define s3  (-0.1666666664)
#define s5   0.0083333315
#define s7  (-0.0001984090)
#define s9   0.0000027526
#define s11 (-0.0000000239)
float PROCEDURE psin(r)
	float r;
{	
	float v, x, xSq;

	if ((r<0.1953125E-2) && (r>-0.1955125E-2))
		v = r;   		/* sin(x) = x approximation */
	else	{ /* compute it */
		x = ABS(r);		/* Map to positve angle */
		while (x > twoPI) x -= twoPI;	/* Map to: 0 to +2Pi */
		xSq = x * x;
		v = x*(s1+xSq*(s3+xSq*(s5+xSq*(s7+xSq*(s9+s11*xSq)))));
		} /* compute it */
	return(v);
#undef s1
#undef s3
#undef s5
#undef s7
#undef s9
#undef s11
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	psinh(x) - return float value of sinh of x (radians)	*/
/*	if |f| > 88.029,					*/
/*		sinh(x)=(exp[|f|-ln(2)])*sign(x)		*/
/*	if |f| <= 0.10,						*/
/*		sinh(x)=x+(x**3)/6.0+(x**5)/120.0		*/
/*	for all other values of x,				*/
/*		sinh(x)=1/2[exp(x)-1/exp(x)]			*/
/*								*/
float PROCEDURE psinh(x)
	float x;
{
	float absX, expX, signX, v, x2, x3, x5;

	if (x<0) 
	  {absX = -x; signX = -1.0;} 
	else {absX=x; signX=1.0;}

	if (absX > 88.029)
	  v = ((pexp(absX-LN2))*signX);
	else if (absX <= 0.10)
		{
		x2=x*x;
		x3=x2*x; x5=x3*x2;
		v = (x + (x3/6.0) + (x5/120.0));
		}
	else	{ /* For all other values of x, */
		expX = pexp(x);
		v = (0.5*(expX-(1.0/expX)));
		} /* For all other values of x, */
	return(v);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	ptan(r) - return float value of tan of r (radians)	*/
/*	NOTE: an alternative method could be a separate power	*/
/*	series (such as A&S #4.3.101 with abs error<2.0E-8	*/
/*								*/
float PROCEDURE ptan(r)
	float r;
{
	float s, c, v;

	s = psin(r);
	c = pcos(r);
	if (c==0.0 && s>0.0)
	  v = posINF;
	else if (c==0.0 && s<0.0)
	  v = negINF;
	else if (c==0.0 && s==0.0)
	  v = 0.0;
	else v = (s/c);
	return(v);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	ptanh(x) - return float value of tanh of x (radians)	*/
/* This routine calculates the tanh by the following algorithm: */
/* if |f| < .00034, then tanh(x) = x				*/
/* if |f| > 12.0, then tanh(x) = 1.0*sign(x)			*/
/* if 0.17 <= x < 12.0, then tanh is calculated as		*/
/*	tanh(x) = 1.0 - 2(1.0 + exp(2*x))**-1			*/
/* if .00034 <= x < 0.17, then tanh is calculated as		*/
/* tanh(x) = f/(A+f^2(B+C/(D+f^2)))				*/
/* where f = 4*x*log(E)  (base 2)				*/
/*								*/
#define ha 5.7707801636
#define hb 0.01732867951
#define hc 14.1384514018
#define hd 349.6699888
float PROCEDURE ptanh(x)
	float x;
{
	float ptanhX, f, fSq;

	f = 4*x*LOGe2;
	if (ABS(f) < 0.00034)
	  ptanhX = x;
	else if (ABS(f) > 12.0)
	  ptanhX = 1.0*SIGN(x);
	else if ((0.17 <= x) && (x < 12.0))
		ptanhX = 1.0 - 2/(1.0 + pexp(2*x));
/*	Else for (.00034 <= x) && (x < 0.17), tanh(x) is 	*/
/*	computed as: 						*/
	else 	{fSq = f*f;
		ptanhX = f/(ha + fSq*(hb + hc/(hd + fSq)));
		}
	return(ptanhX);
#undef ha
#undef hb
#undef hc
#undef hd
}

/* ---> END OF PSRUN9.C <--- */
