/* 	File: PSRUNG.C 						*/
/*								*/
/*	SAIL G.C. compatibility package for PSAIL 		*/
/*								*/
char *v_psrung={"psrung@V.57.10"};	/* CURRENT VERSION */
/*								*/
/*	P. Lemkin						*/
/*	Image Processing Section				*/
/*	National Cancer Institute, DCBD, LTB			*/
/*	FCRF, Build 469 Room 150B				*/
/*	Frederick, MD 21701					*/
/*								*/

/*
*    COPYRIGHT NOTICE - Copyright (c) 1983-1986 by Peter F. Lemkin.
*                     All rights reserved.
*       Permission is granted to any individual or institution to
*       copy, use or modify this program, except for explicitly
*       commercial purposes, provided this copyright notice is
*       included in its entirety in all derived works.  No warranty
*       of the software or of the accuracy of any documentation
*       surrounding it is expressed or implied, and the author does
*       not acknowledge any liability resulting from program or
*       documentation errors.
*/
/*	Revised October 10, 1987   				*/
/*	Revised October 4, 1987   				*/
/*	Revised September 7, 1987   				*/
/*	Revised August 12, 1987   				*/
/*	Revised July 1, 1987    				*/
/*	Revised June 30, 1987   				*/
/*	Revised December 21, 1986				*/
/*	Revised November 16, 1986				*/
/*	Revised October 22, 1986				*/
/*	Revised September 20, 1986				*/
/*	Revised September 11, 1986				*/
/*	Revised August 15, 1986					*/
/*	Revised August 14, 1986					*/
/*	Revised August 11, 1986					*/
/*	Revised August 7, 1986					*/
/*	Revised July 14, 1986					*/
/*	Revised July 8, 1986					*/
/*	Revised June 28, 1986					*/
/*	Revised April 6, 1986					*/
/*	Revised March 29, 1986					*/
/*	Revised March 17, 1986					*/
/*	Revised March 15, 1986					*/
/*	Revised March 5, 1986					*/
/*	Revised March 3, 1986					*/
/*	Revised February 28, 1986				*/
/*	Revised January 18, 1986				*/
/*	Revised December 14, 1985				*/
/*	Revised August 8, 1985					*/
/*	Revised June 19, 1985					*/
/*	Revised March 8, 1985					*/
/*	Revised June 30, 1984					*/
/*	September 28, 1984					*/
/*								*/
/*	This procedure package includes the garbage collector 	*/
/*	runtime functions necessary for easy use of the PSAIL	*/
/*	SAIL style string garbage collection in a UNIX environment.*/
/*	Note: if the string garbage collector GC is being used, */
/*	then 'salloc' (found in <sairun.c>) will invoke garbage	*/
/*	collection otherwise the user must handle it - TOO BAD!	*/
/*								*/
/*								*/
/*								*/
/*  1. Storage garbage collection procedures			*/
/* INTEGER gc_init() - do what ever needs to be done to 
				initialize the GC data structures
				required by salloc() and str_GC(). */
/* INTEGER str_GC() - invoke string garbage collector */

/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    C O M P I L E   T I M E   M A C R O S              */
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/

#include "config.h"	/* declare primary HAS... macros */
#include "confg2.h"	/* declare 2ndary HAS... macros */

#define GC_RUN 1     /* to keep <psrdef.h> from requiring <psrung.h> */
#include "psrdef.h"	 /* need <sairun.c> compile-time macros 
			  * includes the SNODE struct defn */
#include "psrglb.h"	 /* need <sairun.c> global variables */

#include "gogtab.h"	 /* to update GC statistics in gogtab[]*/

static INTEGER pgogtab[10+3+256];   /* actual array used by gc_init()
					in call to aryHdr() to setup
					gogtab[] ptr so the gogtab[]
					can be accessed with dynamic
					arrays. */
INTEGER *gogtab;         /* gogtab[0:255] - simulate SAIL GOGTAB */

#define MAXRHO 2048	 /* initial size to alloc for rho in str_GC()*/
#define FUDGEFACTOR 2	 /* initial size to estim. rho in str_GC() */
static INTEGER sizeRho=MAXRHO;	 /* initial size to GC */

static sNodeNbr = 0;            /* SNODE counter during G.C. */

static snode_t *strAvail=0;	/* ptr to linked list of SNODEs */
static snode_t *currentSnode=0;	/* ptr to SNODE at time GC is invoked */

static INTEGER nStrings=0; 	/* number of strings to process*/
static INTEGER nSnodes=0;	/* number of SNODES to process*/

/*      The following procedures are found in module PSRUN1.C */
INTEGER *aryHdr();      /* install the header of a static array */
STRING Pmalloc();       /* Storage allocatior */
void Pfree();           /* storage deallocator */

/*	The following are FORWARD static declarations in PSRUNG.C */
INTEGER binarySearch();	/* binary search */
void chrblt();		/* string block transfer */
INTEGER qsorti();	/* integer quicksort */
void qsort2();		/* another integer quicksort */
INTEGER slength();	/* quick length computation */
INTEGER findSAILptrs();	/* find all STRING ptrs */
INTEGER pushrho();	/* push string ptr and var==>PDL */

/*	The following are FORWARD INTERNAL declarations in PSRUNG.C */
INTEGER gc_init();		/* initialize GC*/
void str_GC();			/* do String garbage collection */
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    G L O B A L    V A R I A B L E S / M A C R O S	*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/

/* NOTE: global variables are defereed in main() module */
/* PSRUNG.C - used by SAIL-style string Garbage Collector */

extern INTEGER gc_sizes[];  /* Magic sizes defined in main().
				* [0]  is GC_STKFSTRMAX,
				* [1] is GC_STRPDLMAX,
				* [2] is GC_ACTIVESIZEMAX.
				* [3]-[9] are reserved for future.
				*/

extern STRING  gc_sRpdl[]; 	/* Temporary String variable stack.
				 * string fcts return values live here.
				 * Of size GC_STRPDLMAX */
extern INTEGER gc_sRtop;	/* index for above */

extern STRING *gc_PDLactive[];  /* Active String Variable Address PDL*/
extern INTEGER gc_topActive; 	/* index to PDL of active string ptrs*/

extern INTEGER gc_sStkF[];  	/* PDL string stack frame */
extern INTEGER gc_sTopF;	/* index for above */

/*	 . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
#define GC_STKFSTRMAX gc_sizes[0]
#define GC_STRPDLMAX gc_sizes[1]
#define GC_ACTIVESIZEMAX gc_sizes[2]
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    G E N E R A L   R O U T I N E S		+	*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/


/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
/*	Binary search for key and return non-zero index if found*/
/*	From VAX-11 Assembly Language Programming. page 112	*/
/*								*/
static INTEGER PROCEDURE binarySearch(num, table, key)
	INTEGER num;
	INTEGER table[];
	INTEGER key;
{ /*BINARYSEARCH*/
	INTEGER first, last, index, middle;

	first=1; last=num; index=0;
	while (index==0 && first<=last)
		{middle = (first+last)/2;
		 if (key==table[middle]) 
		   index = middle;
		 else if (key<table[middle])
		   last = middle-1;
		 else if (key>table[middle])
		   first = middle+1;
		}
	return(index);
} /*BINARYSEARCH*/


/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
/*	chrblt(dest,src,cnt) - block transfer the string at	*/
/*	src[1 to cnt] to dst[1 to cnt]. If the C runtime	*/
/*	has a memcpy() then use it...				*/
/*								*/
static void PROCEDURE chrblt(dest,src,cnt)
	STRING dest, src;
	INTEGER cnt;
{ /*CHRBLT*/
	INTEGER k;

#ifndef HASMEMCPY
	for (k=1; k<=cnt; k++)
	  *(dest++) = *(src++);
#endif
#ifdef HASMEMCPY
	memcpy(dest,src,cnt);
#endif
} /*CHRBLT*/




/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	gc_init() - do what ever has to be done to initialize the GC*/
/*								*/
INTEGER PROCEDURE gc_init()
{ /*GC_INIT*/
int i;
gogtab = aryHdr(pgogtab,1L,0L,255L,"gogtab",'i');  /* setup the gogtab[]
						      dynamic array*/

sizeRho = MAXRHO;	 	/* initial size to GC */
strAvail = (snode_t *)0;	/* Linked list of string spaces */
currentSnode = (snode_t *)0;	/* At time GC is invoked */
nStrings = 0; 			/* number of strings to process*/
nSnodes = 0;	 		/* number of Snodes to process*/
for(i=0;i<=255; i++)
        gogtab[i] = 0;              /* zero out the GOGTAB[0:155]  */
return(TRUE);	/* always succeed */
} /*GC_INIT*/



/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
/*	osort - integer one-way, stackless quicksort from 	*/
/*	Bing-Chao, Huang, Knuth, D.E., A One-way, Stackless 	*/
/*	Quicksort Algorithm, BIT 26(1986),127-130. This algorithm */
/*	avoids recursion and assumes that all of the keys to be	*/
/*	sorted are positive numbers. It is Order 		*/
/*		19NlnN-4.033N+O(logN).				*/
/*	The tuned-up quicksort in Knuth Vol 3 (Sorting and Searching)*/
/*	is	11.667NlnN-1.74N+O(logN).			*/
/*	Which is approximately 60% faster, but requires a stack.*/
/*								*/
/*	[MODIFY] so sort on *r[j] not just r[j].		*/
/*								*/
static INTEGER PROCEDURE osort(n,r)
	INTEGER n;		/* # or elements to be sorted in r[] */
	STRING r[];		/* array of string pointers (RECORDS)*/
{ /*OSORT*/
/*Label oSortStep3;  # --  label to step [3] */
INTEGER
	l,	/* progress indicator */
	j,	/* end of current pivoting region */
	i;	/* gap in current pivoting region */
STRING  p;	/* the current pivot entry*/

union {	INTEGER i;
	STRING sp;
	} cvt;		/* used for pointer conversion */

#define NEGATESTRPTR(x) x=(STRING)(-(INTEGER)((INTEGER *)r[l]))

/*	O.1. [Initialize] Set L_1. (Variable L represents our progress
* through the array; records R1,...,Rl-1 will be in their final form).
*/
(INTEGER)r[n+1] = -1;		/* must cast (STRING *) to (INTEGER)
				 * so can save negative pointer).
				*/

/*	O.2 [Begin pivot operation] If Kl<0, goto step O.6. Otherwise
* set I_L, J_L, R_Rl, K_Kl. [NOTE: K is the record KEY or pivot value
* p and Kl is r[l].]
*/
for(l=1; l<=n; l++)
	{ /* try pivot on r[i] */
	while ((INTEGER *)r[l]> 0)
		{ /* do pivot on r[l] */
		j=l;  i=l;  p=r[l]; 

/*	O.3 [Advance and compare] Increase j by one; then repeat this
* step if K<=Kj. (Since Kn+1 < 0 and K > 0, we must eventually find K >
* Kj).
*/
oSortStep3:
		do
		  j++;
		while (p < r[j]);

/*	O.4 [Rearrange] If Kj<0, set Ri_Rj, i_i+1, Rj_Ri, and go back
* to step O.3. (The invariant relations stated above are being
* maintained).
*/
		if ((cvt.sp=r[j]) && (cvt.i>0))
			{ /* rearrange */
			r[i]=r[j]; i++; r[j]=r[i];
			} /* rearrange */

/*	O.5 [Finish pivot operation] Set Ri_R, Ki_-K, and go back to
* step O.2. (The minus sign indicates that the record is in the final
* position) - (NOTE: WE MAY HAVE A PROBLEM WITH C (STRING *) VALUES
* HERE).
*/
		cvt.sp = p;   cvt.i = -cvt.i;
		p = cvt.sp;		/* Mark as NEGATIVE!!! */
		r[l] = p;		/* WATCH OUT! */
		} /* do pivot on r[l] */

/*	O.6 [Restore the sign.] Set Kl_-Kl, L_L+1. Then if L<=N
* return to step O.2. Otherwise the algorithm terminates.
*/
	cvt.sp = r[l];  cvt.i = -cvt.i;	  /* Put back to normal */
	r[l] = cvt.sp;
	} /* try pivot on r[l] */
} /*OSORT*/



/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
/*	qsorti - integer quicksort from BYTE 8/83 pg 121	*/
/*	Sort on ** ptr not just * ptr.				*/
/*								*/
static INTEGER PROCEDURE qsorti(lo,hi,base)
	INTEGER lo, hi;
	STRING base[];		/* array of string pointers */
{ /*QSORTI*/
	INTEGER i,j;
	STRING pivot, temp;
	if (lo < hi) 
		{for (i=lo, j=hi, pivot=base[hi]; i<j; ) 
			{while ((i<j) && (base[i]<=pivot)) i++;
			while ((j>i) && (base[j]>=pivot)) j--;
			if (i<j) {
				temp = base[i];
				base[i] = base[j];
				base[j] = temp;
				}
			}
		}
	temp = base[i];
	base[i] = base[hi];
	base[hi] = temp;
	qsorti(lo,(i-1),&base[0]);
	qsorti(i+1,hi,&base[0]);
} /*QSORTI*/


/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
/*	qsort2 - STRING pointer quicksort: CACM, J. Bentley, J, 27 */
/*	(1984),April 289-291. Quicksort.2 (page 290)		*/
/*								*/
static void PROCEDURE qsort2(lo, u, x)
	INTEGER lo, u;
	STRING x[];
{ /*QSORT2*/
#define Q_SWAPS(a,b) {qTmp=b; a=b; b=qTmp;}
#define Q_RANDINT(lo,u) (lo+FLOOR(prand(0.0)*(u+1-lo)))
	INTEGER i,
		j,
		k;
	STRING  t,
		st1,
		qTmp;

	if (u > lo)
		{
		st1 = x[lo];
		x[lo] = x[k = Q_RANDINT(lo,u)];
		x[k] = st1;
		i = lo;
		j = u+1;
		t = x[lo];
		while (TRUE)
			{
			do i++; while (x[i]<t);
			do j--; while (x[j]>t);
			if (i<j) break;
			Q_SWAPS(x[i], x[j]);
			}
		Q_SWAPS(x[lo], x[j]);
		}
	qsort2(lo, (j-1), x);
	qsort2(i, u, x);
} /*QSORT1*/



/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
/*	slength(s) - quickly compute length of string s.	*/
/*	NOTE: if we ever change the STRING typedef to a struct	*/
/*	with {INTEGER lth; char *ptr} then we can really speed	*/
/*	things up here by doing a simple lookup - probably with	*/
/*	a macro.						*/
/*								*/
static INTEGER PROCEDURE slength(s)
	STRING s;
{ /*SLENGTH*/
	INTEGER k;
#ifdef HASSTRINGSTRUCT
        return(s.lth);
#endif

#ifndef HASSTRINGSTRUCT
	k=0;
	while (*(s++) != '\0') 
	  k++;                 /* count characters */
	return(k);
#endif
} /*SLENGTH*/


/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    G A R B A G E   C O L L E C T O R		+	*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/


/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
/*	findSAILptrs() - build list in each SNODE's rho[] of all*/
/*      activedynamic SAIL string pointers for each SNODE. 	*/
/*								*/
static INTEGER PROCEDURE findSAILptrs()
{ /*FINDSAILPTRS*/
	INTEGER i,      /* working index of gc_PDLactive[i] */
		j,      /* if string ptr is array then j is working
			   count of string array */
		aSize,  /* if var is str. array then size of array*/
		n;      /* count of string pointers found */
	STRING	*p,     /* ptr to current string variable */
		*pa;    /* ptr to current str. array element variables*/

	n=0;		/* set count of string pointers found */

/*	[1] Push active string variable PDL for string pointers */
	for (i=1; (i<=gc_topActive); i++)  /* active string variables */
		{ /* check active string variable PDL for string ptrs */
		p = gc_PDLactive[i]; 	/* pointer to next var */
		if (*p==0)
			{ /* push ARRAY list of ptrs */
			pa = gc_PDLactive[++i]; 	/* array ptr*/
			aSize = arrinfo((INTEGER *)*pa,0L);  /* array size*/
			for (j=0; j<=aSize; j++)
				pushrho(pa++,&n);	/* next ptr */
			} /* push ARRAY list of ptrs */
		else pushrho(p,&n); 		/* point to var */
		} /* check active string variable PDL for string ptrs */

/*	[2] Push string temporary variable PDL for string pointers */
	for (i=1; (i<=gc_sRtop); i++)
		{ /* string temp var PDL */
		p = &(gc_sRpdl[i]); 		/* point to next var */
		pushrho(p,&n); 			/* point to var */
		} /* string temp var PDL */

/*	[3] Push active string LEAP items for string pointers */
/* **** ADD **** */

/*	[4] Push active record data members for record pointers */
/* **** ADD **** */

	return(n);
} /*FINDSAILPTRS*/


/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
/*	pushRho(ptrToStrPtr,n) - check for valid string space	*/
/*	pointer and then push the variable's address into rho[]	*/
/*	stack. We will use the rho[] list on each SNODE to      */
/*      direct the G.C. in the later phase.			*/
/*								*/
static INTEGER PROCEDURE pushrho(ptr,n)
	STRING *ptr;
	INTEGER *n;
{ /*PUSHRHO*/
	snode_t *cp;         /* active SNODE ptr if found */
	STRING newRho;     /* new rho[] created if ovf old
			      SNODE's rho[] */
	INTEGER i,         /* ??? */
		oldLthRho,
		sNodeIdx;

/* [1] Find SNODE which contains this pinter in which case set
 *     sNodeIdx to the SNODe number.
*/
	i = sNodeIdx = 0;


	for(cp=strAvail; (cp != 0); cp=cp->nextSnode)
		{ /* find SNODE index which contains '*ptr' */
		i++;
		if ((cp->bot <= *ptr) && (*ptr <= cp->top))
			{ /* found it! */
			sNodeIdx=i;
			break;
			} /* found it! */
		} /* find SNODE index which contains '*ptr' */


/* [2] Push active string variable ptr ==> rho[] */
	if (sNodeIdx!=0)
		{ /* Push ptr ==> rho[] */

/* [2.1] Test if would overflow the rho[] list */
		if (cp->topRho++ > (oldLthRho = cp->lthRho))
			{ /* RE-alloc rho - list and copy old one */
			printf(
   " DRYROT String GC 'pushrho()' overflow. Current lthRho=%d\n%s",
						cp->lthRho,
   " Copying rho[lthRho] to rho[lthRho+MAXRHO]\n");
			newRho = Pmalloc(cp->lthRho=(cp->lthRho+MAXRHO));  
							/* expand it */
			if (newRho==0)
				{ /* NO MORE MEMORY - DIE!!!! */
				printf(
   "DOUBLE DRYROT in str_GC! Can't expand 'rho' stack to lthRho=%d\n",
						cp->lthRho);
				PEXIT;
				} /* NO MORE MEMORY - DIE!!!! */
			chrblt(newRho, *cp->rho, oldLthRho);
					 	/* copy old list */
			Pfree((STRING)cp->rho);  /* old rho list */
			strAvail->rho = cp->rho = (STRING *)newRho;
                        sizeRho = MAX(cp->lthRho, sizeRho);  /* global*/
			} /* RE-alloc rho - list and copy old one */

/* [2.2] Push pointer to string variable */
		*(cp->rho + cp->topRho) = (STRING)ptr;
		*n++;
		} /* Push ptr ==> rho[] */

	return(*n);		/* return new count */
} /*PUSHRHO*/


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	str_GC() - invoke string G.C.				*/
/*	Garbage collector using active string pointer PDLS kept	*/
/*	by code generated by PSAIL.				*/
/*	The ring of SNODES is pointed to by strAvail.		*/
/*								*/
/*	The string space is a set of disjoint char arrays pointed to
*	by a doubly-linked list of SNODEs. A SNODE points to the
*	AVAIL string space with (.bot,.next,.top). NOTE that both
*	SNODE and string space is gotten by salloc() from Pmalloc().
*	PSAIL allocations from salloc() must never be return
*	using Pfree() since it will corrupt both Pmalloc() and the
*	SNODEs. However, non-string Pmalloc()'s storage can be returned
*	using corresponding Pfree() calls.
*
*	Let n be # entries in list.
*	Let B be base address of NEXT FREE allocation space.
*	Let lastPtr be POINTER to last space collected
*
*	NOTE: the typedef for SNODE  is defined in <psrdef.h>
*
*       NOTE: the rho[] array is only allocated during G.C.
*
*/
void PROCEDURE str_GC()
{ /*STR_GC*/

	STRING  lastSpace;
	INTEGER i,		/* SNODE number being analyzed */
		lth,		/* length of string being compacted */
		n,		/* number of active string ptrs found
				 * and placed in rho[] PDL.
				*/
		s,		/*  size of compacted string space */
		strSpace,	/* string space index eqv SNODE. ???*/
		t;		/*  size of total string space */
	float	ts;		/* t/s ratio for post SNODE analysis */
	STRING	b; 	        /* working bottom of avail list */
	snode_t	*cp;	        /* working SNODE */


/*	[1] Initialize pointers to SNODE and string space	*/
	currentSnode = strAvail;   /* points to current avail node */
	cp = currentSnode;
	if (nSnodes==0) 
		{ /* No string space allocated */
		  /* Do the initial alloc and fixup */
/* @V.53.93 --- add code .... */
		currentSnode = (snode_t *)Pmalloc(sizeof(snode_t)); 
						 /* make it */
		cp = strAvail = currentSnode;
		cp->prevSnode = cp;		/* point to itself */
		cp->bot = (STRING)Pmalloc(gc_allocsize);
			/* Initial salloc() storage allocation*/
		cp->next = cp->bot;   /* make it a ring point to itself*/
		cp->top = (cp->bot) + gc_allocsize-1;
		cp->nextSnode = 0;    /* i.e. end of list */
		nSnodes = 1;
		return;
		} /* No string space allocated */


/*	[1.1] Build 'rho' list of all active string ptrs.	*/
	strAvail = currentSnode;	/* Linked list of string spaces */
	sizeRho = FUDGEFACTOR * nSnodes * 
			  (gc_topActive + gc_sRtop);	/* heuristic */
	cp->rho = (STRING *)Pmalloc(sizeRho);  /* setup a rho[] stack
						* of string ptrs */
        cp->lthRho = sizeRho;


/*	[1.2] PASS 1. Build sorted list rho[#strSpaces] lists in each
*       SNODE of all active string pointers for each entry is a pointer 
*       to C-string pointer. If multiple pointers to same string,
*	then we need to keep ALL pointers for later update.
*	We must build this list from ALL sources of strings
*	including: (string pointers, string pointer arrays,
*	string datums for LEAP, string record members (if ever 
*	implemented - since we need to keep class definition
*	around), etc.).
*/
	nStrings = n = findSAILptrs(); 	/* sorted list rho[1:n] */

	if (gogtab[DBUGRUNTIMES])
		printf("DEBUG: str_GC() nStrings=%d\n",nStrings);
	gogtab[SGCNUM] = nStrings;		/* number of strings GCed*/
	if (nStrings == 0)
		{ /*ERROR -- */
		printf("DRYROT in 'str_GC()' nStrings=0!\n");
		PEXIT;
		} /*ERROR -- */


/*	[1.2.1] Sort rho[]'s string variables by their contents in */
/*	increasing string space address order. 			   */
	for (cp=strAvail; cp!=0; cp=cp->nextSnode)
          qsorti(1L, cp->topRho, cp->rho);



/*	[2] PASS 2. Compaction pass. Move all strings down to the
*	bottom of allocation space.
*	For each SNODE in turn, compact storage pointed to by that
*	    part of the sorted list which points to areas in memory.
*	    If there is room left in the previous SNODE, then
*	    compact it into that SNODE's allocation before starting
*	    to use the current SNODE's storage. Do not split
*	    strings between SNODEs.
*	[2.1] Check if in trying to compact a string S of size Ns in
*	      SNODE i+1 with size Ni+1 it won't fit in SNODE i with
*	      size Ni (i.e. Ni-Nc < Ns). Where already compacted
*	      strings in SNODE i are total size Nc. Then check if
*			Nc/Ni < 0.75
*	      to see whether it is a candidate for expanding SNODE i.
*	      If not, then just compact string s in SNODE i+1 and
*	      continue. Otherwise, we will replace SNODE i with a new
*	      SNODE i' into which we will be able to compact S.
*	[2.1.1] Get new SNODE i' of size (Ns+Nc) from Pmalloc().
*	[2.1.2] Copy Nc strings in SNODE i to SNODE i'.
*	[2.1.3] Replace SNODE i' for SNODE i in linked SNODE list.
*	[2.1.4] Return old SNODE i and its storage to Pfree().
*	[2.1.5] Compact S in SNODE i+1 to new SNODE i.
*/
        sNodeNbr=0;
	for (cp=strAvail; (cp!=0); sNodeNbr++,cp=cp->nextSnode)
		{ /* process next SNODE */
		b = cp->bot;      /* point to start of this SNODE's
				     associated string space */
		lastSpace = b;
                nStrings = cp->lthRho;
		for (i=1; i<=nStrings; i++)
			{ /* squish i'th string space */
			if ( (lastSpace == *(cp->rho+i)) && 
			     ((**(cp->rho+i)) != 0))
			        {**(cp->rho+i) = **(cp->rho+i-1);
						/* >1 SAIL string
						  * ptr points to
						  * same allocated
						  * string.
						  */
				}

			else if ((lastSpace = *(cp->rho+i)) > b)
				{/* copy string down to dest b*/
				lth = slength(*(cp->rho+i))+1;
				chrblt(b,*(cp->rho+i),lth); 
						/* copy lth characters
						 * down to b.
						*/
				} /* copy string down to dest b*/
			*(cp->rho+i) = b; /* set user string variable
				           * to new allocated string
					   * space location.
					  */
                        lastSpace = b;
			b += lth;   	/* update next free avail ptr 
					 * for this sNodeNbr */
			} /* squish i'th string space */


/*		[3.2] Set current avail ptr for this block 	*/
		cp->next= b;	/* set new AVAIL ptr for this block */
		} /* process next SNODE */

/*	[3] Release (a) all currently unused SNODEs which are
 * overallocated (except for storage attached to each SNODE to be
 * freed) and (b) temporary storage used to sort the pointers.
 * When finished compacting, check on Hoare's
 *	    'overallocation factor' [HoarC74]
 *		T/S = Total-available / Compacted-used.
 *	    If (1.2 < T/S < 1.7) then do nothing.
 *	    If (T/S < 1.2) then add SNODES until T/S is about 1.5.
 * 	    If (T/S > 1.7) then Pfree() SNODES until T/S is about 1.5.
*/
	ts = ((float)t) / ((float)s);   /* total/compacted ratio */
	if (ts < 1.2)
		{ /* Add SNODES to bring T/S up to 1.5 */
if (gogtab[DBUGRUNTIMES])
   printf("DEBUG:str_GC() - Add SNODES to bring T/S==>1.5 not implemented\n");
		} /* Add SNODES to bring T/S up to 1.5 */
	else if (ts > 1.7)
		{ /* Free SNODES to bring T/S down to 1.5 */
if (gogtab[DBUGRUNTIMES])
   printf("DEBUG:str_GC() - Free SNODES to bring T/S==>1.5 not implemented\n");
		Pfree((STRING)cp->rho);	     /* release list of sorted
					      *  string ptrs */
		} /* Free SNODES to bring T/S down to 1.5 */

} /*STR_GC*/


/* ---> END OF PSRUNG.C <--- */
