/* 	File: PSRUNG.H 						*/
/*								*/
/*	SAIL GARBAGE COLLECTOR compatibility package for PSAIL  */
/*								*/
/*								*/
/*        P. Lemkin                                             */
/*        Image Processing Section                              */
/*        National Cancer Institute, DCBD, LTB                  */
/*        FCRF, Build 469 Room 150B			   	*/
/*        Frederick, MD 21701				   	*/
/*								*/
/*	Version: 57.10						*/
/*								*/
/*      Revised October 7, 1987                               */
/*      Revised September 7, 1987                               */
/*      Revised July 23, 1987                                   */
/*      Revised June 30, 1987                                   */
/*      Revised June 27, 1987                                   */
/*	Revised February 28,1987				*/
/*	Revised December 21, 1986				*/
/*	Revised November 24, 1986				*/
/*	Revised October 21, 1986				*/
/*	Revised September 20, 1986				*/
/*	Revised August 3, 1986					*/
/*	Revised July 15, 1986					*/
/*	Revised July 8, 1986					*/
/*	Revised July 1, 1986					*/
/*	Revised June 27, 1986					*/
/*	Revised April 6, 1986					*/
/*	Revised March 16, 1986					*/
/*	Revised March 8, 1986					*/
/*	Revised March 3, 1986					*/
/*	Revised January 25, 1986				*/
/*	Revised December 24, 1985				*/
/*	Revised August 6, 1985					*/
/*	Revised June 19, 1985					*/
/*	Revised March 18, 1985					*/
/*	November 3, 1984					*/
extern char *v_psrung;	/*Version of PSAIL expected */

/*
*    COPYRIGHT NOTICE - Copyright (c) 1983-1987 by Peter F. Lemkin.
*                     All rights reserved.
*       Permission is granted to any individual or institution to
*       copy, use or modify this program, except for explicitly
*       commercial purposes, provided this copyright notice is
*       included in its entirety in all derived works.  No warranty
*       of the software or of the accuracy of any documentation
*       surrounding it is expressed or implied, and the author does
*       not acknowledge any liability resulting from program or
*       documentation errors.
*/
/*								*/
/*	This procedure package includes the garbage collector 	*/
/*	runtime functions necessary for easy use of the PSAIL	*/
/*	SAIL style string garbage collection in a UNIX environment.*/
/*	Note: if the string garbage collector GC is being used, */
/*								*/
#ifndef GC_RUN
#define GC_RUN 1     /* to keep <psrdef.h> from requiring 
			<psrung.h> 2nd time */
#endif

#ifndef HASCONFIG 	/* primary configurations */
#include "config.h"  /* get it from this area when compile runtimes! */
#endif
/*								*/
/*  1. Storage garbage collection procedures			*/
INTEGER gc_init(); /* do what ever needs to be done to
			     initialize the GC */
INTEGER *str_GC(); /* invoke string garbage collector for
			   * string allocation system
			  */

/*	******************************************************	  */
/*	STRING G.C. MACROS FOR IMPLEMENTING PSAIL GENERATED CALLS */
/*	******************************************************	  */

#ifndef GC_STRPDLMAX
	/* This was defined in main(). If not then add the externs */
	/* for compiling <sairun.c> 				   */
/*
.LABEL [<GCRUN-H-EXTERNS>]
*/
/*	These variables are defined in PAIL's main()   	   */
extern INTEGER gc_sizes[];  /* Magic sizes defined in main().
				* [0] is GC_STKFSTRMAX,
				* [1] is GC_STRPDLMAX,
				* [2] is GC_ACTIVESIZEMAX.
				* [3]-[9] are reserved for future.
				*/

extern STRING  gc_sRpdl[]; 	/* Temporary String variable stack.
				 * string fcts return values live here.
				 * Of size GC_STRPDLMAX */
extern INTEGER gc_sRtop;	/* index for above */

extern STRING *gc_PDLactive[];	/* Active String Pointer Address PDL 
				 * of size GC_ACTIVESIZEMAX*/
extern INTEGER gc_topActive;	/* index for above */

extern INTEGER gc_sStkF[];	/* Active String Pointer stk frame stk
				 * of size GC_STKFSTRMAX */
extern INTEGER gc_sTopF;	/* index for above */
/*
.ENDLABEL
*/
#endif

/*
.LABEL [<GCRUN-H-FIGS>]
 *	Relationship of various String Garbage Collector PDL's.
 *	------------------------------------------------------



          	TEMPORARY STRING VARIABLE PDL
	        -----------------------------
                STRING gc_sRpdl[0:GC_SPDLMAX]
             -----------------------------------------
           1 |  ptr to dynamic or constant string    |--->string
             -----------------------------------------
           2 |  ptr to dynamic or constant string    |--->string
             -----------------------------------------
             |             . . . etc. . . .          |--->string
             -----------------------------------------
  gc_sRtop-->|  ptr to dynamic or constant string    |--->string
             -----------------------------------------
             |              free                     |
GC_SPDLMAX-->|            . . . .                    |
             -----------------------------------------



          	ACTIVE STRING VARIABLE POINTER PDL
	        ----------------------------------
                      STRING *gc_PDLactive[0:GC_ACTIVESIZEMAX]
                   -----------------------------------------
                 1 |  ptr to string variable               |---> &v1
                   -----------------------------------------
                 2 |  ptr to string variable               |---> &v2
                   -----------------------------------------
                   |             . . . etc. . . .          |---> {&vj}
                   -----------------------------------------
    gc_topActive-->|  ptr to string variable               |--->& vk
                   -----------------------------------------
                   |              free                     |
GC_ACTIVESIZEMAX-->|            . . . .                    |
                   -----------------------------------------


          	STRING BLOCK STRUCTURED STACK FRAME PDL
	        ---------------------------------------
                      INTEGER gc_sStkF[0:GC_STKFSTRMAX]
                   -----------------------------------------
                 1 |  block 1's gc_topActive index         |
                 2 |  block 1's gc_sRtop index             |
                   -----------------------------------------
                 3 |  block 2's gc_topActive index         |
                 4 |  block 2's gc_sRtop index             |
                   -----------------------------------------
                   |             . . . etc. . . .          | 
                   -----------------------------------------
    (gc_sTopF-1)-->|  block k's gc_topActive index         |
        gc_sTopF-->|  block k's gc_sRtop index             |
                   -----------------------------------------
                   |              free                     |
   GC_STKFSTRMAX-->|            . . . .                    |
                   -----------------------------------------


.ENDLABEL
*/
/* .................................................. */
/*
.LABEL [<GCRUN-H-MACROS>]
*/

/*
 * The following macros operate on the Temporary String Variable PDL
 * which passes a cast (STRING *) as a STRING and so we must uncast it
 * when the arg is 'pop'ed and the gc_sRtop also needs to be adjusted.
 * The C compiler must support the K&R Left-to-right COMMA operator
* Section 7.15 in K&R.
*/
#define GC_PUSHS(x)  ((gc_sRpdl[++gc_sRtop] = (x)), \
		      (STRING)(&gc_sRpdl[gc_sRtop]))  /* temp String PDL */

#define GC_PUSHAS(x) ((gc_sRpdl[++gc_sRtop] = (STRING)(x)), \
                      (STRING)(&gc_sRpdl[gc_sRtop])) /* temp Str array PDL */

#define GC_EPOPS(x)  ((STRING)(x), gc_sRpdl[gc_sRtop--]) \
                                         /* Eval (x) then pop \
			 		  * temp string PDL */

#define GC_EPOPAS(x) ((STRING *)(x), (STRING *)gc_sRpdl[gc_sRtop--])\
                                         /* Eval (x) and \
                                          * then return temp string arry PDL */

#define GC_POPS(x)  (gc_sRtop--, x = *((STRING *)x)) /* temp string PDL*/

#define GC_POPAS(x) (gc_sRtop--, x = (STRING *)(*((STRING *)x))) \
                                         /* temp string arry PDL */

#define GC_GETARGS(r,n) r = gc_sRpdl[gc_sRtop-(n)]  /* temp string PDL*/

#define GC_SADJN(n) gc_sRtop -= (n)      /* adjust temp string PDL */


/*
 * The following macros push the names of active string variables
 * into the Active String Variable Pointer PDL.
*/
#define GC_NOTESTRING(x) gc_PDLactive[++gc_topActive] = &x

#define GC_NOTEASTRING(x) (gc_PDLactive[++gc_topActive] = (STRING *)0, \
			    gc_PDLactive[++gc_topActive] = (STRING *)&x)

/*
 * The following macros operate on the Stack Frame PDL which notes
 * positions on Temp String Var PDL and Active String Pointer PDL.
*/
#define GC_MARKSSTKFRAME (gc_sStkF[++gc_sTopF] = gc_topActive, \
			  gc_sStkF[++gc_sTopF] = gc_sRtop)

#define GC_UNMARKSSTKFRAME (gc_sRtop = gc_sStkF[gc_sTopF--], \
			   gc_topActive = gc_sStkF[gc_sTopF--])

#define GC_ADJSTKFRAME(n) (gc_sRtop = gc_sStkF[gc_sTopF -= 2*((n)-1)], \
			   gc_topActive = gc_sStkF[gc_sTopF--])

/*	-- Used in place of C 'return' if strings are involved --
*/
#define GC_SRETURN(x) { \
	GC_UNMARKSSTKFRAME; 	/* remove loc ptrs */ \
	GC_PUSHS(x); 		/* stack onString return pdl */ \
	return(gc_sRpdl[gc_sRtop]);   /* return ptr to STRING PDL */ \
	}

#define GC_NRETURN(x) {GC_UNMARKSSTKFRAME; return(x);}

#define GC_RETURN {GC_UNMARKSSTKFRAME; return;}
/*
.ENDLABEL
*/

  extern INTEGER *gogtab;      /* pointer to aryHdr for gogtab[0:255] */


/* ---> END OF PSRUNG.H <--- */
