

/* 	File: PSRUNP.C 						*/
/*								*/
/*	SAIL RUNTIME  PORTABLE TRIG PROCEDURES			*/
/*								*/
/*								*/
/*	P. Lemkin						*/
/*	Image Processing Section				*/
/*	National Cancer Institute, DCBD, LTB			*/
/*	National Institutes of Health				*/
/*      FCRF Bld 469 Rm 150B                                    */
/*      Frederick, MD 21701                                     */
/*                                                              */
/*      Revised October 10, 1987                               */
/*      Revised September 8, 1987                               */
/*      Revised July 1, 1987                                    */
/*      Revised June 30, 1987                                   */
/*      Revised January 26, 1987                                */
/*
*    COPYRIGHT NOTICE - Copyright (c) 1983-1987 by Peter F. Lemkin.
*                     All rights reserved.
*       Permission is granted to any individual or institution to
*       copy, use or modify this program, except for explicitly
*       commercial purposes, provided this copyright notice is
*       included in its entirety in all derived works.  No warranty
*       of the software or of the accuracy of any documentation
*       surrounding it is expressed or implied, and the author does
*       not acknowledge any liability resulting from program or
*       documentation errors.
*/
/*								*/
/*	Version: 57.10						*/

/*								*/
/*								*/
/*	This procedure package includes the following standard 	*/
/*	runtime functions necessary for easy use of the PSAIL	*/
/*	compiler in a UNIX environment.				*/

/*								*/
/*	INTRODUCTION						*/
/*	------------						*/
/*								*/

/* P. PROCESSES, EVENTS and INTERRUPTS package */
/* INTEGER answer()  - wake up procedure */
/* void apply()      - apply procedures to arglist */
/* ITEM arg_list()   - return variable length arg list item */
/* void asktnc()     - interrogate event type */
/* void assign()     - assign procedure name to item */
/* ITEM caller()     - return calling process of process item arg */
/* void cause()      - eval wait queue */
/* void cause1()     - wakeup process */
/* void dfrint()     - queue deferred interrupt */
/* void dfr1in()     - record deferred interrupt pointer */
/* ITEM interrogate() - examine event queue */
/* void intpro()     - interrupt process */
/* void intset()     - sprout interrupt process intpro() */
/* void inttbl()     - change size of deferred interrupt queue */
/* void join()       - wait until all process items terminate */
/* void mkevtt()     - return item as event notice type item */
/* ITEM myproc()     - return process item it is being executed in */
/* void priset()     - set priority of process */
/* INTEGER pstatus() - return scheduling state of process */
/* ITEM ref_item()   - set type of item to 'reference' */
/* ITEM resume()     - restore suspended process */
/* void setcp()      - associate process with 'cause' event type */
/* void setip()      - associate process with 'interrogate' event type */
/* void sprout()     - fork a process */
/* ITEM suspend()    - suspend a running process */
/* void terminate()  - terminate a running process */
/* void urschd()     - force SAIL process scheduler to schedule */

/*								*/

/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    C O M P I L E   T I M E   M A C R O S              */
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/

#include "config.h"	/* declare HAS... macros */
#include "confg2.h"	/* declare 2ndary HAS... macros */

#include "psrdef.h"	/* need <sairun.c> & <psrung.h> compile-time macros */
#include "psrglb.h"	/* SAIRUN global variables */


/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*      + PROCESSES, EVENTS and INTERRUPTS              +       */
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/* NOTATION:
*   et = event type ITEM
*   en = event notice ITEM
*   i  = ITEM or ITEMVAR
*   le = ITEM or ITEMVAR expression
*   o  = INTEGER option word
*   pi = process ITEM
*   pn = procedure (i.e. function) name
*   py = INTEGER priority word
*   si = send ITEM
*/



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* answer(et,en,pn) - wake up procedure                         */
/*								*/
INTEGER PROCEDURE answer(et,en,pn)
	ITEM et;
        ITEM en;
        void (*pn)();
{
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* apply(pn,le) - apply procedure to arglist                    */
/*								*/
void PROCEDURE apply(pn,le)
        void (*pn)();
        ITEMVAR le;
{
}

/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* arg_list(le) - return variable arg list item                 */
/*								*/
ITEMVAR PROCEDURE arg_list(le)
	ITEM *le;
{
ITEMVAR rList;
return(rList);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* asktnc(et,o) - wake up procedure                         */
/*								*/
ITEMVAR PROCEDURE asktnc(et,o)
	ITEM et;
        INTEGER o;
{
ITEMVAR rEventType;
return(rEventType);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* assign(pi,pn) - assign procedure name to item             */
/*								*/
void PROCEDURE assign(pi,pn)
        ITEMVAR pi;
        void (*pn)();
{
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* caller(pi) - return calling process of process item arg      */
/*								*/
ITEMVAR PROCEDURE caller(pi)
        ITEMVAR pi;
{
ITEMVAR rProcessItem;
return(rProcessItem);

}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* cause(et,en,o) - eval the wait queue.                        */
/*								*/
void PROCEDURE cause(et,en,o)
	ITEM et;
        ITEM en;
        INTEGER o;
{
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* cause1(et,en,pn) - wakeup process                            */
/*								*/
void PROCEDURE cause1(et,en,pn)
	ITEM et;
        ITEM en;
        void (*pn)();
{
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* dfrint() - queue deferred interrupt                          */
/*								*/
void PROCEDURE dfrint()
{
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* dfr1in(AOBJN_ptrOf_INTMAP) - record deferred interrupt pointer */
/*								*/
void PROCEDURE dfr1in(AOBJN_ptrOf_INTMAP)
        void (*AOBJN_ptrOf_INTMAP)();
{
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* interrogate(et,o) - examine event queue                  */
/*								*/
ITEMVAR PROCEDURE interrogate(et,o)
	ITEM et;
        INTEGER o;
{
  ITEMVAR evtQueue;
return(evtQueue);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* intpro() - interrupt process currently executing in.         */
/*								*/
void PROCEDURE intpro()
{
}




/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* intset(pi,o) - sprout interrupt process intpro()             */
/*								*/
void PROCEDURE intset(pi,o)
        ITEMVAR pi;
        INTEGER o;
{
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* inttbl(new_size) - change size of deferred interrupt queue   */
/*								*/
void PROCEDURE inttbl(new_size)
        INTEGER new_size;
{
}




/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* join(setOfProcessItems) - wait until all process items terminate */
/*								*/
void PROCEDURE join(setOfProcessItems)
        SET setOfProcessItems;
{
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* mkevtt(i) - return item as event notice typed item.          */
/*								*/
void PROCEDURE mkevtt(i)
        ITEMVAR i;
{
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* myproc() - return process item it is being executed in.      */
/*								*/
ITEMVAR PROCEDURE myproc()
{
ITEMVAR processItem;
return(processItem);
}




/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* priset(pi,py) - set priority of process.                     */
/*								*/
void PROCEDURE priset(pi,py)
        ITEMVAR pi;
        INTEGER py;
{
}




/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* pstatus(pi) - return scheduling state of process.            */
/*								*/
INTEGER PROCEDURE pstatus(pi)
	ITEM pi;
{
  INTEGER statusState;
  return(statusState);
}




/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* ref_item(ie) - set type of item to 'reference'               */
/*								*/
ITEMVAR PROCEDURE ref_item(ie)
	ITEM ie;
{
ITEMVAR iv;
return(iv);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* resume(pi,si,o) - restore suspended process.                 */
/*								*/
ITEMVAR PROCEDURE resume(pi,si,o)
	ITEM pi;
        ITEM si;
        INTEGER o;
{
ITEMVAR susPro;
return(susPro);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* setcp(et,pn) - associate process with 'cause' event type     */
/*								*/
void PROCEDURE setcp(et,pn)
	ITEM et;
        void (*pn)();
{
}




/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* setip(et,pn) - associate process with 'interrogate' event type*/
/*								*/
void PROCEDURE setip(et,pn)
	ITEM et;
        void (*pn)();
{
}




/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* sprout(pi,pn,o) - fork a process.                            */
/*								*/
void PROCEDURE sprout(pi,pn,o)
	ITEM pi;
        void (*pn)();

        INTEGER o;
{
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* suspend(pi) - suspend a running process                      */
/*								*/
ITEMVAR PROCEDURE suspend(pi)
	ITEM pi;
{
ITEMVAR runPro;
return(runPro);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* terminate(pi) - terminate a running process.                 */
/*								*/
void PROCEDURE terminate(pi)
	ITEM pi;
{
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* ursched() - force SAIL process scheduler to schedule         */
/*								*/
void PROCEDURE ursched()
{
}


/* ===> End of psrunP.c <=== */
