/* 	File: PSRUNW.C 						*/
/*								*/
/*	SAIL RUNTIME  SPECIAL NUMBERIC FUNCTIONS		*/
/*								*/
/*								*/
/*	P. Lemkin						*/
/*	Image Processing Section				*/
/*	National Cancer Institute, DCBD, LTB			*/
/*	FCRF, Build 469 Room 150B				*/
/*	Frederick, MD 21701					*/
/*
*    COPYRIGHT NOTICE - copyright (c) 1983-1987 by Peter F. Lemkin.
*                     All rights reserved.
*       Permission is granted to any individual or institution to
*       copy, use or modify this program, except for explicitly
*       commercial purposes, provided this copyright notice is
*       included in its entirety in all derived works.  No warranty
*       of the software or of the accuracy of any documentation
*       surrounding it is expressed or implied, and the author does
*       not acknowledge any liability resulting from program or
*       documentation errors.
*/
/*								*/
/*	Version: 57.10						*/
/*								*/
/*      Revised September 8, 1987                               */
/*      Revised September 7, 1987                               */
/*      Revised July 23, 1987                                   */
/*	Revised June 30, 1987 				        */
/*	Revised September 26, 1986 				*/
/*	Revised September 19, 1986 				*/
/*	Revised September 11, 1986 				*/
/*	Revised August 15, 1986 				*/
/*	Revised August 12, 1986 - set up psrunW.c		*/
/*	Revised April 6, 1986					*/
/*	Revised March 16, 1986					*/
/*	Revised March 17, 1986					*/
/*	Revised March 8, 1986					*/
/*	Revised March 5, 1986					*/
/*	Revised May 30, 1984					*/
/*	November 3, 1984					*/
/*								*/
/*	This procedure package includes the following standard 	*/
/*	runtime functions necessary for easy use of the PSAIL	*/
/*	compiler in a UNIX environment.				*/
/*								*/
/*	Profile summary procedure				*/
/*	w_profil_print(title) - print the title and profile summary */
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    C O M P I L E   T I M E   M A C R O S              */
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/


#include "config.h"	/* declare HAS... macros */

#include "psrdef.h"	/* need <sairun.c> & <psrung.h> compile-time macros */
#include "psrglb.h"	/* need <sairun.c> global variables */


/*	The following are found in PSRUN3.C			*/
INTEGER length();    /* return length of string s. */
STRING subsr();      /* return STRING s[a For b] substring */
STRING catlist();    /* return STRING concatenation of list of strings */

/*	The following are found in PSRUN7.C			*/
INTEGER setformat(); /* set default field width & precision */
#ifndef __SABER__
void PSprintf();		/* PSAIL printf() type function */
#else
void PSprintf(char *, ...);
#endif

#define ISRUNW_C 1       
#include "psrunw.h"	    /* needs P_MAX_PROFILES */

INTEGER w_time[P_MAX_PROFILES]={0};	/* array to hold the times */
STRING w_proc[P_MAX_PROFILES]={0};	/* array to hold the names */
INTEGER w_count[P_MAX_PROFILES]={0};	/* array to hold the counts */
INTEGER w_startCpuTime=0;		/* when first start up */
INTEGER w_stopCpuTime=0;		/* when finally stop */
INTEGER w_idxFree=0;			/* next free procedure index */

#define CLKTICS 1000.0	/* floating # clock ticks/second */

/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/* 	w_profil_print(title) - print the title and then print the */
/*	sorted list of procedure names, total time, %total time,   */
/*	time/call and #times called. The list is sorted by maximum */
/*	time used. 						   */
/*								   */
void w_profil_print(title) 
	STRING title;
{ /* w_profil_print */
	STRING s, s1, spaces;
	INTEGER i, j, k, maxProcNameLth;
	float tTime, cpuTime, cnf;

/*	[0] Trap final system time			*/
	spaces = "                                          ";
	tTime = 0.0;
	w_stopCpuTime = callsys(0,"RUNTIM");
	cpuTime=(w_stopCpuTime-w_startCpuTime)/CLKTICS;  /* in seconds */

/*	[1] Bubble sort the list of (names, counts and times) by */
/*	    maximum times.					 */
	maxProcNameLth = 0;
	for(i=w_idxFree-1; i<=1; i--)
		{ /* find max length and total accounting time */
		if (k=length(w_proc[i]) > maxProcNameLth)
			maxProcNameLth = k;   /* compute it */
		tTime += (float)w_time[i];  /* total accounting time */
		} /* find max length and total accounting time */
	tTime = tTime/CLKTICS;	/* so in seconds */

	for(i=w_idxFree-1; i<=1; i--)
		{ /*outer loop */
		if (k=length(w_proc[i]) > maxProcNameLth)
			maxProcNameLth = k;   /* compute it */
		for(j=w_idxFree-1; j<=1; i--)
			if (w_time[j] < w_time[j+1])
				{ /* swap j and j+1 */
				s=w_proc[j+1];
				w_proc[j+1]=w_proc[j];
				w_proc[j]=s;

				k=w_time[j+1];
				w_time[j+1]=w_time[j];
				w_time[j]=k;

				k=w_count[j+1];
				w_count[j+1]=w_count[j];
				w_count[j+1]=k;
				} /* swap j and j+1 */;
		} /*outer loop */;

/*	[2] Compute and print the header.		*/
	cnf=(tTime/cpuTime);	/* Complexity Nesting Factor */

	s = catlist(4L,GC_PUSHS("\n--- WATCH: "),
		    GC_PUSHS(title),GC_PUSHS(
    "\nPROCEDURE accounting system was on with the following times\n"),
		    GC_PUSHS("given with \n"));

	setformat((INTEGER)0,(INTEGER)2);
	s = catlist(8L,GC_PUSHS(s),GC_PUSHS("\t"),cvs(cpuTime),
		GC_PUSHS(" Sec. total CPU time,\n\t"), 
		cvs(tTime),
		GC_PUSHS(" Sec. total accounted CPU time,\n\t"),
		cvf(cnf),
	" = Complexity Nesting Factor = Accounting time/CPU time.\n\n");
	s = catlist(4L,GC_PUSHS(s),
		GC_PUSHS(
	"                    TOTAL ACCT  %TOTAL  AVG TIME   NUMBER \n"),
		GC_PUSHS(
	"PROCEDURE NAME      TIME(Sec)   cpuTIME PER CALL   CALLS  \n"),
		GC_PUSHS(
	"--------------      ----------  ------- ---------  ------ \n"));

	PSprintf("%s",s);	/* dump it */

	for(i=1; i<=w_idxFree; i++)
		{ /* print entry */
		s1 = subsr(spaces,1,maxProcNameLth-length(w_proc[i]));
		s = catlist(2L,GCPUSHS(w_proc[i]),GC_PUSHS(s1));
		setformat(9L,3L);
		s = catlist(2L,GC_PUSHS(s), cvf(w_time[i]/CLKTICS) );
		setformat(7L,2L);
		s = catlist(2L,GC_PUSHS(s),
				 cvf(100.0*cnf*w_time[i]/tTime));
		setformat(12L,3L);
		s = catlist(2L, GC_PUSHS(s),
			cvf(w_time[i]/(w_count[i]*CLKTICS)));
		setformat(8L,2L);
		s = catlist(3L,GC_PUSHS(s), cvs(w_count[i]), 
			GC_PUSHS("\n"));
		PSprintf("%s",s);	/* dump it */
		} /* print entry */
} /* w_profil_print */
/* ---> END OF PSRUNW.C <--- */
