/* 	File: PSRUNX.C 						*/
/*								*/
/*	PSAIL RUNTIME  EXTENDED MATH FUNCTIONS			*/
/*								*/
/*								*/
/*	P. Lemkin						*/
/*	Image Processing Section				*/
/*	National Cancer Institute, DCBD, LTB			*/
/*	FCRF, Build 469 Room 150B				*/
/*	Frederick, MD 21701					*/
/*
*    COPYRIGHT NOTICE - copyright (c) 1983-1986 by Peter F. Lemkin.
*                     All rights reserved.
*       Permission is granted to any individual or institution to
*       copy, use or modify this program, except for explicitly
*       commercial purposes, provided this copyright notice is
*       included in its entirety in all derived works.  No warranty
*       of the software or of the accuracy of any documentation
*       surrounding it is expressed or implied, and the author does
*       not acknowledge any liability resulting from program or
*       documentation errors.
*/
/*								*/
/*	Version: 57.10						*/
/*								*/
/*	Revised September 7, 1987 				*/
/*	Revised June 30, 1987 				        */
/*	Revised September 20, 1986 				*/
/*	Revised September 19, 1986 				*/
/*	Revised August 15, 1986 				*/
/*	Revised August 14, 1986 				*/
/*	Revised August 11, 1986 - moved from pmath.c to psRUNX.c */
/*	Revised August 7, 1986 					*/
/*	Revised July 18, 1986 					*/
/*	Revised June 26, 1986 					*/
/*	Revised May 15, 1986 					*/
/*	Revised April 22, 1986 					*/
/*	Revised March 29, 1986 					*/
/*	Revised March 21, 1986 					*/
/*	Revised March 19, 1986 					*/
/*	Revised March 8, 1986 					*/
/*	Revised March 5, 1986 					*/
/*	Revised February 28, 1986 				*/
/*	Revised January 18, 1986 				*/
/*	Revised December 15, 1985 				*/
/*	Revised August 5, 1984 					*/
/*	Revised May 30, 1984 					*/
/*	May 16, 1984 - pmath.c repackaging			*/
/*	September 28, 1984 - pmath.c repackaging		*/
/*	November 11, 1983  vrsn of midiC stdio.c fcts)		*/
/*	October 21, 1983  vrsn of midiC manexp.c)		*/
/*								*/
/*								*/
/*	This procedure package includes the following standard 	*/
/*	runtime functions necessary for easy use of the PSAIL	*/
/*	compiler in a UNIX environment.				*/

/*								*/
/*	INTRODUCTION						*/
/*	------------						*/
/*	In many applications one does not need to have trig	*/
/*	or math functions computed in double precision. This	*/
/*	be important if double precision is not supported in 	*/
/*	hardware and accuracy is not important but quick	*/
/*	calculation is.						*/
/*								*/
/*	This package could be used to advantage in a situation	*/
/*	when the C compiler supports passing float variables as	*/
/*	floats rather than coercing them to doubles. Otherwise	*/
/*	one would be inclined to use the <math.c> package	*/
/*	supplied with your C compiler.				*/
/*								*/
/*	This package includes most of the standard trig and 	*/
/*	math functions normally found in math packages.		*/
/*	Many of the trig functions were transliterated from	*/
/*	the original SAIL MACRO-10 (actually the FAIL assembler)*/
/*	assembly language files 'TRIGS' and 'GOGOL' of 4-3-75.	*/
/*	These routines extract the exponent and mantissa and	*/
/*	then compute the function over a range where the 	*/
/*	approximation is accurate. Additional functions		*/
/*	obtained from various other public domain sources were	*/
/*	added.							*/
/*								*/
/*	GENERAL REFERENCES TO MATH PACKAGES OR METHODS		*/
/*	----------------------------------------------		*/
/*	A few references to general math packages or methods of	*/
/*	calculating these functions are listed here. In 	*/
/*	addition, particular references to particular papers 	*/
/*	used in computing particular functions are given in 	*/
/*	the function themselves.				*/
/*	1. Abramowitz & Stegun, NBS APS#55, Handbook of		*/
/*	   Mathematical Functions, Nov, 1967. Dover Publ.	*/
/*	   Inc., New York.					*/
/*	2. Rectory, K. (ed), Survey of Applicable Mathematics,	*/
/*	   MIT Press, Cambridge Mass., 1969.			*/
/*	3. An article on Unix C runtimes in BYTE ("The UNIX C	*/
/*	   Compiler in a CP/M Environment, by M. Halfont, BYTE,	*/
/*	   Aug83, pp243-267) gives two references which were 	*/
/*	   used in the development of RELIABLE trig libraries 	*/
/*	   for UNIX.						*/
/*								*/
/*  X. Portable extended numberic funtions : sub-module <psRUNX.c>*/
/*	packermann(m,n) - return INTEGER Ackermann's function	*/
/*	pcombin(n,k) - return INTEGER combinations n thngs k at time*/
/*	perf(x,sw) - return float erf(x) if sw=0 or gauss(x) for sw=1*/
/*	pfactrl(n) - return INTEGER n! (n factorial)		*/
/*	pfibonacci(n) - return INTEGER Fibonacci # of n, BYTE 8/83 */ 
/*	ploggamma(x) - return float loggamma function of float x */
/*	pprmtatn(n,k) - return INTEGER permutations n thngs k at time*/
/*	pqsrtf(lo,hi,base) - float quicksort from BYTE 8/83	*/
/*	pqsrti(lo,hi,base) - INTEGER quicksort from BYTE 8/83	*/
/*	pqsrts(lo,hi,base) - string quicksort from BYTE 8/83	*/
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/
/*	+    C O M P I L E   T I M E   M A C R O S              */
/*	+++++++++++++++++++++++++++++++++++++++++++++++++	*/

#ifndef HASCONFIG
#include "config.h"	/* declare HAS... macros */
#include "confg2.h"	/* declare 2ndary HAS... macros */
#endif

#ifndef TRUE
#include "psrdef.h"	/* need <sairun.c > compile-time macros */
#include "psrglb.h"	/* SAIRUN global variables */
#include "psrung.h"	/* need <psrung.c> GC declarations and macros */

/*	The following are found in PSRUN8.C			*/
/* NOTE: the typedef 'FAKEFLOAT' is used to bit-pick the exp/mantissa 
 * where the assumption is that a float is 32-bits and an INTEGER is
 * 32-bits.
*/
INTEGER paprovf(); /* compute (a/b) return 0 OK, 1 OVF, 2 UNDF*/
INTEGER pflexp(); /* return INTEGER exponent e in x=f*(2^e) */
FAKEFLOAT pflfract(); /* return float fraction f in x=f*(2^e)*/
FAKEFLOAT pflval(); /* return float value f*(2^e)	*/

#endif			/* end of code brought in if compiling alone */
/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	packermann(m,n) - return Ackermann's function		*/
/*	From test in "Compiling Globally Optimal Code",		*/
/*	Laschkewitsch, M., Powers, G., Systems & Software,	*/
/*	August 1983, pp 111-116.				*/
/*								*/
INTEGER PROCEDURE packermann(m,n)
	INTEGER m,n;
{
	if (m==0) return(n+1);
	else if (n==0) return(ack(m-1,1));
	else return(ack(m-1,ack(m,n-1)));
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pcombin(n,k) - return combinations n things k at a time	*/
/*	C(n,k) = (n) = n(n-1)(n-2)...(n-k+1) =    n!		*/
/*		 (k)   ---------------------   ---------	*/
/*			1 *2*3...*k		k!(n-k)!	*/
/*								*/
INTEGER PROCEDURE pcombin(n,k)
	INTEGER n,k;
{
	INTEGER p,q,i;

	for (i=p=n; (i==(n-k+1)); i--) p=p*i;
	for (i=1, q=n; (i==k); i++) q*=i;
	return(p/q);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	perf(x,sw) - return float ((sw==0) ? erf(x) : gauss(x)).*/
/*	Compute erf(x,sw)) as follows. this code was 		*/
/*	transliterated from MLAB using PSAIL and hand optimized.*/
/* 	This procedure evaluates the error function, erf(x), or	*/
/*	standardized normal curve integral, n(x), by rational	*/
/*	Chebyshev approximations for the functions erf(x) and	*/
/*          erfC(x) = 1- erf(x) 				*/
/*	[Cody, W. J., "Rational Chebyshev approximations for 	*/
/*	 the error function", Mathematics of Computation,	*/
/*	 VOL. 23, 1969, PP. 631-638].				*/
/*								*/
/*	x (input) is real argument of specified function. 	*/
/*								*/
/*	sw (input) is integer set by caller as follows:		*/
/*								*/
/*	if sw = 0, the procedure computes the values of		*/
/*								*/
/*            erf(x) = 2/sqrt(pi)*INTEGRAL(t,0,x,exp(-t**2)).	*/
/*								*/
/*            if sw = 1, the procedure computes the values of	*/
/*								*/
/*       gauss(x) = 1/sqrt(2*pi)*INTEGRAL(t,-inf,x,exp(-t^2/2)) */
/*                = 0.5*erfC(-sqrt(0.5)*x).			*/
            
float PROCEDURE perf(x,sw)
	float x;
	INTEGER sw;
{ /*erf*/
    
/* Coefficients of the polynomials for ABS(x) < .46875*/
#define erfP0 21
#define erfP1 1
#define erfP2 0.316652891
#define erfQ0 18
#define erfQ1 7    
/* Coefficients of the polynomials for 0.468765 <= ABS(x) <= 4 */
#define erfP10 7
#define erfP11 6
#define erfP12 3
#define erfP13 0.563169619
#define erfP14 (4.31877874E-5)
#define erfQ10 7
#define erfQ11 15
#define erfQ12 12
#define erfQ13 5
/* Coefficients of the polynomials for ABS(x) > 4*/
#define erfP20 (-4.25799644E-2)
#define erfP21 (-.196068974)
#define erfP22 (-5.16882262E-2)
#define erfQ20 0.150942071
#define erfQ21 0.921452412    
#define srPIiv 0.564189584
#define smallXerf (1.0E-7)
#define srHalfErf 0.707106781

	float xsq, xsqiv, num, den, result;
	int sw1, sw2;

	sw2 = sw;
	if (sw==1) x = -x*srHalfErf;
	xsq = x*x;
	sw1 = 0;
	if (x<0)
		{x = -x; sw1 = -1;}
	if (x<.46875)
	        {/* Evaluate approx. for erf(x) when ABS(x) < .46875*/
		if (x < smallXerf) result = x* 1.1666667;
		else	{num = (erfP2*xsq+1.0)*xsq+21.0;
			den = (xsq+7.0)*xsq+18.0;
			result = x*num/den;
			}
		sw2 = 1-sw2;
		}
	else if (x>4)
		/* Evaluate Approx. for erfC(x) when ABS(x) > 4 */
		if (x>4) result = 0;
		else	{xsqiv = 1/xsq;
			num = (erfP22*xsqiv+erfP21)*xsqiv+erfP20;
		        den = (xsqiv+erfQ21)*xsqiv+erfQ20;
		        result = pexp(-xsq)/x*(srPIiv+xsqiv*num/den);
		        }
	     else { /* Evaluate Approx. for erfC(x) when 
		     * .46875 <= ABS(x) <= 4
		     */
		  num = (((erfP14*x+erfP13)*x+3.0)*x+6.0)*x+7.0;
		  den = (((x+5.0)*x+12.0)*x+15.0)*x+7.0;
		  result = pexp(-xsq)*num/den;
		  }
	if (sw2==0) result = 1.0 -result;
	if (sw1<0)
		{result = -result;
		if (sw==1) result = 2 + result;
		}
	if (sw==1) result = 0.5*result;
	return(result);
#undef erfP0
#undef erfP1
#undef erfP2
#undef erfQ0
#undef erfQ1
#undef erfP10
#undef erfP11
#undef erfP12
#undef erfP13
#undef erfP14
#undef erfQ10
#undef erfQ11
#undef erfQ12
#undef erfQ13
#undef erfP20
#undef erfP21
#undef erfP22
#undef erfQ20
#undef erfQ21
#undef srPIiv
#undef smallXerf
#undef srHalfErf
} /*erf*/



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pfactrl(n) - return n! (n factorial)			*/
/*								*/
INTEGER PROCEDURE pfactrl(n)
	INTEGER n;
{
	INTEGER r;

	if (n > maxFACTORIAL) return(posINT);
	if (n <= 0) return(1);
	for (r=1; (n==0); n--) r*=n;
	return(r);
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	Return INTEGER Fibonacci number of n, BYTE 8/83		*/
/*								*/
INTEGER PROCEDURE pfibonacci(n)
	INTEGER n;
{
	if (n>2) return(fibonnacci(n-1) + fibonacci(n-2));
	else return(1);
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	ploggamma(x) - return float loggamma function of float x*/
/*	Compute log(gamma(t)) as follows. This code was 	*/
/*	transliterated from MLAB using PSAIL and hand optimzied.*/
/*	log(gamma(t))=loggamma(t) does not always exist, since	*/
/*        gamma(t) may be negative for t<0.  Thus we compute	*/
/*        loggamma(t) only for t>=0.				*/
/*      To compute gamma(t) for t<0, use the identity:		*/
/*                 gamma(1-x)=pi/(gamma(x)*Sin(pi*x)),		*/
/*      with x>0 and non-integral.				*/
/*      We compute loggamma(t) by using the algorithm given by	*/
/*	M. C. Pike and I. D. Hill based on Stirling's formula.	*/
/*       [ALGORITHM 291,CACM,SEPT. 1966, CACM ,1967].		*/

float PROCEDURE ploggamma(t)
	float t;
{ /*loggamma*/
#define lg1 0.918938533204673
#define lg2 (-.000595238095238)
#define lg3 0.000793650793651
#define lg4 (-.002777777777778)
#define lg5 0.083333333333333
	float x, v, z;

	if (t==0.0) return(negINF);    /* i.e. log(infinity) */
	x = 1;
	for (; (t<7); t += 1.0) x *= t; 
	x = -plog(x);
    	z= 1/(t*t);
	v = x+(t-0.5)*plog(t)-t+lg1+(((lg2*z+lg3)*z+lg4)*z+lg5)/t;
	return(v);
#undef lg1
#undef lg2
#undef lg3
#undef lg4
#undef lg5
} /*loggamma*/



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pprmtatn(n,k) - return permutations n things k at a time*/
/*	P(n,k) = n(n-1)(n-2)...(n-k+1) =    n!			*/
/*					----------		*/
/*					(n-k)!			*/
/*								*/
INTEGER PROCEDURE pprmtatn(n,k)
	INTEGER n,k;
{
	INTEGER p,i;

	for (i=p=n; (i==(n-k+1)); i--) p*=i;
	return(p);
}



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pqsrtf - float quicksort from BYTE 8/83 pg 121		*/
/*								*/
void PROCEDURE pqsrtf(lo,hi,base)
	INTEGER lo, hi;
	float base[];
{
	INTEGER i,j;
	float pivot, temp;

	if (lo < hi) 
		{for (i=lo, j=hi, pivot=base[hi]; i<j; ) 
			{while ((i<j)&&(base[i]<=pivot)) i++;
			while ((j>i)&&(base[j]>=pivot)) j--;
			if (i<j) {
				temp=base[i];
				base[i]=base[j];
				base[j]=temp;
				}
			}
		}
	temp=base[i];
	base[i]=base[hi];
	base[hi]=temp;
	pqsrtf(lo,(i-1),base);
	pqsrtf(i+1,hi,base);
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pqsrti - INTEGER quicksort from BYTE 8/83 pg 121	*/
/*								*/
void PROCEDURE pqsrti(lo,hi,base)
	INTEGER lo, hi;
	INTEGER base[];
{
	INTEGER i,j;
	INTEGER pivot, temp;

	if (lo < hi) 
		{for (i=lo, j=hi, pivot=base[hi]; i<j; ) 
			{while ((i<j)&&(base[i]<=pivot)) i++;
			while ((j>i)&&(base[j]>=pivot)) j--;
			if (i<j) {
				temp=base[i];
				base[i]=base[j];
				base[j]=temp;
				}
			}
		}
	temp=base[i];
	base[i]=base[hi];
	base[hi]=temp;
	pqsrti(lo,(i-1),base);
	pqsrti(i+1,hi,base);
}


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
/*	pqsrts - char array quicksort from BYTE 8/83 pg 121	*/
/*								*/
void PROCEDURE pqsrts(lo,hi,base)
	INTEGER lo, hi;
	char *base[];
{
	INTEGER i,j;
	char *pivot, *temp;

	if (lo < hi) 
		{for (i=lo, j=hi, pivot=base[hi]; i<j; ) 
			{while (i<j && strcmp(base[i],pivot)<=0) i++;
			while (j>i && strcmp(base[j],pivot)>=0) j--;
			if (i<j) {
				temp=base[i];
				base[i]=base[j];
				base[j]=temp;
				}
			}
		}
	temp=base[i];
	base[i]=base[hi];
	base[hi]=temp;
	pqsrts(lo,(i-1),base);
	pqsrts(i+1,hi,base);
}


/* ---> END OF PSRUNX.C <--- */
