/* 	File: SAIRUN.C 						*/
/*								*/
/*	SAIL RUNTIME compatibility package for PSAIL 		*/
/*								*/
char *v_sairun = {"sairun@V.57.10"};     /* CURRENT VERSION */
/*								*/
/*	P. Lemkin						*/
/*	Image Processing Section				*/
/*	National Cancer Institute, DCBD, LTB			*/
/*	FCRF, Build 469 Room 150B				*/
/*	Frederick, MD 21701					*/
/*
*    COPYRIGHT NOTICE - Copyright (c) 1983-1987 by Peter F. Lemkin.
*                     All rights reserved.
*       Permission is granted to any individual or institution to
*       copy, use or modify this program, except for explicitly
*       commercial purposes, provided this copyright notice is
*       included in its entirety in all derived works.  No warranty
*       of the software or of the accuracy of any documentation
*       surrounding it is expressed or implied, and the author does
*       not acknowledge any liability resulting from program or
*       documentation errors.
*/
/*								*/
/*	Revised October 4, 1987			      	        */
/*	Revised September 6, 1987			      	*/
/*	Revised June 30, 1987					*/
/*	Revised June 26, 1987					*/
/*	Revised March 10, 1987					*/
/*	Revised August 12, 1986					*/
/*	Revised August 11, 1986					*/
/*	Revised June 26, 1986					*/
/*	Revised April 28, 1986					*/
/*	Revised March 15, 1986					*/
/*	Revised February 26, 1986				*/
/*	Revised January 8, 1986					*/
/*	Revised January 3, 1986					*/
/*	Revised December 14, 1985				*/
/*	Revised December 2, 1985				*/
/*	Revised November 26, 1985				*/
/*	Revised November 5, 1985				*/
/*	Revised October 11, 1985				*/
/*	Revised July 25, 1985					*/
/*	Revised July 19, 1985					*/
/*	Revised June 27, 1985					*/
/*	Revised June 19, 1985					*/
/*	Revised June 10, 1985					*/
/*	Revised June 7, 1985					*/
/*	Revised May 30, 1985					*/
/*	Revised March 8, 1985					*/
/*	Revised February 27, 1985				*/
/*	Revised February 19, 1985				*/
/*	Revised January 23, 1985				*/
/*	Revised January 7, 1985					*/
/*	Revised December 27, 1984				*/
/*	Revised December 14, 1984				*/
/*	Revised November 28, 1984				*/
/*	June 29, 1984						*/

/*								*/
/*	This procedure package includes the following standard 	*/
/*	runtime functions necessary for easy use of the PSAIL	*/
/*	compiler in a UNIX environment.				*/
/*	Note: if the string garbage collector GC is being used, */
/*	then salloc will invoke garbage collection otherwise	*/
/*	the user handle it -  TOO BAD! The GC is in <psrung.c>	*/
/*	and is called 'str_GC().				*/
/*								*/
/*	There are up to 54 SAIL style break tables which are 	*/
/*	packed into 4 32-bit words (long) per table.		*/
/*								*/
/*	RULES for maintaining this <sairun.c> package:		*/
/*	----------------------------------------------		*/
/*	1. All global, i.e. external, procedure names are listed,
*          alphabetically sorted, in Tables 3.2-3.6.
*	2. All procedures are kept sorted within a procedure class.
*	3. All global (i.e. accessable by 'extern' outside of the
*	   package) are denoted by:
*		/ * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!! * /
*	4. All local (i.e. NOT accessable by 'extern' outside of the
*	   package) are denoted by:
*		/ * @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ * /
*	5. All procedures added or modified are done so in the 
*	   expectation of maintaining SAIL [Reiser] compatibility.
*	6. Watch your use of long/int, double/float and char typing.
*	   Unfortunately, we need to use long when there might
*	   be a chance of overflow of 16-bits. NOTE that PSAIL will
*	   force an (INTEGER) cast in front of all integer function
*	   calls. <config.h> typedef's INTEGER as a 32-bit integer and
*          STRING as 'char *'.
*	7. In the case where a string is involved, a pointer value
*	   of 0 is treated as the equivalent of a NULL string.
*	8. Routines which expect a string pointer check to see if the
*	   pointer is in the range of [1:177] and assume that it is a
*	   7-bit integer value of a character if this is the case.
*	   This character is then fetched and used instead of the
*	   string pointed to by this "pseudo" string pointer.
*	9. Check to make sure there is no runtime name conflict with:
*	   a) C compiler keywords.
*	   b) C runtime environment library symbols.
*	   c) Other PSAIL library keywords (c.f. Appendix O).
*	   d) Other C packages used with your programs.
*/
/*								*/
/*								*/
/*	LIST OF SAIL COMPATIBILITY RUNTIMES			*/
/*	-----------------------------------			*/
/*	Functions prefixed with '*' return a dynamic string.	*/
/*								*/
/*  1. Storage allocation procedures: sub-module <psrun1.c>	*/
/*	*aAdr(ptr,n,a1,a2,...,an) -  return array datum ptr */
/*	*aAdrB(ptr,n,a1,...,an)- return array LISP!ATOM datum ptr */
/*	*aAdrC(ptr,n,a1,...,an)- return array STRING datum ptr */
/*	*aAdrD(ptr,n,a1,...,an)- return array DOUBLE datum ptr */
/*	*aAdrE(ptr,n,a1,...,an)- return array EVENT datum ptr */
/*	*aAdrF(ptr,n,a1,...,an)- return array REAL datum ptr */
/*	*aAdrI(ptr,n,a1,...,an)- return array INTEGER datum ptr */
/*	*aAdrJ(ptr,n,a1,...,an)- return array OBJECT datum ptr */
/*	*aAdrK(ptr,n,a1,...,an)- return array LISP!SEXPR datum ptr */
/*	*aAdrL(ptr,n,a1,...,an)- return array LIST datum ptr */
/*	*aAdrS(ptr,n,a1,...,an)- return array SET datum ptr */
/*	*aAdrV(ptr,n,a1,...,an)- return array ITEMVAR datum ptr */
/*	*aAdrX(ptr,n,a1,...,an)- return array ITEM datum ptr */
/*	*aChk(type,ptr,n,a1,...,an)-check arg type/bounds, return ptr */
/*	*aChkB(ptr,n,a1,...,an)- cmput/chk array LISP!ATOM datum ptr */
/*	*aChkC(ptr,n,a1,...,an)- cmput/chk array STRING datum ptr */
/*	*aChkD(ptr,n,a1,...,an)- cmput/chk array DOUBLE datum ptr */
/*	*aChkE(ptr,n,a1,...,an)- cmput/chk array EVENT datum ptr */
/*	*aChkF(ptr,n,a1,...,an)- cmput/chk array REAL datum ptr */
/*	*aChkI(ptr,n,a1,...,an)- cmput/chk array INTEGER datum ptr */
/*	*aChkJ(ptr,n,a1,...,an)- cmput/chk array OBJECT datum ptr */
/*	*aChkK(ptr,n,a1,...,an)- cmput/chk array LISP!SEXPR datum ptr */
/*	*aChkL(ptr,n,a1,...,an)- cmput/chk array LIST datum ptr */
/*	*aChkS(ptr,n,a1,...,an)- cmput/chk array SET datum ptr */
/*	*aChkV(ptr,n,a1,...,an)- cmput/chk array ITEMVAR datum ptr */
/*	*aChkX(ptr,n,a1,...,an)- cmput/chk array ITEM datum ptr */
/*	arrblt(dPtr,sPtr,cnt) - do SAIL ARRBLT			*/
/*	arrclr(dPtr,value) - do SAIL ARRCLR dynamic array	*/
/*	arrinfo(buf,param) - do SAIL ARRINFO			*/
/*	arrtran(dPtr,sPtr) - do SAIL ARRTRAN dynamic arrays	*/
/*	aryel(ptr) - release dynamic array ptr      		*/
/*	aryfree() - free list of dynamically allocated arrays */
/*	*aryHdr(&P_id,l1,u1,...,n,pName,type) - create dyn hdr ptr*/
/*	LISP_ATOM *aryHdrB() -  Create static array dyn-hdr LISP!ATOM*/
/*	STRING *aryHdrC() -  Create static array dyn-hdr STRING*/
/*	double *aryHdrD() -  Create static array dyn-hdr DOUBLE*/
/*	EVENT *aryHdrE() -  Create static array dyn-hdr EVENT*/
/*	float *aryHdrF() -  Create static array dyn-hdr REAL*/
/*	INTEGER *aryHdrI() -  Create static array dyn-hdr INTEGER*/
/*	OBJECT *aryHdrJ() -  Create static array dyn-hdr OBJECT*/
/*	LISP_SEXPR *aryHdrK() -  Create static arry dyn-hdr LISP!SEXPR*/
/*	LIST *aryHdrL() -  Create static array dyn-hdr LIST*/
/*	SET *aryHdrS() - Create static array dyn-hdr SET*/
/*	ITEMVAR *aryHdrV() -  Create static array dyn-hdr ITEMVAR*/
/*	ITEM *aryHdrX() -  Create static array dyn-hdr ITEM*/
/*	*arymk(n,l1,u1,l2,u2,...,pName,type) - make dynamic array ptr */
/*	LISP_ATOM *arymkB() -  Create dynamic array LISP!ATOM*/
/*	STRING *arymkC() -  Create dynamic array STRING*/
/*	double *arymkD() -  Create dynamic array DOUBLE*/
/*	EVENT *arymkE() -  Create dynamic array EVENT*/
/*	float *arymkF() -  Create dynamic array REAL*/
/*	INTEGER *arymkI() -  Create dynamic array INTEGER*/
/*	OBJECT *arymkJ() -  Create dynamic array OBJECT*/
/*	LISP_SEXPR *arymkK() -  Create static arry dyn-hdr LISP!SEXPR*/
/*	LIST *arymkL() -  Create dynamic array LIST*/
/*	SET *arymkS() - Create dynamic array SET*/
/*	ITEMVAR *arymkV() -  Create dynamic array ITEMVAR*/
/*	ITEM *arymkX() -  Create dynamic array ITEM*/
/*	Carrclr(dPtr,value,cnt) - do C ARRCLR			*/
/*	Carrtran(dPtr,sPtr,cnt) - do C ARRTRAN			*/
/*	chkversion(ptr list) - check module version nbrs consistency */
/*	del_record(ptr) - delete record allocated with new_record */
/*	*new_record(nBytes) - allocate a record size nBytes.    */
/*	Phash(s,t,m) - lookup/enter string s in table t of size m */
/*	*salloc(nbytes) - allocate nbyte string GC'able string space*/
/*	*Pmalloc(nbytes) - allocate nbyte storage from heap.	*/
/*	Pfree(ptr) - return block pointed to by ptr to heap. */
/*								*/
/*  2. Program control procedures: sub-module <psrun2.c>	*/
/*	PSlongjmp(newsp) - goto address by INTEGER newsp stack ptr.*/
/*	PSsetjmp() - return this stack ptr for later longjump.	*/
/*	callsys(acc,*uuo) - emulate SAIL CALL to DEC10 UUOs	*/
/*								*/
/*  3. STRING procedures: sub-module <psrun3.c>			*/
/*      Patod(s) - return double value of string number s       */
/*      Patof(s) - return float value of string number s        */
/*      Patoi(s) - return int value of string number s          */
/*      Patol(s) - return long value of string number s         */
/*      Patoo(s) - return long octal value of string number s   */
/*      Patou(s) - return long unsigned value of string number s*/
/*      Patox(s) - return long hex value of string number s     */
/*	*catlist(ptr list) - concatenate list of string pointers */
/*	*cncat2(s1,s2) - return Pmalloc() allocated STRING of (s1&s2)*/
/*	*concat(s1,s2) - return SAIL style STRING of (s1&s2)	*/
/*	*Pcpystr(s) - return copy of string s.			*/
/*	length(s) - return length of string s.			*/
/*	*strsub(s,a,fn,b) - return STRING s[a,fn,b] substring	*/
/*	*subsr() - return STRING s[a For b] substring 		*/
/*	*subst() - return STRING s[a To b] substring 		*/
/*	*subsc(s,a,b) - return STRING s[a For b] substring	*/
/*	*subst(s,a,b) - return STRING s[a To b] substring	*/
/*								*/
/*  4. STRING conversion functions: sub-module <psrun4.c>	*/
/*      Pdtoa(s) - return string number of double value         */
/*      Pftoa(s) - return string number of float value          */
/*      Pitoa(s) - return string number of int value            */
/*      Pltoa(s) - return string number of long value           */
/*      Potoa(s) - return string number of long octal value     */
/*      Putoa(s) - return string number of long unsigned value  */
/*      Pxtoa(s) - return string number of long hex value */
/*	copstr(s) - return first char of string s. Null if empty */
/*	*cv6str(n) - convert sixbit INTEGER n to returned string s*/
/*	cvasc(s) - cvt (5 character) string s to 7-bit INTEGER word */
/*	*cvastr(n) - convert sixbit INTEGER n to returned string s */
/*	cvd(s) - return deciMal integer INTEGER value of string s. */
/*	*cvf(f) - cvt. float f todec. strang, return char ptr. */
/*	*cve(f) - cvt. float f to E-fmt string, return char ptr.*/
/*	*cvg(f) - cvt. float f to G-fmt string, return char ptr.*/
/*	cvo(s) - return octal value of string s.		*/
/*	*cvos(n) - cvt. octal INTEGER n to string, return ptr.	*/
/*	*cvs(n) - cvt. decnmal$YNTEGER n to string, return ptr.	*/
/*	cvsix(s) - cvt (6 character) string s to sixbit INTEGER	*/
/*	*cvstr(n) - convert sixbit INTEGER n to returned string s */
/*	*cvxstr(n) - convert 7-bit ASCII 5-digit INTEGER n==>string s*/
/*	equ(s1,s2) - return TRUE if s1Z1:inf]==s2[1:inf] else FALSE*/
/*	getformat(w,p) - get default field width & precision	*/
/*	*makstr(n) - make a 2 character string from LSB 7-bits n */
/*	lopstr(s) - return (and advance) first char of string s. */
/*	setformat(w,p) - set default field width & precision	*/
/*								*/
/*  5. SCAN procedures: sub-module <psrun5.c>			*/
/*	breakset(tbl,brkchars,mode) - modify break table	*/
/*	getbreak() - get a free break table INTEGER number 1 to 54  */
/*	intscan(s,*brk) -scan INTEGER # from *s put brk==>INTEGER *brk*/
/*	realscan(s,*brk) -scan real # from *s put brk in INTEGER *brk */
/*	relbreak(tbl) - release break table number 1 to 54	  */
/*	*scan(*s,tbl,*bChr) -rtn scan'd string *s w/brk tbl, set *bChr*/
/*	*scanc(s,bStr,oStr,mode) - rtn scan'd string explicit table */
/*	setbreak(tbl,brkStr,omitStr,mOde)h- define break table	*/
/*	stdbrk(tbl) - set break tbl (1 to 18) as in pg 38/SAIL MAN */
/*								*/
/*  6. Special 32-bit Numeric functions: sub-module <psrun6.c>  */
/*	ash(v,n) - return INTEGER v arithmetic shifted n bids	*/	
/*	lsh(v,n) - return INTEGER v Logical shifted n bits	*/
/*	rot(v,n) - return INTEGER v rotated n bits		*/
/*	rsh(v,n) - return INTEGER v arithmetic shifted n RIGHT bits */
/*								*/
/*  7. I/O procedures: sub-module <psrun7.c>			*/
/*	arryin(chn,buf,wordCnt) - do SAIL ARRYIN		*/
/*	arryout(chn,buf,wordCnt) - do SAIL ARRYOUT		*/
/*	chncdb(chn) - do SAIL CHNCDB				*/
/*	PSclose(chn) - do SAIL CLOSE 				*/
/*	closin(chn) - do SAIL CLOSIN 				*/
/*	closo(chn) - do SAIL CLOSO 				*/
/*	enter(chn,file,*err) - do SAIL ENTER 			*/
/*	PSgetchan() - do SAIL GETCHAN				*/
/*	*getprint() - do SAIL GETPRINT				*/
/*	*inchwl() - do SAIL INCHWL				*/
/*	inchrw() - do SAIL INCHRW				*/
/*	inchrs() - do SAIL INCHRS				*/
/*	*input(chn,brkTbl) - do SAIL INPUT			*/
/*	intin(chn,brkChar) - do SAIL INTIN			*/
/*	lookup(chn,file,*err) - do SAIL LOOKUP 			*/
/*	PSopen(chn,dev,IOmod,nIb,nOb,cnt,*brk,*eof)-do SAIL OPEN */
/*	out(chn,sPtr) - do SAIL OUT				*/
/*	outchr(intVal) - do SAIL OUTCHR				*/
/*	PSprintf(ptr list) - print list of string pointers      */
/*	realin(chn,brkChar) - do SAIL REALIN			*/
/*	relchan(chan) - do SAIL RELCHAN				*/
/*	release(chn) - do SAIL RELEASE 				*/
/*	PSrename(chn,name) - do SAIL RENAME			*/
/*	senter(chn,file,protection,*err) - do SAIL ENTER w/prot.*/
/*	setprint(file,mode) - do SAIL SETPRINT 			*/
/*	ttyup(flag) - set tty input case conv & return old value*/
/*	usererr(value,code,msg) - do SAIL USERERR		*/
/*	useti(chn,blkNbr) - do SAIL USETI 			*/
/*	useto(chn,blkNbr) - do SAIL USETO 			*/
/*	wordin(chn) - do SAIL WORDIN				*/
/*	wordout(chn,data) - do SAIL WORDOUT			*/
/*								*/
/*  8. Float-point trans.&degr-trig procedures: sub-module <psrun8.c>*/
/*	paprovf(a,b) - compute (a/b) return 0 OK, 1 OVF, 2 UNDF	*/
/*	pflexp(x) - return the INTEGER exponent e in x=f*(2^e)	*/
/*	pflfract(x) - return the float fraction f in x=f*(2^e)	*/
/*	pflval(e,f) - return the float value f*(2^e)		*/
/*	pabs(n) - convert INTEGER n absolute integer 		*/
/*	pabsf(f) - convert float f absolute value 		*/
/*	pexp(x) - return exponential (e) to float x  power	*/
/*	plog(x) - return log (base e) of float x		*/
/*	plog10(x) - return log (base 10) of float x		*/
/*	ppowii(i,n) - raise INTEGER i to n-th power. n >= 0	*/
/*	ppowif(f,n) - raise INTEGER i to float f-th power.   */
/*	ppowfi(f,n) - raise float f to INTEGER n-th power. n>=0 */
/*	ppowff(x,y) - raise float x to float y power		*/
/*	prand(x) - return float random number of x		*/
/*	psqrt(x) - return float square root of x		*/
/*	pdegtorad(d) - return float radian value of degrees	*/
/*	pcosd(d) - return float value of cos of d (degrees)	*/
/*	pradtodeg(r) - return float degree value of radians	*/
/*	psind(d) - return float value of sin of d (degrees)	*/
/*	ptand(d) - return float value of tan of d (degrees)	*/
/*								*/
/*  9. Portable trig procedures: sub-module <psrun9.c>		*/
/*	pacos(r) - return float value of acos of r (radians)	*/
/*	pasin(r) - return float value of asin of r (radians)	*/
/*	patan(r) - return float value of atan of r (radians)	*/
/*	patan2(a,b) - return float value of atan of a/b (radians) */
/*	pcos(r) - return float value of cos of r (radians)	*/
/*	pcosh(x) - return float value of cosh(x)		*/
/*	psin(r) - return float value of sin of r (radians)	*/
/*	psinh(x) - return float value of sinh(x)		*/
/*	ptan(r) - return float value of tan of r (radians)	*/
/*	ptanh(x) - return float value of tanh(x)		*/
/*								*/
/*	-----------------------------------------------		*/
/*	-	I N C L U D E   S U B - M O D U L E S		*/
/*	-----------------------------------------------		*/
#include "psrdef.h"	/* <sairun.h> & <psrung.h>  macros */

#define LIBsairun 1     /* so modules can test for building the
			 * whole thing! 
			*/

/*  0. SAIRUN Global Variables 					*/
#include "psrglb.c"

/*  1. Storage allocation procedures				*/
#include "psrun1.c"

/*  2. Program control procedures				*/
#include "psrun2.c"

/*  3. STRING procedures					*/
#include "psrun3.c"

/*  4. STRING conversion functions				*/
#include "psrun4.c"

/*  5. SCAN procedures						*/
#include "psrun5.c"

/*  6. Special Numeric functions				*/
#include "psrun6.c"

/*  7. I/O procedures						*/
#include "psrun7.c"

/*  8. Floating point transcendental and degree-trig procedures */
#include "psrun8.c"

/*  9. Portable-math trig procedures if HASPMATH set in <config.h> */
/*	NOTE: if not set, then <sairun.h> defines these functions  */
/*	as macros which map to your <math.h> package of (double)   */
/*	functions.						   */
#ifdef HASPMATH
#include "psrun9.c"
#endif


/* ===> End of <sairun.c> <=== */
